      SUBROUTINE ARBALL
C
C  THIS SUBROUTINE INTERPOLATES AND(OR) EXTRAPOLATES CROSS-
C  SECTIONAL AREA AND TOP-WIDTH PROPERTIES FROM TABULAR FUNCTIONS.
C  IF WETTED PERIMETER IS NOT INPUT, THE HYDRAULIC RADIUS IS 
C  APPROXIMATED BY THE HYDRAULIC DEPTH (AREA/WIDTH).
C
      INTEGER i, j, k, ij, nd, ns, km1, ijp1, ijpns, iret
      REAL zpij, r0
      INCLUDE 'dimens.cmn'
      INCLUDE 'arblog.cmn'
      INCLUDE 'dtcomp.cmn'
      INCLUDE 'comcon.cmn'
      INCLUDE 'dtypes.cmn'
      INCLUDE 'geom.cmn'
      INCLUDE 'branch.cmn'
      INCLUDE 'lunums.cmn'
      INCLUDE 'datime.cmn'
      EXTERNAL XSTRAP, ERRSCR
      CHARACTER*64 msg
      DATA msg/'INITIAL OR COMPUTED STAGE YIELDS ZERO OR NEGATIVE AREA O
     &R WIDTH '/
C
      DO 70 i = 1, Nbch
        ij = MAXS - Xskt(i)
        ns = Nsec(i)
        DO 60 j = 1, ns
          ij = ij + 1
          nd = Ipt(ij)
          k = Indxs(ij)
          IF (j.NE.1) THEN
            zpij = Zp(ij-1)
C
C---indxs must be initialized to 2 in bricxs routine
            IF (zpij.LT.Za(k-1,ij) .OR. zpij.GT.Za(k,ij)) THEN
C
C---CALL EXTRAPOLATION ROUTINE IF STAGE OUTSIDE OF GEOMETRY TABLE
              IF (zpij.LT.Za(1,ij) .OR. zpij.GT.Za(nd,ij)) THEN
                CALL XSTRAP(zpij,i,j,M,ij,nd,iret)
                IF (iret.EQ.0) GOTO 30
                WRITE (Printr,9005)
                GOTO 80
              ENDIF
C
C---LINEAR INTERPOLATE IF STAGE WITHIN DEFINED RANGE OF GEOMETRY TABLE
              IF (zpij.GT.Za(k,ij)) THEN
   10           k = k + 1
                IF (zpij.GT.Za(k,ij)) GOTO 10
              ELSE
   20           k = k - 1
                IF (zpij.LT.Za(k-1,ij)) GOTO 20
              ENDIF
              Indxs(ij) = k
            ENDIF
            r0 = (Za(k,ij)-zpij)/(Za(k,ij)-Za(k-1,ij))
            Apzpij(ij) = Aa(k,ij) - r0*(Aa(k,ij)-Aa(k-1,ij))
            IF (Apzpij(ij).EQ.0.0) CALL ERRSCR(1,i,j,msg)
          ENDIF
C
   30     zpij = Zp(ij)
          IF (zpij.LT.Za(k-1,ij) .OR. zpij.GT.Za(k,ij)) THEN
C
            IF (zpij.LT.Za(1,ij) .OR. zpij.GT.Za(nd,ij)) THEN
              CALL XSTRAP(zpij,i,j,M,ij,nd,iret)
              IF (iret.EQ.0) GOTO 60
              GOTO 80
            ENDIF
C
C---LINEAR INTERPOLATE IF STAGE WITHIN DEFINED RANGE OF GEOMETRY TABLE
            IF (zpij.GT.Za(k,ij)) THEN
   40         k = k + 1
              IF (zpij.GT.Za(k,ij)) GOTO 40
            ELSE
   50         k = k - 1
              IF (zpij.LT.Za(k-1,ij)) GOTO 50
            ENDIF
            Indxs(ij) = k
          ENDIF
          km1 = k - 1
          r0 = (Za(k,ij)-zpij)/(Za(k,ij)-Za(km1,ij))
          Ap(ij) = Aa(k,ij) - r0*(Aa(k,ij)-Aa(km1,ij))
          Bp(ij) = Bb(k,ij) - r0*(Bb(k,ij)-Bb(km1,ij))
          Btp(ij) = Bs(k,ij) - r0*(Bs(k,ij)-Bs(km1,ij)) + Bp(ij)
          IF (Ap(ij).EQ.0.0 .OR. Bp(ij).EQ.0.0) CALL ERRSCR(1,i,j,msg)
          IF (Rrflg(ij)) THEN
            Rp(ij) = Ap(ij)/(Wp(k,ij) - r0*(Wp(k,ij)-Wp(km1,ij)))
            Hyrad(ij) = Rp(ij)
          ELSE
            Rp(ij) = Ap(ij)/Bp(ij)
          ENDIF
C
C Begin code to compute hydraulic radius of rectangular channel
C for use in friction term evaluation
C         Rp(ij) = Ap(ij)/(Bp(ij)+2.0*(zpij-Za(1,ij)+0.01))
C         Hyrad(ij) = Rp(ij)
C End code to compute hydraulic radius of rectangular channel
C for use in friction term evaluation
C
          IF (Linprt(ij)) THEN
            WRITE (Printr,9020) i, j
            WRITE (*,9020) i, j
            Linprt(ij) = .FALSE.
          ENDIF
   60   CONTINUE
   70 CONTINUE
      RETURN
   80 DO 100 k = 1, 2
        WRITE (Printr,9010) Ztype
        DO 90 i = 1, Nbch
          ij = MAXS - Xskt(i)
          ijp1 = ij + 1
          ijpns = ij + Nsec(i)
          IF (k.EQ.1) THEN
            WRITE (Printr,9015) Ztype, i, (Z(ij)+Zdatum,ij=ijp1,ijpns)
          ELSE
            WRITE (Printr,9015) Ztype, i, (Zp(ij)+Zdatum,ij=ijp1,ijpns)
          ENDIF
   90   CONTINUE
        Ztype = Zptype
  100 CONTINUE
      STOP
 9005 FORMAT (
     &        ' ADDITIONAL AREA [AA(ZA)] NEEDED TO EVALUATE dA/dX TERM')
 9010 FORMAT (/,12X,A2,'(J,I),J=1,NSEC(I)')
 9015 FORMAT (2X,A2,'( J,',I2,')',12F10.4/11X,12F10.4)
 9020 FORMAT (/,
     &' *WARNING* COMPUTED STAGE IS NOW BACK WITHIN DEFINED RANGE OF GEO
     &METRY TABLE',/,11X,'BRANCH',I3,' SECTION',I3,
     &', AREA & WIDTH INTERPOLATED.')
      END
C
      SUBROUTINE XSTRAP(ZPIJ,I,J,M,IJ,ND,IRET)
C
C  THIS SUBROUTINE INTERPOLATES AND(OR) EXTRAPOLATES CROSS-
C  SECTIONAL AREA AND TOP-WIDTH PROPERTIES FROM TABULAR FUNCTIONS
C  WHEN STAGE IS OUT OF GEOMETRY TABLE.
C
      LOGICAL zl, zh
      INTEGER I, J, M, IJ, ND, IRET, n1, n2, n3, initfl, lastm, ipct, k
      REAL ZPIJ, z0, z2, r0, z1, r1, r2
      INCLUDE 'dimens.cmn'
      INCLUDE 'arblog.cmn'
      INCLUDE 'geom.cmn'
      INCLUDE 'lunums.cmn'
      INCLUDE 'dtcomp.cmn'
      INTEGER IFIX
      INTRINSIC IFIX
      EXTERNAL ERRSCR
      SAVE initfl, lastm
      DATA initfl/1/, lastm/0/
C
C---INITIALIZE OUT OF GEOMETRY RANGE ARRAYS
      IF (initfl.EQ.1) THEN
        initfl = 2
        DO 10 k = 1, MAXS
          Zhigh(k) = .FALSE.
          Zlow(k) = .FALSE.
          Larbpr(k) = .TRUE.
   10   CONTINUE
      ENDIF
C
      IRET = 0
      zl = Zlow(IJ)
      zh = Zhigh(IJ)
      IF (M.NE.lastm) THEN
        IF (zl .OR. zh) Linprt(IJ) = .TRUE.
        lastm = M
        IF (.NOT.Larbpr(IJ)) THEN
          Larbpr(IJ) = .TRUE.
          IF (zl) WRITE (Printr,9005) 'LOW', I, J
          IF (zh) WRITE (Printr,9005) 'UPP', I, J
        ENDIF
      ENDIF
      Zlow(IJ) = ZPIJ.LT.Za(1,IJ)
      Zhigh(IJ) = ZPIJ.GT.Za(ND,IJ)
C
C---EXTRAPOLATION OF CROSS-SECTIONAL PROPERTIES FOR COMPUTED STAGE OUT
C---OF DEFINED RANGE OF THE GEOMETRY TABLE.
      IF (Zlow(IJ) .OR. Zhigh(IJ)) THEN
        IF (.NOT.(zl.OR.zh)) Larbpr(IJ) = .FALSE.
        z0 = Za(1,IJ)
        z2 = Za(ND,IJ)
        IF (Zlow(IJ)) THEN
          Indxs(IJ) = 2
          ipct = IFIX((z0-ZPIJ)/(z2-z0)*100.0)
        ELSE
          Indxs(IJ) = ND
          ipct = IFIX((ZPIJ-z2)/(z2-z0)*100.0)
        ENDIF
        IF (ipct.GT.19) THEN
          CALL ERRSCR(0,i,j,
     &        'COMPUTED STAGE EXCEEDS RANGE OF CHANNEL GEOMETRY BY 20%')
          IRET = 1
C
C---LINEAR EXTRAPOLATION FOR COMPUTED STAGE OUT OF DEFINED RANGE OF
C---CROSS-SECTIONAL GEOMETRY TABLE, WHEN TABLE CONSISTS OF TWO POINTS.
        ELSEIF (ND.EQ.2) THEN
          r0 = (ZPIJ-z0)/(z0-Za(2,IJ))
          Ap(IJ) = Aa(1,IJ) + r0*(Aa(1,IJ)-Aa(2,IJ))
          Apzpij(IJ) = Ap(IJ)
          Bp(IJ) = Bb(1,IJ) + r0*(Bb(1,IJ)-Bb(2,IJ))
          Btp(IJ) = Bs(1,IJ) + r0*(Bs(1,IJ)-Bs(2,IJ)) + Bp(IJ)
          IF (Rrflg(IJ)) THEN
            Rp(IJ) = Ap(IJ)/(Wp(1,IJ) + r0*(Wp(1,IJ)-Wp(2,IJ)))
            Hyrad(ij) = Rp(ij)
          ELSE
            Rp(IJ) = Ap(IJ)/Bp(IJ)
          ENDIF
        ELSE
C
C---QUADRATIC EXTRAPOLATION FOR COMPUTED STAGE OUT OF DEFINED RANGE OF
C---CROSS-SECTIONAL GEOMETRY TABLE USING FIRST THREE POINTS OF TABLE.
          IF (Zlow(IJ)) THEN
            z1 = Za(2,IJ)
            z2 = Za(3,IJ)
            n1 = 1
            n2 = 2
            n3 = 3
C
C---QUADRATIC EXTRAPOLATION FOR COMPUTED STAGE OUT OF DEFINED RANGE OF
C---CROSS-SECTIONAL GEOMETRY TABLE USING LAST THREE POINTS OF TABLE.
          ELSE
            n3 = ND
            n2 = ND - 1
            n1 = ND - 2
            z0 = Za(n1,IJ)
            z1 = Za(n2,IJ)
          ENDIF
          r0 = ((ZPIJ-z1)*(ZPIJ-z2))/((z0-z1)*(z0-z2))
          r1 = ((ZPIJ-z0)*(ZPIJ-z2))/((z1-z0)*(z1-z2))
          r2 = ((ZPIJ-z0)*(ZPIJ-z1))/((z2-z0)*(z2-z1))
          Ap(IJ) = r0*Aa(n1,IJ) + r1*Aa(n2,IJ) + r2*Aa(n3,IJ)
          Apzpij(IJ) = Ap(IJ)
          Bp(IJ) = r0*Bb(n1,IJ) + r1*Bb(n2,IJ) + r2*Bb(n3,IJ)
          Btp(IJ) = r0*Bs(n1,IJ) + r1*Bs(n2,IJ) + r2*Bs(n3,IJ) + Bp(IJ)
          IF (Rrflg(IJ)) THEN
            Rp(IJ) = Ap(IJ)/(r0*Wp(n1,IJ) + r1*Wp(n2,IJ) + r2*Wp(n3,IJ))
            Hyrad(ij) = Rp(ij)
          ELSE
            Rp(IJ) = Ap(IJ)/Bp(IJ)
          ENDIF
        ENDIF
      ENDIF
      RETURN
 9005 FORMAT (' *WARNING* COMPUTED STAGE IS NOW OUT OF BOUNDS ON ',A3,
     &        'ER END OF GEOMETRY TABLE, BRANCH',I3,' SECTION',I3,
     &        ', AREA & WIDTH EXTRAPOLATED.')
      END
