C
C     c2akm.f
C
      SUBROUTINE C2AKM(NDATA,XDATA,FDATA,BREAK,CSCOEF,IPVT)
C
C     + + + PURPOSE + + +
C     Compute the Akima cubic spline interpolant
C     Usage:  CALL C2AKM (NDATA, XDATA, FDATA, BREAK, CSCOEF, IPVT)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NDATA, IPVT(*)
      REAL XDATA(*), FDATA(*), BREAK(*), CSCOEF(4,*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NDATA  - Number of data points.  (Input)
C     XDATA  - Array of length NDATA containing the data point
C              abscissas.  (Input)
C              The data point abscissas must be distinct.
C     FDATA  - Array of length NDATA containing the data point
C              ordinates.  (Input)
C     BREAK  - Array of length NDATA containing the breakpoints for the
C              piecewise cubic representation.  (Output)
C     CSCOEF - Matrix of size 4 by NDATA containing the local
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i
      REAL b, rm1, rm2, rm3, rm4, t1, t2
C
C     + + + EXTERNALS + + +
      EXTERNAL C1SOR
C
C     + + + INTRINSICS + + +
      REAL ABS
      INTRINSIC ABS
C
C     + + + END SPECIFICATIONS + + +
C
C                                  CHECK ARGUMENT NDATA
      IF (NDATA.LT.4) THEN
        WRITE (*,9005) NDATA
C
        STOP
      ENDIF
C                                  COPY AND SORT INPUT DATA
      CALL C1SOR(NDATA,XDATA,FDATA,BREAK,CSCOEF,4,IPVT)
C
      rm3 = (CSCOEF(1,2)-CSCOEF(1,1))/(BREAK(2)-BREAK(1))
      t1 = rm3 - (CSCOEF(1,2)-CSCOEF(1,3))/(BREAK(2)-BREAK(3))
      rm2 = rm3 + t1
      rm1 = rm2 + t1
C                                  NOW GET THE SLOPES
      DO 10 i = 1, NDATA
        IF (i.LT.NDATA-1) THEN
          rm4 = (CSCOEF(1,i+2)-CSCOEF(1,i+1))/(BREAK(i+2)-BREAK(i+1))
        ELSE
          rm4 = rm3 - rm2 + rm3
        ENDIF
        t1 = ABS(rm4-rm3)
        t2 = ABS(rm2-rm1)
        b = t1 + t2
        IF (b.EQ.0.0) THEN
C                                  IF DENOMINATOR IS ZERO, GET AVERAGE
          CSCOEF(2,i) = 0.5*(rm2+rm3)
        ELSE
          CSCOEF(2,i) = (t1*rm2+t2*rm3)/b
        ENDIF
        rm1 = rm2
        rm2 = rm3
        rm3 = rm4
   10 CONTINUE
C                                  COMPUTE THE COEFFICIENTS FOR THE
C                                    NDATA-1 INTERVALS
      DO 20 i = 1, NDATA - 1
        t1 = 1.0/(BREAK(i+1)-BREAK(i))
        t2 = (CSCOEF(1,i+1)-CSCOEF(1,i))*t1
        b = (CSCOEF(2,i)+CSCOEF(2,i+1)-t2-t2)*t1
        CSCOEF(4,i) = 6.0*b*t1
        CSCOEF(3,i) = 2.0*(-b+(t2-CSCOEF(2,i))*t1)
   20 CONTINUE
      RETURN
C
 9005 FORMAT (/,' **Error** in C2AKM:',/,
     &      ' The number of data points must be 4 or more while NDATA ='
     &      ,I3,' is given.')
      END
C
C     csakm.f
C
      SUBROUTINE CSAKM(NDATA,XDATA,FDATA,BREAK,CSCOEF)
C
C     + + + PURPOSE + + +
C     Compute the Akima cubic spline interpolation
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NDATA
      REAL XDATA(*), FDATA(*), BREAK(*), CSCOEF(4,*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NDATA  - Number of data points.  (Input)
C     XDATA  - Array of length NDATA containing the data point
C              abscissas.  (Input)
C              The data point abscissas must be distinct.
C     FDATA  - Array of length NDATA containing the data point
C              ordinates.  (Input)
C     BREAK  - Array of length NDATA containing the breakpoints for the
C              piecewise cubic representation.  (Output)
C     CSCOEF - Matrix of size 4 by NDATA containing the local
C     LEN    - length of character string
C     STRING - character string to be made upper case
C              coefficients of the cubic pieces.  (Output)
C
C  Remarks:
C  1. Automatic workspace usage is
C              CSAKM    NDATA units
C     Workspace may be explicitly provided, if desired, by use of
C     C2AKM.  The reference is
C              CALL C2AKM (NDATA, XDATA, FDATA, BREAK, CSCOEF, IWK)
C     The additional argument is
C     IWK    - Work array of length NDATA.
C
C  2. The cubic spline can be evaluated using CSVAL; its derivative can
C     be evaluated using CSDER.
C
C  3. Note that column NDATA of CSCOEF is used as workspace.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER indata
C
C     + + + COMMON SPECIFICATIONS + + +
      INCLUDE 'cubspl.inc'
C
C     + + + FUNCTIONS + + +
      INTEGER I1KGT
      EXTERNAL I1KGT
C
C     + + + EXTERNALS + + +
      EXTERNAL C2AKM
C
C     + + + END SPECIFICATIONS + + +
C
C---Check argument NDATA
      IF (NDATA.LT.4) THEN
        WRITE (*,9005) NDATA
        STOP
      ENDIF
      indata = I1KGT(NDATA,2)
C
      CALL C2AKM(NDATA,XDATA,FDATA,BREAK,CSCOEF,Iwksp(indata))
C
      RETURN
C
 9005 FORMAT (/,' **Error** in CSAKM:',/,
     &      ' The number of data points must be 4 or more while NDATA ='
     &      ,I3,' is given.')
      END
C
C     c1sor.f
C
      SUBROUTINE C1SOR(NDATA,XDATA,FDATA,BREAK,CSCOEF,LDC,IWORK)
C
C     + + + PURPOSE + + +
C     Copy and sort XDATA into BREAK and FDATA into the first row of CSCOEF.
C     Usage:  CALL C1SOR (NDATA, XDATA, FDATA, BREAK, CSCOEF, LDC, IWORK)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NDATA, LDC, IWORK(*)
      REAL XDATA(*), FDATA(*), BREAK(*), CSCOEF(LDC,*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NDATA  - Number of data points (must be at least 2).  (Input)
C     XDATA  - Array of length NDATA containing the data point
C              abscissas.  (Input)
C     FDATA  - Array of length NDATA containing the data point
C              ordinates.  (Input)
C     BREAK  - Array of length NDATA containing the breakpoints
C              for the piecewise cubic representation.  (Output)
C     CSCOEF - Matrix of size 4 by NDATA containing the cubic spline
C              (piecewise polynomial) representation in the first
C              NDATA-1 columns.  (Output)
C     LDC    - Leading dimension of CSCOEF exactly as specified in the
C              dimension statement of the calling program.  (Input)
C     IWORK  - Work array containing the permutation vector.  (Output)
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, j
C
C     + + + EXTERNALS + + +
      EXTERNAL SCOPY, SVRGP
C
C     + + + END SPECIFICATIONS + + +
C
C                                  CHECK FOR SORTED ARRAY
      DO 40 i = 2, NDATA
        IF (XDATA(i-1).GE.XDATA(i)) THEN
C                                  CHECK THAT XDATA VALUES ARE DISTINCT
          IF (XDATA(i-1).EQ.XDATA(i)) THEN
            WRITE (*,9005) i, i - 1, XDATA(i)
            STOP
          ENDIF
C                                  SET INITIAL PERMUTATION
          DO 10 j = 1, NDATA
            IWORK(j) = j
   10     CONTINUE
C                                  FIND SORTING PERMUTATION
          CALL SVRGP(NDATA,XDATA,BREAK,IWORK)
C                                  APPLY PERMUTATION
          DO 20 j = 1, NDATA
            CSCOEF(1,j) = FDATA(IWORK(j))
   20     CONTINUE
C                                  CHECK THE XDATA VALUES ARE DISTINCT
          DO 30 j = 2, NDATA
            IF (BREAK(j-1).EQ.BREAK(j)) THEN
              WRITE (*,9005) IWORK(j-1), IWORK(j), BREAK(j)
              STOP
            ENDIF
   30     CONTINUE
          RETURN
        ENDIF
   40 CONTINUE
C                                  DATA IS ALREADY SORTED.  M1VE TO
C                                  BREAK AND CSCOEF.
      CALL SCOPY(NDATA,XDATA,1,BREAK,1)
      CALL SCOPY(NDATA,FDATA,1,CSCOEF,LDC)
C
      RETURN
C
 9005 FORMAT (/,' **Error** in C1SOR:',/,
     &       ' Points in the data point abscissas array, XDATA, must be'
     &       ,' distinct,',/,' but XDATA(',I5,') = XDATA(',I5,') =',
     &       F10.4)
      END
C
      SUBROUTINE SVRGP(N,RA,RB,IPERM)
C
C     + + + PURPOSE + + +
C     Sort a real array by algebraic values and return the permutations.
C     Usage:  CALL SVRGP (N, RA, RB, IPERM)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER N, IPERM(*)
      REAL RA(*), RB(*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C
C     N      - Number of elements in the array to be sorted.  (Input)
C     RA     - Vector of length N containing the array to be sorted.
C              (Input)
C     RB     - Vector of length N containing the sorted array.  (Output)
C              If RA is not needed, RA and RB can share the same
C              storage locations.
C     IPERM  - Vector of length N.  (Input/Output)
C              On input IPERM should be initialized to the values
C              1, 2, ... N.  On output IPERM contains a record of
C              permutations made on the vector RA.
C
C  Remark:
C     For wider applicability, integers (1, 2, ... N) that are to be
C     associated with RA(I) for I = 1, 2, ... N may be entered into
C     IPERM(I) in any order.  Note that these integers must be unique.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, ij, il(21), it, itt, iu(21), j, k, l, m
      REAL r, t, tt
C
C     + + + EXTERNALS + + +
      EXTERNAL SCOPY
C
C     + + + INTRINSICS + + +
      INTEGER INT
      REAL FLOAT
      INTRINSIC INT, FLOAT
C
C     + + + END SPECIFICATIONS + + +
C
      CALL SCOPY(N,RA,1,RB,1)
C                                  CHECK FOR INPUT ERRORS
      IF (N.LE.0) THEN
        WRITE (*,9005) N
        STOP
      ENDIF
C
      m = 1
      i = 1
      j = N
      r = .375
   10 IF (i.EQ.j) GOTO 50
      IF (r.LE..5898437) THEN
        r = r + 3.90625E-2
      ELSE
        r = r - .21875
      ENDIF
   20 k = i
C                                  SELECT A CENTRAL ELEMENT OF THE
C                                  ARRAY AND SAVE IT IN LOCATION T
      ij = i + INT(FLOAT(j-i)*r+0.5)
      t = RB(ij)
      it = IPERM(ij)
C                                  IF FIRST ELEMENT OF ARRAY IS GREATER
C                                  THAN T, INTERCHANGE WITH T
      IF (RB(i).GT.t) THEN
        RB(ij) = RB(i)
        RB(i) = t
        t = RB(ij)
        IPERM(ij) = IPERM(i)
        IPERM(i) = it
        it = IPERM(ij)
      ENDIF
      l = j
C                                  IF LAST ELEMENT OF ARRAY IS LESS THAN
C                                  T, INTERCHANGE WITH T
      IF (RB(j).LT.t) THEN
        RB(ij) = RB(j)
        RB(j) = t
        t = RB(ij)
        IPERM(ij) = IPERM(j)
        IPERM(j) = it
        it = IPERM(ij)
C                                  IF FIRST ELEMENT OF ARRAY IS GREATER
C                                  THAN T, INTERCHANGE WITH T
        IF (RB(i).GT.t) THEN
          RB(ij) = RB(i)
          RB(i) = t
          t = RB(ij)
          IPERM(ij) = IPERM(i)
          IPERM(i) = it
          it = IPERM(ij)
        ENDIF
      ENDIF
C                                  FIND AN ELEMENT IN THE SECOND HALF OF
C                                  THE ARRAY WHICH IS SMALLER THAN T
   30 l = l - 1
      IF (RB(l).GT.t) GOTO 30
C                                  FIND AN ELEMENT IN THE FIRST HALF OF
C                                  THE ARRAY WHICH IS GREATER THAN T
   40 k = k + 1
      IF (RB(k).LT.t) GOTO 40
C                                  INTERCHANGE THESE ELEMENTS
      IF (k.LE.l) THEN
        IF (RB(l).NE.RB(k)) THEN
          tt = RB(l)
          RB(l) = RB(k)
          RB(k) = tt
          itt = IPERM(l)
          IPERM(l) = IPERM(k)
          IPERM(k) = itt
        ENDIF
        GOTO 30
      ELSE
C                                  SAVE UPPER AND LOWER SUBSCRIPTS OF
C                                  THE ARRAY YET TO BE SORTED
        IF (l-i.LE.j-k) THEN
          il(m) = k
          iu(m) = j
          j = l
          m = m + 1
        ELSE
          il(m) = i
          iu(m) = l
          i = k
          m = m + 1
        ENDIF
        GOTO 60
      ENDIF
C                                  BEGIN AGAIN ON ANOTHER PORTION OF
C                                  THE UNSORTED ARRAY
   50 m = m - 1
C                                  M = 0 MEANS DONE
      IF (m.EQ.0) RETURN
C
      i = il(m)
      j = iu(m)
   60 IF (j-i.GE.11) GOTO 20
      IF (i.EQ.1) GOTO 10
      i = i - 1
   70 i = i + 1
      IF (i.EQ.j) GOTO 50
      t = RB(i+1)
      it = IPERM(i+1)
      IF (RB(i).LE.t) GOTO 70
      k = i
   80 RB(k+1) = RB(k)
      IPERM(k+1) = IPERM(k)
      k = k - 1
      IF (t.LT.RB(k)) GOTO 80
      RB(k+1) = t
      IPERM(k+1) = it
      GOTO 70
C
 9005 FORMAT (/,' **Error** in SVGRP:',/,
     &    ' The length of vectors RA, RB and IPERM must be greater than'
     &    ,' or equal to one.',/,' On input the length, N, is given as',
     &    I5,'.')
      END
C
C     scopy.f
C
      SUBROUTINE SCOPY(N,SX,INCX,SY,INCY)
C
C     + + + PURPOSE + + +
C     Copy a vector X to a vector Y, both single precision.
C     Usage:  CALL SCOPY (N, SX, INCX, SY, INCY)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER N, INCX, INCY
      REAL SX(*), SY(*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     N      - Length of vectors X and Y.  (Input)
C     SX     - Real vector of length MAX(N*IABS(INCX),1).  (Input)
C     INCX   - Displacement between elements of SX.  (Input)
C              X(I) is defined to be.. SX(1+(I-1)*INCX) if INCX .GE. 0
C              or SX(1+(I-N)*INCX) if INCX .LT. 0.
C     SY     - Real vector of length MAX(N*IABS(INCY),1).  (Output)
C              SCOPY copies X(I) to Y(I) for I=1,...,N. X(I) and Y(I)
C              refer to specific elements of SX and SY, respectively.
C              See INCX and INCY argument descriptions.
C     INCY   - Displacement between elements of SY.  (Input)
C              Y(I) is defined to be.. SY(1+(I-1)*INCY) if INCY .GE. 0
C              or SY(1+(I-N)*INCY) if INCY .LT. 0.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, ix, iy, m, mp1
C
C     + + + INTRINSICS + + +
      INTEGER MOD
      INTRINSIC MOD
C
C     + + + END SPECIFICATIONS + + +
C
      IF (N.GT.0) THEN
        IF (INCX.NE.1 .OR. INCY.NE.1) THEN
C                                  CODE FOR UNEQUAL INCREMENTS
          ix = 1
          iy = 1
          IF (INCX.LT.0) ix = (-N+1)*INCX + 1
          IF (INCY.LT.0) iy = (-N+1)*INCY + 1
          DO 10 i = 1, N
            SY(iy) = SX(ix)
            ix = ix + INCX
            iy = iy + INCY
   10     CONTINUE
        ELSE
C                                  CODE FOR BOTH INCREMENTS EQUAL TO 1
          m = MOD(N,7)
C                                  CLEAN-UP LOOP
          DO 20 i = 1, m
            SY(i) = SX(i)
   20     CONTINUE
          mp1 = m + 1
          DO 30 i = mp1, N, 7
            SY(i) = SX(i)
            SY(i+1) = SX(i+1)
            SY(i+2) = SX(i+2)
            SY(i+3) = SX(i+3)
            SY(i+4) = SX(i+4)
            SY(i+5) = SX(i+5)
            SY(i+6) = SX(i+6)
   30     CONTINUE
        ENDIF
      ENDIF
      RETURN
      END
C
C     i1kgt.f
C
      INTEGER FUNCTION I1KGT(NELMTS,ITYPE)
C
C     + + + PURPOSE + + +
C     Allocate numerical workspace.
C     Usage:  I1KGT(NELMTS,ITYPE)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NELMTS, ITYPE
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NELMTS - Number of elements of data type ITYPE to be
C              allocated.  (Input)
C     ITYPE  - Data type of array to be allocated.  (Input)
C                 1 - logical
C                 2 - integer
C                 3 - real
C                 4 - double precision
C                 5 - complex
C                 6 - double complex
C     I1KGT  - Integer function.  (Output)  Returns the index of the
C              first element in the current allocation.
C
C  Remarks:
C  1. On return, the array will occupy
C     WKSP(I1KGT), WKSP(I1KGT+1), ..., WKSP(I1KGT+NELMTS-1) where
C     WKSP is an array of data type ITYPE equivalenced to RWKSP.
C
C  2. If I1KGT is negative, the absolute value of I1KGT is the
C     additional workspace needed for the current allocation.
C
C  3. The allocator reserves the first sixteen integer locations of
C     the stack for its own internal bookkeeping.  These are initialized
C     by the function IWKIN upon the first call to the allocation
C     package.
C
C  4. The use of the first ten integer locations is as follows:
C      WKSP( 1) - LOUT    The number of current allocations
C      WKSP( 2) - LNOW    The current active length of the stack
C      WKSP( 3) - LUSED   The maximum value of WKSP(2) achieved
C                         thus far
C      WKSP( 4) - LBND    The lower bound of permanent storage which
C                         is one numeric storage unit more than the
C                         maximum allowed length of the stack.
C      WKSP( 5) - LMAX    The maximum length of the storage array
C      WKSP( 6) - LALC    The total number of allocations handled by
C                         I1KGT
C      WKSP( 7) - LNEED   The number of numeric storage units by which
C                         the array size must be increased for all past
C                         allocations to succeed
C      WKSP( 8) - LBOOK   The number of numeric storage units used for
C                         bookkeeping
C      WKSP( 9) - LCHAR   The pointer to the portion of the permanent
C                         stack which contains the bookkeeping and
C                         pointers for the character workspace
C                         allocation.
C      WKSP(10) - LLCHAR  The length of the array beginning at LCHAR
C                         set aside for character workspace bookkeeping
C                         and pointers.
C                 NOTE -  If character workspace is not being used,
C                         LCHAR and LLCHAR can be ignored.
C  5. The next six integer locations contain values describing the
C     amount of storage allocated by the allocation system to the
C     various data types.
C      WKSP(11) - Numeric storage units allocated to LOGICAL
C      WKSP(12) - Numeric storage units allocated to INTEGER
C      WKSP(13) - Numeric storage units allocated to REAL
C      WKSP(14) - Numeric storage units allocated to DOUBLE PRECISION
C      WKSP(15) - Numeric storage units allocated to COMPLEX
C      WKSP(16) - Numeric storage units allocated to DOUBLE COMPLEX
C
C     + + + LOCAL VARIABLES + + +
      INTEGER i, idumal, igap, ileft, Isize(6), jtype, Lalc, Lbnd, 
     &        Lbook, Lmax, Lneed, lneed1, Lnow, Lout, Lused
C
C     + + + SAVE VARIABLES + + +
      LOGICAL first
      SAVE first
C
C     + + + COMMON SPECIFICATIONS + + +
      INCLUDE 'cubspl.inc'
C
C     + + + EQUIVALENCES + + +
      EQUIVALENCE (Lout,Iwksp(1))
      EQUIVALENCE (Lnow,Iwksp(2))
      EQUIVALENCE (Lused,Iwksp(3))
      EQUIVALENCE (Lbnd,Iwksp(4))
      EQUIVALENCE (Lmax,Iwksp(5))
      EQUIVALENCE (Lalc,Iwksp(6))
      EQUIVALENCE (Lneed,Iwksp(7))
      EQUIVALENCE (Lbook,Iwksp(8))
      EQUIVALENCE (Isize(1),Iwksp(11))
C
C     + + + FUNCTIONS + + +
      INTEGER I1KQU
      EXTERNAL I1KQU
C
C     + + + EXTERNALS + + +
      EXTERNAL IWKIN
C
C     + + + INTRINSICS + + +
      INTEGER IABS, MAX0, MOD
      INTRINSIC IABS, MAX0, MOD
C
C     + + + DATA + + +
      DATA first/.TRUE./
C
C     + + + END SPECIFICATIONS + + +
C
      IF (first) THEN
C                                  INITIALIZE WORKSPACE IF NEEDED
        first = .FALSE.
        CALL IWKIN(0)
      ENDIF
C                                  NUMBER OF ELEMENTS LESS THAN 0
      IF (NELMTS.LT.0) THEN
        WRITE (*,9005) NELMTS
        GOTO 10
      ENDIF
C                                  ILLEGAL DATA TYPE REQUESTED
      IF (ITYPE.EQ.0 .OR. IABS(ITYPE).GE.7) THEN
        WRITE (*,9010) ITYPE
        GOTO 10
      ENDIF
C                                  BOOKKEEPING OVERWRITTEN
      IF (Lnow.LT.Lbook .OR. Lnow.GT.Lused .OR. Lused.GT.Lmax .OR. 
     &    Lnow.GE.Lbnd .OR. Lout.GT.Lalc) THEN
        WRITE (*,9020)
        GOTO 10
      ENDIF
C
C                                  DETERMINE NUMBER OF LOCATIONS STILL
C                                  AVAILABLE FOR DATA TYPE ITYPE
C                                  NOTE: I1KQU ALLOWS FOR 2 INTEGER
C                                        POINTERS WHICH MUST BE HANDLED
C                                        ARTIFICIALLY IF ILEFT = 0.
      ileft = I1KQU(IABS(ITYPE))
C
      IF (ITYPE.GT.0) THEN
C                                  RELEASABLE STORAGE
        IF (ileft.GE.NELMTS) THEN
          I1KGT = (Lnow*Isize(2)-1)/Isize(ITYPE) + 2
          i = ((I1KGT-1+NELMTS)*Isize(ITYPE)-1)/Isize(2) + 3
C                                  IWKSP(I-1) CONTAINS THE DATA TYPE FOR
C                                  THIS ALLOCATION. IWKSP(I) CONTAINS
C                                  LNOW FOR THE PREVIOUS ALLOCATION.
          Iwksp(i-1) = ITYPE
          Iwksp(i) = Lnow
          Lout = Lout + 1
          Lalc = Lalc + 1
          Lnow = i
          Lused = MAX0(Lused,Lnow)
          Lneed = 0
        ELSE
C                                  RELEASABLE STORAGE WAS REQUESTED
C                                  BUT THE STACK WOULD OVERFLOW.
C                                  THEREFORE, ALLOCATE RELEASABLE
C                                  SPACE THROUGH THE END OF THE STACK
          IF (Lneed.EQ.0) THEN
            idumal = (Lnow*Isize(2)-1)/Isize(ITYPE) + 2
            i = ((idumal-1+ileft)*Isize(ITYPE)-1)/Isize(2) + 3
C                                  ADVANCE COUNTERS AND STORE POINTERS
C                                  IF THERE IS ROOM TO DO SO
            IF (i.LT.Lbnd) THEN
C                                  IWKSP(I-1) CONTAINS THE DATA TYPE FOR
C                                  THIS ALLOCATION. IWKSP(I) CONTAINS
C                                  LNOW FOR THE PREVIOUS ALLOCATION.
              Iwksp(i-1) = ITYPE
              Iwksp(i) = Lnow
              Lout = Lout + 1
              Lalc = Lalc + 1
              Lnow = i
              Lused = MAX0(Lused,Lnow)
            ENDIF
          ENDIF
C                                  CALCULATE AMOUNT NEEDED TO ACCOMODATE
C                                  THIS ALLOCATION REQUEST
          lneed1 = (NELMTS-ileft)*Isize(ITYPE)
          IF (ileft.EQ.0) THEN
            igap = Isize(ITYPE) - MOD(Lnow+Lneed,Isize(ITYPE))
            IF (igap.EQ.Isize(ITYPE)) igap = 0
            lneed1 = lneed1 + 2*Isize(2) + igap
          ENDIF
C                                  MODIFY LNEED ACCORDING TO THE SIZE
C                                  OF THE BASE BEING USED (D.P. HERE)
          Lneed = Lneed + ((lneed1+Isize(3)-1)/Isize(3))
C                                  SINCE CURRENT ALLOCATION IS ILLEGAL,
C                                  RETURN THE NEGATIVE OF THE ADDITIONAL
C                                  AMOUNT NEEDED TO MAKE IT LEGAL
          I1KGT = -Lneed
        ENDIF
C                                  PERMANENT STORAGE
      ELSEIF (ileft.GE.NELMTS) THEN
        jtype = -ITYPE
        I1KGT = (Lbnd*Isize(2)-1)/Isize(jtype) + 1 - NELMTS
        i = ((I1KGT-1)*Isize(jtype))/Isize(2) - 1
C                                  IWKSP(I) CONTAINS LBND FOR PREVIOUS
C                                  PERMANENT STORAGE ALLOCATION.
C                                  IWKSP(I+1) CONTAINS THE DATA TYPE FOR
C                                  THIS ALLOCATION.
        Iwksp(i) = Lbnd
        Iwksp(i+1) = jtype
        Lalc = Lalc + 1
        Lbnd = i
        Lneed = 0
      ELSE
C                                  PERMANENT STORAGE WAS REQUESTED
C                                  BUT THE STACK WOULD OVERFLOW,
C                                  THEREFORE, ALLOCATE RELEASABLE
C                                  SPACE THROUGH THE END OF THE STACK
        IF (Lneed.EQ.0) THEN
          jtype = -ITYPE
          idumal = (Lnow*Isize(2)-1)/Isize(jtype) + 2
          i = ((idumal-1+ileft)*Isize(jtype)-1)/Isize(2) + 3
C                                  ADVANCE COUNTERS AND STORE POINTERS
C                                  IF THERE IS ROOM TO DO SO
          IF (i.LT.Lbnd) THEN
C                                  IWKSP(I-1) CONTAINS THE DATA TYPE FOR
C                                  THIS ALLOCATION. IWKSP(I) CONTAINS
C                                  LNOW FOR THE PREVIOUS ALLOCATION.
            Iwksp(i-1) = jtype
            Iwksp(i) = Lnow
            Lout = Lout + 1
            Lalc = Lalc + 1
            Lnow = i
            Lused = MAX0(Lused,Lnow)
          ENDIF
        ENDIF
C                                  CALCULATE AMOUNT NEEDED TO ACCOMODATE
C                                  THIS ALLOCATION REQUEST
        lneed1 = (NELMTS-ileft)*Isize(-ITYPE)
        IF (ileft.EQ.0) THEN
          igap = Isize(-ITYPE) - MOD(Lnow+Lneed,Isize(-ITYPE))
          IF (igap.EQ.Isize(-ITYPE)) igap = 0
          lneed1 = lneed1 + 2*Isize(2) + igap
        ENDIF
C                                  MODIFY LNEED ACCORDING TO THE SIZE
C                                  OF THE BASE BEING USED (D.P. HERE)
        Lneed = Lneed + ((lneed1+Isize(3)-1)/Isize(3))
C                                  SINCE CURRENT ALLOCATION IS ILLEGAL,
C                                  RETURN THE NEGATIVE OF THE ADDITIONAL
C                                  AMOUNT NEEDED TO MAKE IT LEGAL
        I1KGT = -Lneed
      ENDIF
C                                  STACK OVERFLOW - UNRECOVERABLE ERROR
      IF (Lneed.GT.0) THEN
        WRITE (*,9025) Lneed + (Lmax/Isize(3))
        GOTO 10
      ENDIF
C
      RETURN
   10 WRITE (*,9015)
      STOP
 9005 FORMAT (/,' Number of elements is less than zero.',/,' NELMTS =',
     &        I8)
 9010 FORMAT (/,' Illegal data type requested. ITYPE =',I3)
 9015 FORMAT (' **Error** in I1KGT:')
 9020 FORMAT (/,' One or more of the first eight bookkeeping locations',
     &        ' in IWKSP have been overwritten.')
 9025 FORMAT (/,' Insufficient workspace for current allocation(s).',/,
     &        ' Correct by increasing WKDIM in CUBSPL.INC to at least',
     &        I8,'.')
      END
C
C     i1kqu.f
C
      INTEGER FUNCTION I1KQU(ITYPE)
C
C     + + + PURPOSE + + +
C     Return number of elements of data type ITYPE that
C     remain to be allocated in one request.
C     Usage:  I1KQU(ITYPE)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER ITYPE
C
C     + + + ARGUMENT DEFINITIONS + + +
C     ITYPE  - Type of storage to be checked (Input)
C                 1 - logical
C                 2 - integer
C                 3 - real
C                 4 - double precision
C                 5 - complex
C                 6 - double complex
C     I1KQU  - Integer function. (Output) Returns number of elements
C              of data type ITYPE remaining in the stack.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER Isize(6), Lalc, Lbnd, Lbook, Lmax, Lneed, Lnow, Lout, 
     &        Lused
C
C     + + + SAVE VARIABLES + + +
      LOGICAL first
      SAVE first
C
C     + + + COMMON SPECIFICATIONS + + +
      INCLUDE 'cubspl.inc'
C
C     + + + EQUIVALENCES + + +
      EQUIVALENCE (Lout,Iwksp(1))
      EQUIVALENCE (Lnow,Iwksp(2))
      EQUIVALENCE (Lused,Iwksp(3))
      EQUIVALENCE (Lbnd,Iwksp(4))
      EQUIVALENCE (Lmax,Iwksp(5))
      EQUIVALENCE (Lalc,Iwksp(6))
      EQUIVALENCE (Lneed,Iwksp(7))
      EQUIVALENCE (Lbook,Iwksp(8))
      EQUIVALENCE (Isize(1),Iwksp(11))
C
C     + + + EXTERNALS + + +
      EXTERNAL IWKIN
C
C     + + + INTRINSICS + + +
      INTEGER MAX0
      INTRINSIC MAX0
C
C     + + + DATA + + +
      DATA first/.TRUE./
C
C     + + + END SPECIFICATIONS + + +
C
      IF (first) THEN
C                                  INITIALIZE WORKSPACE IF NEEDED
        first = .FALSE.
        CALL IWKIN(0)
      ENDIF
C                                  BOOKKEEPING OVERWRITTEN
      IF (Lnow.LT.Lbook .OR. Lnow.GT.Lused .OR. Lused.GT.Lmax .OR. 
     &    Lnow.GE.Lbnd .OR. Lout.GT.Lalc) THEN
        WRITE (*,9015)
      ELSEIF (ITYPE.LE.0 .OR. ITYPE.GE.7) THEN
C                                  ILLEGAL DATA TYPE REQUESTED
        WRITE (*,9005) ITYPE
      ELSE
C                                  THIS CALCULATION ALLOWS FOR THE
C                                  TWO POINTER LOCATIONS IN THE STACK
C                                  WHICH ARE ASSIGNED TO EACH ALLOCATION
        I1KQU = MAX0(((Lbnd-3)*Isize(2))/Isize(ITYPE)-(Lnow*Isize(2)-1)
     &          /Isize(ITYPE)-1,0)
        RETURN
      ENDIF
C
      WRITE (*,9010)
      STOP
 9005 FORMAT (/,' Illegal data type requested. ITYPE =',I3)
 9010 FORMAT (' **Error** in I1KQU:')
 9015 FORMAT (/,' One or more of the first eight bookkeeping locations',
     &        ' in IWKSP have been overwritten.')
      END
C
C     iwkin.f
C
      SUBROUTINE IWKIN(NSU)
C
C     + + + PURPOSE + + +
C     Initialize bookkeeping locations describing the workspace stack.
C     Usage:  CALL IWKIN (NSU)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NSU
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NSU    - Number of numeric storage units to which the workspace
C              stack is to be initialized
C
C     + + + LOCAL VARIABLES + + +
      INTEGER Isize(6), Lalc, Lbnd, Lbook, Lmax, Lneed, Lnow, Lout, 
     &        Lused, melmts
C
C     + + + SAVE VARIABLES + + +
      LOGICAL first
      SAVE first
C
C     + + + COMMONS + + +
      INCLUDE 'cubspl.inc'
C
C     + + + EQUIVALENCES + + +
      EQUIVALENCE (Lout,Iwksp(1))
      EQUIVALENCE (Lnow,Iwksp(2))
      EQUIVALENCE (Lused,Iwksp(3))
      EQUIVALENCE (Lbnd,Iwksp(4))
      EQUIVALENCE (Lmax,Iwksp(5))
      EQUIVALENCE (Lalc,Iwksp(6))
      EQUIVALENCE (Lneed,Iwksp(7))
      EQUIVALENCE (Lbook,Iwksp(8))
      EQUIVALENCE (Isize(1),Iwksp(11))
C
C     + + + INTRINSICS + + +
      INTEGER MAX0
      INTRINSIC MAX0
C
C     + + + DATA + + +
      DATA first/.TRUE./
C
C     + + + END SPECIFICATIONS + + +
C
      IF (first) THEN
C
        IF (NSU.EQ.0) THEN
C                                  IF NSU=0 USE DEFAULT SIZE WKDIM
          melmts = WKDIM
        ELSE
          melmts = NSU
        ENDIF
C                                  NUMBER OF ITEMS .LT. 0
        IF (melmts.LE.0) THEN
          WRITE (*,9005) melmts
          STOP
        ENDIF
C
        first = .FALSE.
C                                  HERE TO INITIALIZE
C
C                                  SET DATA SIZES APPROPRIATE FOR A
C                                  STANDARD CONFORMING FORTRAN SYSTEM
C                                  USING THE FORTRAN
C                                  *NUMERIC STORAGE UNIT* AS THE
C                                  MEASURE OF SIZE.
C
C                                  TYPE IS REAL
Crsr    mtype = 3
C                                  LOGICAL
        Isize(1) = 1
C                                  INTEGER
        Isize(2) = 1
C                                  REAL
        Isize(3) = 1
C                                  DOUBLE PRECISION
        Isize(4) = 2
C                                  COMPLEX
        Isize(5) = 2
C                                  DOUBLE COMPLEX
        Isize(6) = 4
C                                  NUMBER OF WORDS USED FOR BOOKKEEPING
        Lbook = 16
C                                  CURRENT ACTIVE LENGTH OF THE STACK
        Lnow = Lbook
C                                  MAXIMUM VALUE OF LNOW ACHIEVED THUS
C                                  FAR
        Lused = Lbook
C                                  MAXIMUM LENGTH OF THE STORAGE ARRAY
        Lmax = MAX0(melmts,((Lbook+2)*Isize(2)+Isize(3)-1)/Isize(3))
C                                  LOWER BOUND OF THE PERMANENT STORAGE
C                                  WHICH IS ONE WORD MORE THAN THE
C                                  MAXIMUM ALLOWED LENGTH OF THE STACK
        Lbnd = Lmax + 1
C                                  NUMBER OF CURRENT ALLOCATIONS
        Lout = 0
C                                  TOTAL NUMBER OF ALLOCATIONS MADE
        Lalc = 0
C                                  NUMBER OF WORDS BY WHICH THE ARRAY
C                                  SIZE MUST BE INCREASED FOR ALL PAST
C                                  ALLOCATIONS TO SUCCEED
        Lneed = 0
      ELSEIF (NSU.NE.0) THEN
        WRITE (*,9010) Lmax
C
        STOP
      ENDIF
C
      RETURN
 9005 FORMAT (/,' **Error** in IWKIN:',/,
     &        ' Number of numeric storage units is not positive. NSU =',
     &        I8)
 9010 FORMAT (/,' **Error** in IWKIN:',/,
     &        'Workspace stack has previously been initialized to',I8,
     &        '.',/,
     &      ' Correct by making the call to IWKIN the first executable '
     &      ,'statement in the main program.')
      END
C
C     csval.f
C
      REAL FUNCTION CSVAL(X,NINTV,BREAK,CSCOEF)
C
C     + + + PURPOSE + + +
C     Evaluate a cubic spline.
C     Usage:  CSVAL(X, NINTV, BREAK, CSCOEF)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NINTV
      REAL X, BREAK(*), CSCOEF(4,*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     X      - Point at which the spline is to be evaluated.
C              (Input)
C     NINTV  - Number of polynomial pieces.  (Input)
C     BREAK  - Array of length NINTV+1 containing the breakpoints
C              for the piecewise cubic representation.  (Input)
C              BREAK must be strictly increasing.
C     CSCOEF - Matrix of size 4 by NINTV+1 containing the local
C              coefficients of the cubic pieces.  (Input)
C     CSVAL  - Value of the polynomial at X.  (Output)
C
C     + + + EXTERNALS + + +
      EXTERNAL PPDER
C
C     + + + END SPECIFICATIONS + + +
C
C                                  CUBIC SPLINE IS A SPECIAL PP
      CALL PPDER(0,X,4,NINTV,BREAK,CSCOEF,CSVAL)
C
      RETURN
      END
C
C     csder.f
C
      REAL FUNCTION CSDER(IDERIV,X,NINTV,BREAK,CSCOEF)
C
C     + + + PURPOSE + + +
C     Evaluate the derivative of a cubic spline.
C     Usage:  CSDER(IDERIV, X, NINTV, BREAK, CSCOEF)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IDERIV, NINTV
      REAL X, BREAK(*), CSCOEF(4,*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IDERIV - Order of the derivative to be evaluated.  (Input)
C              In particular, IDERIV = 0 returns the value of the
C              polynomial.
C     X      - Point at which the polynomial is to be evaluated.
C              (Input)
C     NINTV  - Number of polynomial pieces.  (Input)
C     BREAK  - Array of length NINTV+1 containing the breakpoints for
C              the piecewise cubic representation.  (Input)
C              BREAK must be strictly increasing.
C     CSCOEF - Matrix of size 4 by NINTV+1 containing the local
C              coefficients of the cubic pieces.  (Input)
C     CSDER  - Value of the IDERIV-th derivative of the polynomial at X.
C              (Output)
C
C     + + + FUNCTIONS + + +
      EXTERNAL PPDER
C
C     + + + END SPECIFICATIONS + + +
C
      CALL PPDER(IDERIV,X,4,NINTV,BREAK,CSCOEF,CSDER)
C
      RETURN
      END
C
C     ppder.f
C
      SUBROUTINE PPDER(IDERIV,X,KORDER,NINTV,BREAK,PPCOEF,CDER)
C
C     + + + PURPOSE + + +
C     Evaluate the derivative of a piecewise polynomial.
C     Usage:  CALL PPDER(IDERIV, X, KORDER, NINTV, BREAK, PPCOEF, CDER)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IDERIV, KORDER, NINTV
      REAL X, BREAK(*), PPCOEF(KORDER,*), CDER
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IDERIV - Order of the derivative to be evaluated.  (Input)
C              In particular, IDERIV = 0 returns the value of the
C              polynomial.
C     X      - Point at which the polynomial is to be evaluated.
C              (Input)
C     KORDER - Order of the polynomial.  (Input)
C     NINTV  - Number of polynomial pieces.  (Input)
C     BREAK  - Array of length NINTV+1 containing the breakpoints of
C              the piecewise polynomial representation.  (Input)
C              BREAK must be strictly increasing.
C     PPCOEF - Array of size KORDER*NINTV containing the
C              local coefficients of the piecewise polynomial pieces.
C              (Input)
C              PPCOEF is treated internally as a matrix of size
C              KORDER by NINTV.
C     CDER   - Value of the IDERIV-th derivative of the piecewise
C              polynomial at X.  (Output)
C
C     + + + LOCAL VARIABLES + + +
      INTEGER j, left
      REAL fmm, h, value
C                                  SPECIFICATIONS FOR SUBROUTINES
C     + + + EXTERNALS + + +
      EXTERNAL P3DER
C
C     + + + END SPECIFICATIONS + + +
C
C                                  IN CASE OF ERRORS
      value = 0.0
C                                  Check argument NINTV
      IF (NINTV.LT.1) THEN
        WRITE (*,9005) NINTV
C                                  Check argument IDERIV
      ELSEIF (IDERIV.LT.0) THEN
        WRITE (*,9010) IDERIV
C                                  Check argument KORDER
      ELSEIF (KORDER.LE.0) THEN
        WRITE (*,9015) KORDER
      ELSE
C                                  Derivatives of order KORDER or
C                                  higher are identically zero
        IF (IDERIV.LT.KORDER) THEN
C                                  Find index I of largest breakpoint
C                                  to the left of X
          CALL P3DER(NINTV,BREAK,X,left)
C                                  Evaluate jderiv-th derivative of
C                                  I-th polynomial piece at X
          fmm = KORDER - IDERIV
          h = X - BREAK(left)
          DO 10 j = KORDER, IDERIV + 1, -1
            value = (value/fmm)*h + PPCOEF(j,left)
            fmm = fmm - 1.0
   10     CONTINUE
        ENDIF
C
        CDER = value
        RETURN
      ENDIF
      WRITE (*,9020)
      STOP
C
 9005 FORMAT (/,
     &       ' The number of intervals must be at least 1 while NINTV ='
     &       ,I5,' is given.')
 9010 FORMAT (/,
     &    ' The order of the derivative must be positive while IDERIV ='
     &    ,I5,' is given.')
 9015 FORMAT (/,
     &     ' The order of the interpolating polynomial must be positive'
     &     ,' while KORDER =',I5,' is given.')
 9020 FORMAT (' **Error** in PPDER:')
      END
C
C     p3der.f
C
      SUBROUTINE P3DER(NINTV,BREAK,X,LEFT)
C
C     + + + PURPOSE + + +
C     Compute MAX (I, 1 .LE. NINTV. .AND. BREAK(I) .LE. X)
C     Usage:  CALL P3DER (NINTV, BREAK, X, LEFT)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NINTV, LEFT
      REAL X, BREAK(*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NINTV  - Number of polynomial pieces.  (Input)
C     BREAK  - Vector of length NINTV+1 containing the breakpoints
C              of the piecewise polynomial representation.  (Input)
C     X      - The point whose location in BREAK is to be found.
C     LEFT   - Integer whose value is
C                LEFT
C                  1      IF                       X .LT.  BREAK(1)
C                  I      IF         BREAK(I).LE. X .LT. BREAK(I+1)
C                 NINTV   IF                    BREAK(NINTV) .LE. X
C              The asymmetric treatment of the interval is due to the
C              decision to make all PP functions continuous from the
C              right.  (Output)
C
C  Remark:
C     This routine is based in INTERV in de Boor, p92-93.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER ihi, istep, middle
C
C     + + + SAVE VARIABLES + + +
      INTEGER ilo
      SAVE ilo
C
C     + + + DATA + + +
      DATA ilo/1/
C
C     + + + END SPECIFICATIONS + + +
C
      ihi = ilo + 1
      IF (ihi.GE.NINTV) THEN
        IF (X.GE.BREAK(NINTV)) THEN
          LEFT = NINTV
          GOTO 40
        ELSEIF (NINTV.LE.1) THEN
          LEFT = 1
          GOTO 40
        ENDIF
        ilo = NINTV - 1
        ihi = NINTV
      ENDIF
C
      IF (X.LT.BREAK(ihi)) THEN
        IF (X.GE.BREAK(ilo)) THEN
          LEFT = ilo
          GOTO 40
        ENDIF
C                                  Now X .LT. BREAK(ILO) - decrease ILO
C                                  to capture X
        istep = 1
   10   ihi = ilo
        ilo = ihi - istep
        IF (ilo.GT.1) THEN
          IF (X.GE.BREAK(ilo)) GOTO 30
          istep = istep*2
          GOTO 10
        ENDIF
        ilo = 1
        IF (X.LT.BREAK(1)) THEN
          LEFT = 1
          GOTO 40
        ENDIF
        GOTO 30
      ENDIF
C                                  Now X .GE. BREAK(IHI) - increase IHI
C                                  to capture X
      istep = 1
   20 ilo = ihi
      ihi = ilo + istep
      IF (ihi.LT.NINTV) THEN
        IF (X.LT.BREAK(ihi)) GOTO 30
        istep = istep*2
        GOTO 20
      ENDIF
      IF (X.GE.BREAK(NINTV)) THEN
        LEFT = NINTV
        GOTO 40
      ENDIF
      ihi = NINTV
C                                  Now BREAK(ILO) .LE. X .LT.
C                                  BREAK(IHI) - narrow the inteval
   30 middle = (ilo+ihi)/2
      IF (middle.EQ.ilo) THEN
        LEFT = ilo
        GOTO 40
      ENDIF
C                                  It is assumed that MIDDLE = ILO in
C                                  case IHI = ILO+1
      IF (X.LT.BREAK(middle)) THEN
        ihi = middle
      ELSE
        ilo = middle
      ENDIF
      GOTO 30
   40 RETURN
      END
