      PROGRAM BRANCH_MAIN
C     Last MODIFIED July 27, 1995 -- RSR
C     MODIFIED August 29, 1994 -- RSR
C     MODIFIED October 8, 1993 -- RSR
C     MODIFIED MAY 14, 1993 -- RSR
C     MODIFIED JULY 22, 1992 -- RSR
C # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
C #    BRANCH-NETWORK FLOW MODEL USING A 4-POINT IMPLICIT TECHNIQUE   #
C #      BY R. W. SCHAFFRANEK, R. A. BALTZER, AND D. E. GOLDBERG      #
C #                      July 27, 1995 VERSION                        #
C #                BY R. W. SCHAFFRANEK AND R. S. REGAN               #
C #   THIS VERSION INCLUDES AN INTERFACE WITH THE USGS GROUNDWATER    #
C #   FLOW MODEL MODFLOW, IN ORDER TO SIMULATE STREAM/AQUIFER         #
C #   RELATIONS.  THIS INTERFACE WAS DEVELOPED BY:                    #
C #                     E. D. SWAIN AND E. J. WEXLER                  #
C # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
C #   THIS PROGRAM CALCULATES TRANSIENT FLOW IN A NETWORK OF          #
C #   INTERCONNECTED OPEN CHANNELS.  TIME DERIVATIVES ARE APPROXIMATED#
C #   AS CENTERED IN SPACE AND TIME; SPATIAL DERIVATIVES ARE TREATED  #
C #   AS CENTERED IN SPACE AND WEIGHTED IN TIME ACCORDING TO A USER   #
C #   DEFINED WEIGHTING FACTOR.  A LINEAR MATRIX SOLUTION IS EFFECTED #
C #   WITH ITERATIVE IMPROVEMENT OF RESULTS OPTIONALLY SPECIFIABLE.   #
C #   THE 4*N BY 4*N MATRIX (N IS THE NUMBER OF BRANCHES) IS SOLVED   #
C #   BY GAUSS ELIMINATION USING MAXIMUM PIVOT STRATEGY.              #
C # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
C #   THE FOLLOWING CONDITIONS ARE STIPULATED FOR DISSEMINATION AND   #
C #      UTILIZATION OF THE BRANCH MODEL SOURCE-CODE SOFTWARE:        #
C #   1) THE SOURCE CODE SHALL BE USED FOR PUBLIC BENEFIT ONLY;  IT   #
C #      SHALL NOT BE USED FOR PRIVATE PROFIT OR GAIN.                #
C #   2) THE SOURCE CODE SHALL BE TREATED AS PROPRIETARY INFORMATION  #
C #      AND THE PROPERTY OF THE U. S. GEOLOGICAL SURVEY; ITS DISSEM- #
C #      INATION TO OTHER FEDERAL, STATE, LOCAL, OR FOREIGN GOVERN-   #
C #      MENTAL AGENCIES OR TO PRIVATE INDIVIDUALS OR CORPORATIONS    #
C #      SHALL BE THROUGH USGS CONTACT, ONLY.  PROCURERS OF THIS      #
C #      SOURCE CODE SHALL SPECIFICALLY REFRAIN FROM PROVIDING OR     #
C #      OTHERWISE DISSEMINATING COMPLETE, PARTIAL, OR MODIFIED       #
C #      COPIES TO THIRD PARTIES.                                     #
C #   3) THE AUTHORS OF THE SOURCE CODE AND THE USGS SHALL BE HELD    #
C #      BLAMELESS FOR ITS MISUSE OR FOR ANY INCORRECT INTERPRETA-    #
C #      TION OF RESULTS GENERATED THROUGH ITS USE.                   #
C #   4) THE SOURCE CODE IS MADE AVAILABLE "AS IS" WITHOUT GUARANTEE  #
C #      OF REVISION MAINTENANCE SUPPORT OR WARRANTY AS TO ITS SUIT-  #
C #      ABILITY FOR IMPLEMENTATION AT OTHER COMPUTER INSTALLATIONS.  #
C #   5) THE AUTHORS OF THE SOURCE CODE AND THE USGS SHALL BE APPRO-  #
C #      PRIATELY ACKNOWLEDGED IN ALL PUBLICATIONS AND JOURNAL PAPERS #
C #      REPORTING ON RESEARCH OR OTHER INVESTIGATIVE EFFORTS AIDED   #
C #      BY THE SOFTWARE PROVIDED.                                    #
C # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
C
      INTEGER mmm, nstp
      INCLUDE 'comcon.cmn'
      INCLUDE 'netwrk.cmn'
      INCLUDE 'datime.cmn'
C
C---variables needed for compatability with MODFLOW interface
      DOUBLE PRECISION hnew(1,1,1)
      REAL hold(1,1,1)
      INTEGER iss, ibrprn, kkiter, ncol, nrow, nlay, ibound(1,1,1), 
     &        kkper
C
      EXTERNAL BRINIT, SOLVER, BREND
C
C---Set version for use with Unix RCS ident command
      Brdate = 
     & '$Id: branch.f,v 4.0 1996/03/04 15:39:30 rsregan Exp rsregan $'
C
C---Initialize constants, arrays, and computational variables
      CALL BRINIT
C
C---Initialize time step loop counter
      mmm = M
      nstp = Nsteps
C
C---Initialize MODFLOW interface dummy variables
      iss = 0
      ibrprn = 1
      ncol = 1
      nrow = 1
      nlay = 1
      kkiter = 1
      kkper = 1
      hnew(1,1,1) = 0.0D0
      hold(1,1,1) = 0.0
      ibound(1,1,1) = 0
C
C---Compute results for 1 through Nsteps time steps
      CALL SOLVER(mmm,nstp,iss,ibrprn,kkiter,kkper,ncol,nrow,nlay,hnew,
     &            hold,ibound)
C
C---TERMINATE EXECUTION
      CALL BREND
      STOP
      END
