      SUBROUTINE OPTETA
C
C  THIS SUBROUTINE DETERMINES FRICTIONAL-RESISTANCE COEFFICIENTS
C  IN ORDER TO OPTIMIZE COMPARISONS OF COMPUTED VERSUS MEASURED
C  STAGES OR DISCHARGES.  DISCRETION SHOULD BE EXERCISED IN THE
C  USE OF THIS AUTOMATIC CALIBRATION AID.  IN PARTICULAR, THE
C  SCHEMATIZATION OF THE OPEN-CHANNEL SYSTEM (NETWORK LAYOUT AND
C  GEOMETRY DESCRIPTION), THE BOUNDARY-CONDITION DEFINITIONS
C  (LOCATION, TYPE, AND DATUM), AND THE FORMAT OF THE FINITE-
C  DIFFERENCE APPROXIMATION (TIME STEP, DISTANCE INTERVALS, AND
C  THETA/CHI FACTORS) SHOULD HAVE BEEN EVALUATED AND BE EXPECTED
C  TO REMAIN FIXED AT THEIR CURRENT DESIGNATIONS.
C
      INCLUDE 'dimens.cmn'
      INCLUDE 'dtcomp.cmn'
      INCLUDE 'xsinit.cmn'
      INCLUDE 'mdatas.cmn'
      INCLUDE 'comcon.cmn'
      INCLUDE 'branch.cmn'
      INCLUDE 'datime.cmn'
      INCLUDE 'bctime.cmn'
      INCLUDE 'logics.cmn'
      DOUBLE PRECISION a0, a1, sdev, aint, bcoef(10)
      INTEGER kteta, maxreg, ket, ijm, ijmrn, kmd, i, ij, nsm1, j, n, 
     &        luregr, npass, ktswap
      REAL cmdold, zqc, azqc, azqm, cmdiff, czq, rnold, rnnew, temp
      CHARACTER*9 refile
      EXTERNAL OPNFIL, REGR, ERRSTP
      REAL ABS, SIGN
      INTRINSIC ABS, SIGN
      SAVE kteta, cmdold, ijm, luregr, maxreg, refile
      DATA kteta/0/, cmdold/0.0/, ijm/0/, luregr/20/
      DATA maxreg/4/, refile/'REGR.DAT '/
      Modeta = .FALSE.
      ket = Ietime + M*Idtm
      IF (ket-Mektim.EQ.Idtm) THEN
C
C DETERMINE CONSTANT, LINEAR, AND QUADRATIC ETA FUNCTIONS
C
        WRITE (*,9005)
        DO 20 i = 1, Nbch
          ij = MAXS - Xskt(i)
          nsm1 = Nsec(i) - 1
          DO 10 j = 1, nsm1
            ij = ij + 1
            a1 = 0.0D0
            a0 = 0.0D0
            IF (Sumczq(ij).NE.0.0) THEN
              a1 = (kteta*Szqeta(ij)-Sumczq(ij)*Sumeta(ij))
     &             /(kteta*Sczqsq(ij)-Sumczq(ij)*Sumczq(ij))
              a0 = (Sumeta(ij)-a1*Sumczq(ij))/kteta
            ENDIF
            Sumeta(ij) = Sumeta(ij)/kteta
            IF (Rn(2,ij).EQ.0.0 .AND. Rn(3,ij).EQ.0.0)
     &          Rn(1,ij) = Sumeta(ij)
            WRITE (*,9010) i, j, Sumeta(ij), a0, a1
   10     CONTINUE
   20   CONTINUE
        DO 40 npass = 1, kteta-1
          ktswap = 0
          DO 30 n = 1, kteta-npass
            IF (Zqopt(n).GT.Zqopt(n+1)) THEN
              ktswap = ktswap + 1
              temp = Zqopt(n)
              Zqopt(n) = Zqopt(n+1)
              Zqopt(n+1) = temp
              temp = Rnopt(n)
              Rnopt(n) = Rnopt(n+1)
              Rnopt(n+1) = temp
            ENDIF
   30     CONTINUE
          IF (ktswap.EQ.0) GOTO 50
   40   CONTINUE
   50   CALL OPNFIL(luregr,refile)
        WRITE (luregr,9015) maxreg, Zqopt(1), Zqopt(kteta), refile
        DO 60 n = 1, maxreg
          WRITE (*,9020) n
          CALL REGR(kteta,Zqopt,Rnopt,MAXMZQ,aint,bcoef,n,sdev)
          IF (sdev.EQ.0.0) THEN
            WRITE (*,9025)
            STOP
          ENDIF
          WRITE (*,9030) sdev, aint, (i+1,bcoef(i),i=1,n)
          WRITE (luregr,'(I1)') n
          WRITE (luregr,'((E16.8))') aint, (bcoef(i),i=1,n)
   60   CONTINUE
        DO 70 i = 1, kteta
          WRITE (luregr,9035) Zqopt(i), Rnopt(i)
   70   CONTINUE
        WRITE (*,9040)
      ELSEIF (ket.GE.Meitim .AND. ket.LE.Mektim) THEN
        IF (ijm.EQ.0) THEN
          WRITE (*,9045) Etamin, Etamax, Tolerr
C ETA OPTIMIZATION ONLY PERMITTED FOR ONE SET OF MEASURED DATA
          IF (Ktmeas.GT.1) WRITE (*,9050)
          ijm = MAXS - Xskt(Mbch(1)) + Msec(1)
          ijmrn = ijm
          IF (Msec(1).EQ.Nsec(Mbch(1))) ijmrn = ijm - 1
C ETA OPTIMIZATION ONLY POSSIBLE FOR CONSTANT ETA IN CHANNEL SEGMENT
          IF (Rn(2,ijmrn).NE.0.0 .OR. Rn(3,ijmrn).NE.0.0) CALL ERRSTP(
     &'OPTIMIZATION ONLY POSSIBLE IF ETA SPECIFIED AS CONSTANT AT MEASUR
     &ED SITE')
        ENDIF
        kmd = (Khr*60+Kmn)/Mdt - (Mihr*60+Mimn)/Mdt + 1
        IF (Stages) THEN
          zqc = (Zp(ijm)+Zdatum)*Qzconv
          azqc = ABS((Zp(ijm)+Zdatum)*Qzconv)
        ELSE
          zqc = Qp(ijm)*Qzconv
          azqc = ABS(Qp(ijm)*Qzconv)
        ENDIF
        azqm = ABS(Zqmeas(kmd,1))
        cmdiff = (azqc-azqm)/azqm
        IF (ABS(cmdiff).GT.0.5) cmdiff = SIGN(0.5,cmdiff)
        IF (ABS(cmdiff).GT.Tolerr) THEN
C
C DETERMINE NEW ETA FOR IMPROVED COMPUTED VERSUS MEASURED COMPARISON
C
          rnold = Rn4(ijmrn)
          rnnew = rnold + cmdiff*rnold
          IF ((cmdiff.LT.0.0 .AND. cmdold.GT.0.0) .OR. 
     &        (cmdiff.GT.0. .AND. cmdold.LT.0.)) rnnew = (rnnew+rnold)
     &        *0.5
          IF (rnnew.LT.Etamin .AND. rnold.GT.rnnew) THEN
            WRITE (*,9055) 'IN'
            cmdold = 0.0
          ELSEIF (rnnew.GT.Etamax .AND. rnold.LT.rnnew) THEN
            WRITE (*,9055) 'AX'
            cmdold = 0.0
          ELSE
            DO 90 i = 1, Nbch
              ij = MAXS - Xskt(i)
              nsm1 = Nsec(i) - 1
              DO 80 j = 1, nsm1
                ij = ij + 1
C modify eta of non-measured segments in branch by computed difference
                Rn4(ij) = Rn4(ij) + cmdiff*Rn4(ij)
C (or alternatively)
C set eta of non-measured segments in branch to measured-segment value
C               Rn4(ij) = rnnew
                IF (Rn(2,ij).EQ.0.0 .AND. Rn(3,ij).EQ.0.0)
     &              Rn(1,ij) = Rn4(ij)
   80         CONTINUE
   90       CONTINUE
            Rn4(ijmrn) = rnnew
            Rn(1,ijmrn) = rnnew
            WRITE (*,9060) rnold, rnnew
            Modeta = .TRUE.
            cmdold = cmdiff
          ENDIF
        ELSE
C
C COMPUTED VERSUS MEASURED COMPARISON WITHIN USER-SPECIFIED TOLERENCE
C
          kteta = kteta + 1
          DO 110 i = 1, Nbch
            ij = MAXS - Xskt(i)
            nsm1 = Nsec(i) - 1
            DO 100 j = 1, nsm1
              ij = ij + 1
              IF (Stages) THEN
                czq = (Zp(ij)+Zdatum)*Qzconv
                Sumczq(ij) = Sumczq(ij) + czq
                Sczqsq(ij) = Sczqsq(ij) + czq*czq
                Szqeta(ij) = Szqeta(ij) + czq*Rn4(ij)
              ELSE
                czq = Qp(ij)*Qzconv
                Sumczq(ij) = Sumczq(ij) + czq
                Sczqsq(ij) = Sczqsq(ij) + czq*czq
                Szqeta(ij) = Szqeta(ij) + czq*Rn4(ij)
              ENDIF
              Sumeta(ij) = Sumeta(ij) + Rn4(ij)
  100       CONTINUE
  110     CONTINUE
          Rnopt(kteta) = Rn4(ijmrn)
          Zqopt(kteta) = zqc
          cmdold = 0.0
        ENDIF
      ENDIF
      RETURN
 9005 FORMAT (/' CONSTANT AND LINEAR FRICTIONAL-RESISTANCE COEFFICIENTS'
     &        ,/,' SEGMENT',5X,'CONSTANT',6X,'INTERCEPT',7X,'SLOPE')
 9010 FORMAT (' (',I2,',',I2,'):',3E15.8)
 9015 FORMAT (I1,/,F10.1,/,F10.1,/,A9)
 9020 FORMAT (/' POLYNOMIAL REGRESSION ORDER =',I3)
 9025 FORMAT (' MATRIX IS SINGULAR')
 9030 FORMAT (' SDEV =',F14.6,/,' RN(1) =',E17.8,/,
     &        (' RN(',I1,') =',E17.8))
 9035 FORMAT (F10.2,F10.8)
 9040 FORMAT (/' ETA OPTIMIZATION COMPLETED'/)
 9045 FORMAT (/' ETA OPTIMIZATION USING:'/' ETAMIN =',F7.4,' ETAMAX =',
     &        F7.4,' TOLERR =',F6.3/)
 9050 FORMAT (/,
     &' *WARNING* OPTIMIZATION ONLY IN EFFECT FOR FIRST MEASURED DATA SE
     &T'/)
 9055 FORMAT (' ETA OPTIMIZATION INHIBITED BY ATTEMPT TO EXCEED M',A2,
     &        'IMUM')
 9060 FORMAT (' ETA',F7.4,' CHANGED TO',F7.4)
      END
C
      SUBROUTINE REGR(M,X,Y,MAXMZQ,A,B,N,REG)
      INTEGER M, N, np1, i, j, MAXMZQ
      REAL X(MAXMZQ), Y(MAXMZQ), eps
      DOUBLE PRECISION B(10), c(11,11), sx(20), syx(10), cyx(10)
      DOUBLE PRECISION A, s, sy, syy, cyy, det, dum, temp, REG, fm, 
     &                 denom
      EXTERNAL SIMUL, ERRSTP
      DOUBLE PRECISION DSQRT
      INTRINSIC DSQRT
      DATA eps/1.0E-20/
C COMPUTE SUMS OF POWERS AND PRODUCTS
      IF (N.GT.10) CALL ERRSTP('IN REGR, EXCEEDED ARRAY BOUNDS')
      np1 = N + 1
      sy = 0.0D0
      syy = 0.0D0
      DO 10 i = 1, 2*N
        sx(i) = 0.0D0
   10 CONTINUE
      DO 20 i = 1, N
        syx(i) = 0.0D0
   20 CONTINUE
      DO 50 i = 1, M
        sy = sy + Y(i)
        syy = syy + Y(i)*Y(i)
        dum = 1.0D0
        DO 30 j = 1, N
          dum = dum*X(i)
          sx(j) = sx(j) + dum
          syx(j) = syx(j) + Y(i)*dum
   30   CONTINUE
        DO 40 j = np1, N*2
          dum = dum*X(i)
          sx(j) = sx(j) + dum
   40   CONTINUE
   50 CONTINUE
C COMPUTE C(I,J) COEFFICIENTS
      fm = 1.00D0/M
      cyy = syy - sy*sy*fm
      DO 70 i = 1, N
        cyx(i) = syx(i) - sy*sx(i)*fm
        c(i,np1) = cyx(i)
        DO 60 j = 1, N
          c(i,j) = sx(i+j) - sx(i)*sx(j)*fm
   60   CONTINUE
   70 CONTINUE
C CALL SIMUL TO SOLVE SIMULTANEOUS EQUATIONS
      CALL SIMUL(N,c,B,eps,1,11,det)
      WRITE (*,9005) det
      IF (det.EQ.0.0) THEN
        REG = 0.0D0
        GOTO 90
      ENDIF
C COMPUTE INTERCEPT A AND STANDARD DEVIATION S
      dum = sy
      temp = cyy
      DO 80 i = 1, N
        dum = dum - B(i)*sx(i)
        temp = temp - B(i)*cyx(i)
   80 CONTINUE
      A = dum*fm
      denom = M - N - 1
      s = DSQRT(temp/denom)
      REG = s
   90 RETURN
 9005 FORMAT (' DET = ',E14.8)
      END
C
      SUBROUTINE SIMUL(N,A,X,EPS,INDIC,NRC,SIMU)
      INTEGER N, INDIC, NRC, i, j, k, max, km1, ir, jc, irowk, jcolk, 
     &        irowi, jcoli, intch, jtemp, irowj, jcolj, Irow, Jcol, 
     &        Jord, MAXSIM
      PARAMETER (MAXSIM=80)
      REAL EPS
      DOUBLE PRECISION aijck, deter, pivot, SIMU, pivoti
      DOUBLE PRECISION Y(MAXSIM), A(NRC,NRC), X(N)
      DIMENSION Irow(MAXSIM), Jcol(MAXSIM), Jord(MAXSIM)
      COMMON /SIMLCM/ Y, Irow, Jcol, Jord
      DOUBLE PRECISION DABS
      INTRINSIC DABS
C
      IF (INDIC.LT.0) THEN
        max = N
      ELSE
        max = N + 1
      ENDIF
      IF (N.GT.MAXSIM) THEN
        WRITE (*,9005)
        SIMU = 0.0D0
        GOTO 180
      ENDIF
C BEGIN ELIMINATION PROCEDURE
      deter = 1.0D0
      DO 80 k = 1, N
        km1 = k - 1
C SEARCH FOR PIVOT ELEMENT
        pivot = 0.0D0
        DO 40 i = 1, N
          DO 30 j = 1, N
C SCAN IROW AND JCOL ARRAYS FOR INVALID PIVOT SUBSCRIPTS
            IF (k.NE.1) THEN
              DO 20 ir = 1, km1
                DO 10 jc = 1, km1
                  IF (i.EQ.Irow(ir) .OR. j.EQ.Jcol(jc)) GOTO 30
   10           CONTINUE
   20         CONTINUE
            ENDIF
            IF (DABS(A(i,j)).GT.DABS(pivot)) THEN
              pivot = A(i,j)
              Irow(k) = i
              Jcol(k) = j
            ENDIF
   30     CONTINUE
   40   CONTINUE
C INSURE THAT SELECTED PIVOT IS LARGER THAN EPS
        IF (DABS(pivot).GT.EPS) THEN
C UPDATE THE DETERMINANT VALUE
          irowk = Irow(k)
          jcolk = Jcol(k)
          deter = deter*pivot
          pivoti = 1.0D0/pivot
C NORMALIZE PIVOT ROW ELEMENTS
          DO 50 j = 1, max
            A(irowk,j) = A(irowk,j)*pivoti
   50     CONTINUE
C CARRY OUT ELIMINATION AND DEVELOP INVERSE
          A(irowk,jcolk) = pivoti
          DO 70 i = 1, N
            aijck = A(i,jcolk)
            IF (i.NE.irowk) THEN
              A(i,jcolk) = -aijck*pivoti
              DO 60 j = 1, max
                IF (j.NE.jcolk) A(i,j) = A(i,j) - aijck*A(irowk,j)
   60         CONTINUE
            ENDIF
   70     CONTINUE
        ELSE
          SIMU = 0.0D0
          GOTO 180
        ENDIF
   80 CONTINUE
C ORDER SOLUTION VALUES (IF ANY) AND CREATE JORD ARRAY
      DO 90 i = 1, N
        irowi = Irow(i)
        jcoli = Jcol(i)
        Jord(irowi) = jcoli
        IF (INDIC.GE.0) X(jcoli) = A(irowi,max)
   90 CONTINUE
C ADJUST SIGN OF DETERMINANT
      intch = 0
      DO 110 i = 1, N-1
        DO 100 j = i+1, N
          IF (Jord(j).LT.Jord(i)) THEN
            jtemp = Jord(j)
            Jord(j) = Jord(i)
            Jord(i) = jtemp
            intch = intch + 1
          ENDIF
  100   CONTINUE
  110 CONTINUE
      IF (intch/2*2.NE.intch) deter = -deter
C IF INDIC IS POSITIVE RETURN WITH RESULTS
      IF (INDIC.LT.1) THEN
C IF INDIC IS NEGATIVE OR ZERO, UNSCRAMBLE INVERSE FIRST BY ROWS
        DO 140 j = 1, N
          DO 120 i = 1, N
            irowi = Irow(i)
            jcoli = Jcol(i)
            Y(jcoli) = A(irowi,j)
  120     CONTINUE
          DO 130 i = 1, N
            A(i,j) = Y(i)
  130     CONTINUE
  140   CONTINUE
C THEN BY COLUMNS
        DO 170 i = 1, N
          DO 150 j = 1, N
            irowj = Irow(j)
            jcolj = Jcol(j)
            Y(irowj) = A(i,jcolj)
  150     CONTINUE
          DO 160 j = 1, N
            A(i,j) = Y(j)
  160     CONTINUE
  170   CONTINUE
C RETURN FOR INDIC NEGATIVE OR ZERO
      ENDIF
      SIMU = deter
  180 RETURN
 9005 FORMAT (' N TOO LARGE')
      END
