      SUBROUTINE SETAB1(IJ)
C
C  THIS SUBROUTINE INTERPOLATES ETA FROM INPUT TABULAR FUNCTIONS.
C  ETA IS INPUT AS A:
C  (1) CONSTANT
C  --OR-- AS A TABULAR FUNCTION OF
C  (2) TEMPERATURE
C      A QUADRATIC RELATIONSHIP IS DEFINED FOR THE KINEMATIC VISCOSITY AS
C      A FUNCTION OF WATER TEMPERATURE WHICH IS VALID IN THE FAHRENHEIT
C      RANGE 32<=T<=95 AND THE CELSIUS RANGE 0<=T<=35.  THE KINEMATIC
C      VISCOSITY (NU) IS USED AS 10**6NU = 1 M**2/S IN METRIC UNITS AND
C      AS 10**5NU = 1 FT**2/S IN ENGLISH UNITS.
C  (3) HYDRAULIC DEPTH OR RADIUS
C  (4) DISCHARGE (4)
C  (5) FROUDE NUMBER
C  (6) REYNOLDS NUMBER
C  (7) WATER LEVEL
C
      REAL nu
      INTEGER IJ, nd, k
      INCLUDE 'dimens.cmn'
      INCLUDE 'geom.cmn'
      INCLUDE 'ceta.cmn'
      INCLUDE 'comcon.cmn'
      REAL SQRT
      INTRINSIC SQRT
      nd = Ipt(IJ)
C
      IF (Typeta.EQ.7) THEN
        DO 10 k = 1, nd
          Funeta(k,IJ) = Za(k,IJ)
   10   CONTINUE
      ELSEIF (Typeta.EQ.3) THEN
C
C---Use hydraulic radius if input
        IF (Rrflg(IJ)) THEN
          DO 20 k = 1, nd
            Funeta(k,IJ) = Aa(k,IJ)/Wp(k,IJ)
   20     CONTINUE
        ELSE
          DO 25 k = 1, nd
            Funeta(k,IJ) = Aa(k,IJ)/Bb(k,IJ)
   25     CONTINUE
        ENDIF
      ELSEIF (Typeta.EQ.4) THEN
        DO 30 k = 1, nd
          Funeta(k,IJ) = Qa(k,IJ)
   30   CONTINUE
      ELSEIF (Typeta.EQ.2) THEN
        DO 40 k = 1, nd
          Funeta(k,IJ) = Ta(k,IJ)
   40   CONTINUE
      ELSEIF (Typeta.EQ.5) THEN
C---V = QA / AA, D = AA / BB
C---FUNETA = V / SQRT(G*D)
        DO 50 k = 1, nd
          Funeta(k,IJ) = Qa(k,IJ)/(Aa(k,IJ)*SQRT(G*Aa(k,IJ)/Bb(k,IJ)))
   50   CONTINUE
      ELSEIF (Typeta.EQ.6) THEN
C---FUNETA = V * R / NU
        DO 60 k = 1, nd
          nu = Ceta1 - (Ceta2-Ceta3*Ta(k,IJ))*Ta(k,IJ)
          Funeta(k,IJ) = Qa(k,IJ)/(Bb(k,IJ)*nu)
   60   CONTINUE
      ENDIF
      RETURN
      END
C
      REAL FUNCTION SETAB(IJ,Z,Q,A,R)
C
C  THIS SUBROUTINE INTERPOLATES ETA FROM INPUT TABULAR FUNCTIONS.
C  ETA IS INPUT AS A CONSTANT (1) OR AS A TABULAR FUNCTION OF
C  TEMPERATURE (2), HYDRAULIC DEPTH (3), DISCHARGE (4), FROUDE
C  NUMBER (5), REYNOLDS NUMBER (6), OR WATER LEVEL (7). A QUADRATIC
C  RELATIONSHIP IS DEFINED FOR THE KINEMATIC VISCOSITY AS A
C  FUNCTION OF WATER TEMPERATURE WHICH IS VALID IN THE FAHRENHEIT
C  RANGE 32<=T<=95 AND THE CELSIUS RANGE 0<=T<=35. THE KINEMATIC
C  VISCOSITY (NU) IS USED AS 10**6NU = 1 M**2/S IN METRIC UNITS
C  AND AS 10**5NU = 1 FT**2/S IN ENGLISH UNITS.
C
      REAL nu, Z, Q, A, R, x
      INTEGER IJ, nd, k
      INCLUDE 'dimens.cmn'
      INCLUDE 'geom.cmn'
      INCLUDE 'ceta.cmn'
      INCLUDE 'comcon.cmn'
      INCLUDE 'xsinit.cmn'
      REAL SQRT, ABS
      INTRINSIC SQRT, ABS
C
      SETAB = Eta(1,IJ)
      IF (Typeta.NE.1) THEN
        IF (Typeta.EQ.7) THEN
          x = Z
        ELSEIF (Typeta.EQ.3) THEN
          x = R
        ELSEIF (Typeta.EQ.4) THEN
          x = Q
        ELSEIF (Typeta.EQ.2) THEN
          x = T(IJ)
        ELSEIF (Typeta.EQ.5) THEN
C---X = V / SQRT(G*R)
          x = ABS(Q)/(A*SQRT(G*R))
        ELSE
C---( Typeta.EQ.6 )
C---X = V * R / NU
          nu = Ceta1 - (Ceta2-Ceta3*T(IJ))*T(IJ)
          x = ABS(Q)*R/(A*nu)
        ENDIF
C
        IF (x.GT.Funeta(1,IJ)) THEN
          nd = Ipt(IJ)
          DO 10 k = 2, nd
            IF (x.GT.Funeta(k,IJ)) GOTO 10
            SETAB = Eta(k,IJ)
     &              - ((Funeta(k,IJ)-x)/(Funeta(k,IJ)-Funeta(k-1,IJ)))
     &              *(Eta(k,IJ)-Eta(k-1,IJ))
            GOTO 20
   10     CONTINUE
          SETAB = Eta(nd,IJ)
        ENDIF
C
      ENDIF
C
   20 RETURN
      END
C
      FUNCTION SETA(IJ,Z,Q,A,R)
C
C  THIS SUBPROGRAM COMPUTES BOTTOM FRICTION AS A FUNCTION OF WATER
C  TEMPERATURE, FLOW DEPTH, DISCHARGE, FROUDE NUMBER, REYNOLDS
C  NUMBER, OR STAGE ACCORDING TO A USER DEFINED LINEAR OR QUADRATIC
C  RELATIONSHIP FOR A BRANCH SEGMENT.  CONSTANT BOTTOM FRICTION IS
C  ALSO ACCOMMODATED.  A QUADRATIC RELATIONSHIP IS DEFINED FOR THE
C  KINEMATIC VISCOSITY AS A FUNCTION OF TEMPERATURE WHICH IS VALID
C  IN THE FAHRENHEIT TEMPERATURE RANGE 32<=T<=95 AND THE CELSIUS
C  TEMPERATURE RANGE 0<=T<=35.  THE KINEMATIC VISCOSITY (NU) IS
C  USED AS 10**6NU = 1 M**2/SEC IN THE METRIC SYSTEM AND AS
C  10**5NU = 1 FT**2/SEC IN THE ENGLISH SYSTEM.  SET TYPETA=1 FOR
C  CONSTANT BOTTOM FRICTION (ETA), 2 FOR ETA AS A FUNCTION OF
C  TEMPERATURE, 3 FOR DEPTH, 4 FOR DISCHARGE, 5 FOR FROUDE NUMBER,
C  6 FOR REYNOLDS NUMBER, OR 7 FOR WATER LEVEL.
C
      REAL nu, Z, Q, A, R, x, SETA
      INTEGER IJ
      INCLUDE 'dimens.cmn'
      INCLUDE 'comcon.cmn'
      INCLUDE 'xsinit.cmn'
      INCLUDE 'ceta.cmn'
      REAL ABS, SQRT
      INTRINSIC ABS, SQRT
C
      SETA = Rn(1,IJ)
      IF (Typeta.NE.1) THEN
        IF (Typeta.EQ.7) THEN
          x = Z
        ELSEIF (Typeta.EQ.3) THEN
          x = R
        ELSEIF (Typeta.EQ.4) THEN
          x = ABS(Q)
        ELSEIF (Typeta.EQ.2) THEN
          x = T(IJ)
        ELSEIF (Typeta.EQ.5) THEN
C
C---VELOCITY = ABS(Q)/A:  X = V / SQRT(G*R)
          x = ABS(Q)/(A*SQRT(G*R))
        ELSEIF (Typeta.EQ.6) THEN
          nu = Ceta1 - (Ceta2-Ceta3*T(IJ))*T(IJ)
C
C---X = V * R / NU
          x = ABS(Q)*R/(A*nu)
        ENDIF
        SETA = SETA + (Rn(2,IJ)+Rn(3,IJ)*x)*x
      ENDIF
      RETURN
      END
