      SUBROUTINE SOLVER(MMM,NSTP,ISS,IBRPRN,KKITER,KKPER,NCOL,NROW,NLAY,
     &                  HNEW,HOLD,IBOUND)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER MMM, NSTP, ISS, IBRPRN, KKITER, KKPER, NCOL, NROW, NLAY
      DOUBLE PRECISION HNEW(NCOL,NROW,NLAY)
      REAL HOLD(NCOL,NROW,NLAY)
      INTEGER IBOUND(NCOL,NROW,NLAY)
C
C     + + + DUMMY ARGUMENT DEFINITIONS + + +
C     MMM    - Number of first time step to solve
C     NSTP   - Number of time steps to solve
C
C     + + + Arguments used in computle MODFLOW/BRANCH simulations + + +
C     ISS    - Steady-state flag
C     IBRPRN - Print flag
C     KKITER - MODFLOW iteration count
C     KKPER  - MODFLOW stress period number
C     NCOL, NROW, NLAY - Number of columns, rows, and layers in MODFLOW grid
C     HNEW, HOLD - Current and previous heads for the MODFLOW grid
C     IBOUND - Boundary array for MODFLOW grid
C
      INCLUDE 'dimens.cmn'
      INCLUDE 'dtcomp.cmn'
      INCLUDE 'matrix.cmn'
      INCLUDE 'branch.cmn'
      INCLUDE 'xsinit.cmn'
      INCLUDE 'cploop.cmn'
      INCLUDE 'modbrn.cmn'
      INCLUDE 'geom.cmn'
      INCLUDE 'ceta.cmn'
      INCLUDE 'comcon.cmn'
      INCLUDE 'untmes.cmn'
      INCLUDE 'units.cmn'
      INCLUDE 'boundy.cmn'
      INCLUDE 'mdatas.cmn'
      INCLUDE 'lunums.cmn'
      INCLUDE 'logics.cmn'
      INCLUDE 'bctime.cmn'
      INCLUDE 'datime.cmn'
      INCLUDE 'wind.cmn'
      INCLUDE 'nodal.cmn'
      INCLUDE 'limits.cmn'
      INCLUDE 'partim.cmn'
C
      DOUBLE PRECISION u(MAXS2), uu(MAXS4), bu(MXBH2), buu(MXBH4)
      DOUBLE PRECISION c1, c2, c3, c4, uuijp1, uuijp2, uuijp3, uuijp4
C
C---CONSTANTS AND COEFFICIENTS
      REAL betcor, thetdx, wratio, rnij, dazpij, dxij, dxiji
C
C---DEPENDENT VARIABLES
      REAL zij, qij, zijp1, qijp1, zp1mz, qp1mq, zp1pz, qp1pq
      REAL zpij, qpij, zpijp1, qpijp1, zpp1mz, qpp1mq, zpp1pz, qpp1pq
      REAL ztemp, qtemp, bigz, bigq, ztol
C
C---QUANTITIES AVERAGED IN SPACE AT CHI IN TIME STEP
      REAL zavg, qavg, ravg, bavg, btavg, aavg, aavgsq, aavgcu, aavgi
C
C---MATRIX COEFFICIENTS
      REAL lambda, sigma, mu, zeta, omega, gamma, delta, epslon, det, 
     &     alpha, tmp
C
C---VARIABLES FOR SCREEN DISPLAY OF EPSILON TERMS
C     REAL ep1, ep2, ep3, ep4, ep5, ep6, ep7, ep8, ep9
C
C---LOCAL VARIABLES
      REAL totc, totm
      INTEGER ijp1, nsm1, jp1, ij2, ij4, ij4p1, ij4p2, ij4p3, ij4p4, 
     &        ij2p1, ij2p2, i2, i4, i4p1, i4p2, i4p3, i4p4, i2p1, i2p2, 
     &        mm, nnn, ibigz, jbigz, ibigq, jbigq, i, j, k, ij
      LOGICAL convrg
C
C---FUNCTIONS AND ROUTINES
      REAL SETA, SETAB
      INTEGER IAR
      EXTERNAL GETWND, DTPREP, ARBALL, AQFRHD, SETAB, DTOUT, DTMAP, 
     &         TRACK, PRTPLT, DAILY, OPLOT, INTBC, GETBVD, GETNDF, 
     &         EXTBC, GEMXP, OPTETA, LEAKVL, SETA, ERRSTP, DTFILE, 
     &         WRTERM, AQRDRY
      INTEGER IFIX, MOD
      REAL ABS, SNGL
      DOUBLE PRECISION DBLE
      INTRINSIC IFIX, ABS, MOD, DBLE, SNGL
C
C     STATEMENT FUNCTION FOR LOCATING ELEMENTS IN COEFFICIENT MATRIX
      IAR(i,j,Ii) = i + Ii*(j-1)
C
C---BEGIN COMPUTATION LOOP
      Icount = 0
      convrg = .TRUE.
      M = MMM
   10 IF (IBRPRN.GT.0 .AND. Khr.EQ.24 .AND. .NOT.Oplots .AND. ISS.EQ.0)
     &    WRITE (*,9005) Kyr, Kmo, Kda, Khr, Kmn
      Lastn = N
      IF (Lastn.GT.Nit) Lastn = Nit
      Kt = Kt + 1
      Icount = Icount + 1
C
C---ASSIGN WIND SPEED AND DIRECTION FROM TIME-DEPENDENT WIND INPUT
      IF (Inwind.NE.0) THEN
        k = IFIX((Nwd-1)*Dt/Wdtt+1.0001)
        IF (Morewd .AND. k.GT.Nwpart) THEN
          CALL GETWND(Prtmsg)
          k = 1
        ENDIF
        wratio = ((Nwd-1)*Dt-(k-1)*Wdtt)/Wdtt
        Wspeed = (Windsp(k)+wratio*(Windsp(k+1)-Windsp(k)))*Wsfact
        Wdirec = Winddr(k) + wratio*(Winddr(k+1)-Winddr(k))
        Cw = Cwfact*Wspeed*Wspeed
CC      cw = WSDRAG*AIRDEN/(H2ODEN*G)*WSPEED*WSPEED
      ENDIF
C
C---CALL ROUTINE TO FILE FLOW RESULTS FOR POST-PROCESS PLOTTING
      IF (Luout.NE.0) CALL DTFILE
C
C---PREPARE FOR NEXT TIME STEP
      CALL DTPREP(Kt)
C
C---WRITE NETCDF FILE
      IF (Otfile.EQ.4) CALL CDFOUT
C
C---BEGIN ITERATIVE IMPROVEMENT LOOP
   20 DO 120 N = 1, Nit
C
C---COMPUTE NEW GEOMETRY
        CALL ARBALL
C
C---IS THIS THE FIRST ITERATION (N=1) OF THIS TIME STEP (M) ?
        IF (N.EQ.1 .AND. .NOT.Modeta) THEN
          IF (IBRPRN.GT.0) THEN
C
C---CALL DTOUT TO PRINT FINAL RESULTS FOR LAST TIME STEP WHEN N EQUALS
C---ONE-- LASTN EQUALS NUMBER OF SOLUTIONS REQUIRED IN LAST TIME STEP
C---(Dtprt IS .TRUE. WHEN IPROPT .EQ. 0, 1, 5, 6, 7, OR 8)
C---(PRINTS TIME-STEP RESULTS WHEN IPROPT = 0, 5, OR 7; ONLY PRINTS
C---DAILY OR FINAL SUMMARY AS NEEDED WHEN IPROPT = 1, 6, OR 8)
            IF (Dtprt .AND. (Lprtdt .OR. MOD(M,Idtrat).EQ.1)) THEN
              CALL DTOUT(0,Lastn,Q,Z,A,B,Bt)
              IF (Lstruc) THEN
                DO 40 i = 1, Nbch
                  ij = MAXS - Xskt(i)
                  nsm1 = Nsec(i) - 1
                  DO 30 j = 1, nsm1
                    ij = ij + 1
                    IF (Istrm(3,ij).LT.0) Rn4(ij) = Rn(3,ij)
   30             CONTINUE
   40           CONTINUE
              ENDIF
            ENDIF
C
C---PRINT NONCONVERGENCE MESSAGE IF NO CONVERGENCE AND IWMOPT 0 OR 2
            IF (.NOT.convrg .AND. Iwmopt.EQ.0 .OR. Iwmopt.EQ.2) THEN
              convrg = .TRUE.
              WRITE (Printr,9010) Khr, Kmn, Kyr, Kmo, Kda, ibigz, jbigz, 
     &                            bigz, ibigq, jbigq, bigq
c             write(*,*) ztol, qtol, zztol, qqtol
            ENDIF
C
C---CALL OUTPUT ROUTINES FOR VARIOUS OPTIONS
            IF (Otfile.EQ.3) CALL DTMAP
            IF (Ipropt.EQ.9) CALL TRACK
            IF (Ptplt) THEN
              IF (Kt.GE.MAXCZQ .OR. Khr.EQ.24) THEN
                Ketime = Ietime + (M-1)*Idtm
                CALL PRTPLT
                Kt = 0
              ENDIF
            ELSEIF (Kt.GE.Idtpdy) THEN
              IF (Daysum) CALL DAILY
              Kt = 0
              IF (Oplots) THEN
                Ketime = Ietime + (M-1)*Idtm
                CALL OPLOT
              ENDIF
            ENDIF
          ENDIF
C
C---IF END OF TIME PERIOD THEN RETURN
          IF (M.EQ.NSTP) GOTO 140
C
C---INCREMENT DATE AND TIME FOR NEXT TIME STEP
C         Kmn = Kmn + Idtm
          Ksc = Ksc + Idts
          Kmn = Ksc/60
          IF (Kmn.GE.60) THEN
            Khr = Khr + Kmn/60
            Kmn = MOD(Kmn,60)
            Ksc = MOD(Ksc,3600)
          ENDIF
          IF (.NOT.(Khr.LT.24 .OR. (Khr.EQ.24 .AND. Kmn.EQ.0))) THEN
            Khr = Khr - 24
            Kda = Kda + 1
          ENDIF
          IF (Kda.GT.Dypmo(Kmo)) THEN
            Kda = Kda - Dypmo(Kmo)
            Kmo = Kmo + 1
          ENDIF
          IF (Kmo.GT.12) THEN
            Kmo = 1
            Kyr = Kyr + 1
            IF (Kyr.GT.99) Kyr = 0
            Dypmo(2) = 28 + (4-MOD(Kyr,4))/4
          ENDIF
C
C---CALL DTOUT TO PRINT INITIAL CONDITIONS FOR START OF NEXT TIME STEP
C---(Noprit IS .TRUE. WHEN IPROPT .NE. 1, 6, AND 8)
C---(ONLY CALLED WHEN IPROPT EQUALS 1, 6, OR 8)
C
          IF (.NOT.Noprit) THEN
            Lastn = N - 1
            IF (Lprtdt .OR. MOD(M+1,Idtrat).EQ.1)
     &          CALL DTOUT(1,Lastn,Qp,Zp,Ap,Bp,Btp)
          ENDIF
C
        ENDIF
C
C---COMPUTE EQUATION COEFFICIENTS AND CONSTRUCT COEFFICIENT MATRICES
        DO 80 i = 1, Nbch
          ij = MAXS - Xskt(i)
          nsm1 = Nsec(i) - 1
C
          DO 70 j = 1, nsm1
            jp1 = j + 1
            ij = ij + 1
            ijp1 = ij + 1
            dxij = Dx(ij)
            dxiji = 1.0/dxij
            thetdx = Theta*dxiji
            qij = Q(ij)
            zij = Z(ij)
            qijp1 = Q(ijp1)
            zijp1 = Z(ijp1)
            zpij = Zp(ij)
            qpij = Qp(ij)
            zpijp1 = Zp(ijp1)
            qpijp1 = Qp(ijp1)
            zp1mz = zijp1 - zij
            zp1pz = zijp1 + zij
            qp1mq = qijp1 - qij
            qp1pq = qijp1 + qij
            zpp1mz = zpijp1 - zpij
            zpp1pz = zpijp1 + zpij
            qpp1mq = qpijp1 - qpij
            qpp1pq = qpijp1 + qpij
            dazpij = Apzpij(ijp1) - Ap(ij)
            bavg = Chihf*(Bp(ij)+Bp(ijp1)) + Chi1hf*(B(ij)+B(ijp1))
            btavg = Chihf*(Btp(ij)+Btp(ijp1)) + Chi1hf*(Bt(ij)+Bt(ijp1))
            aavg = Chihf*(Ap(ij)+Ap(ijp1)) + Chi1hf*(A(ij)+A(ijp1))
            aavgi = 1.0/aavg
            aavgsq = aavgi*aavgi
            ravg = Chihf*(Rp(ij)+Rp(ijp1)) + Chi1hf*(R(ij)+R(ijp1))
            qavg = Chihf*qpp1pq + Chi1hf*qp1pq
            zavg = Chihf*zpp1pz + Chi1hf*zp1pz + Zdatum
            betcor = (Betvel(ij)+Betvel(ijp1))*0.5
            aavgcu = aavgsq*aavgi*betcor*qavg*qavg
C
C---STRUCTURES (PUMPS, GATES, CULVERTS) ARE INCLUDED IN THE MATRIX OF
C---EQUATIONS
            IF (Istrm(3,ij).LT.0) THEN
              CALL STRUC(ij,zpij,zpijp1,qpij,qpijp1,zij,zijp1,qij,qijp1,
     &                   Zdatum,G,IBRPRN,KKPER,NCOL,NROW,NLAY,HOLD,Z,
     &                   Rn(3,ij),epslon,zeta,omega,gamma,alpha,delta,
     &                   det)
              GOTO 60
            ENDIF
C
C---SET FRICTION COEFFICIENT FROM INPUT CONSTANT, RELATION, OR TABLE
            IF (Tabeta .AND. Eta(1,ij).GT.SMLVL) THEN
              Rn4(ij) = SETAB(ij,zavg,qavg,aavg,ravg)
            ELSE
              IF (Typeta.NE.1) Rn4(ij) = SETA(ij,zavg,qavg,aavg,ravg)
            ENDIF
            rnij = Rn4(ij)
C
C---THE MATRIX COEFFICIENTS LAMBDA, SIGMA, MU, EPSLON, ZETA, OMEGA,
C---GAMMA, AND DELTA ARE COMPUTED FOR BRANCH AND ALPHA FOR MODFLOW.
            lambda = dxij*Dt2gth*aavgi
            sigma = ABS(qavg)*rnij*rnij*dxij*Chi2th*aavgsq/ravg**
     &              1.3333333
C
C---THE DRYNESS CONDITION IS CHECKED FOR FRICTION AND LEAKAGE
            IF (Modflow) THEN
              IF (ISS.NE.0) lambda = 0.0
C
C---USE NEW AND OLD AQUIFER HEADS TO DETERMINE HEAD AT THIS TIME STEP
              CALL AQFRHD(ij,NCOL,NROW,NLAY,HNEW,HOLD,IBOUND)
              IF (Itrial(Icount+1,ij).GT.9) sigma = sigma*Dcfm64
              DO 50 k = 3, 8
                IF (Itrial(Icount+1,ij).GE.k*10) sigma = sigma*2.0
   50         CONTINUE
              CALL AQRDRY(ij)
              alpha = dxij*Chi*Clkp*Bp(ij)*Theta2i
              gamma = Chi*dxij*Clkpp1*Bp(ijp1)*Theta2i
              tmp = -Dtheta*qp1mq + 
     &              dxij*(Chi*(Clkp*Haqp*Bp(ij)+Clkpp1*Haqpp1*Bp(ijp1))
     &              +Chi1hf*(Clkij*Haq*B(ij)+Clkp1*Haqp1*B(ijp1)))
     &              *Theta2i
              IF (ISS.EQ.0) THEN
                alpha = alpha + dxij*btavg*Twodtn*Thetai
                gamma = gamma + dxij*btavg*Dt2tha
                delta = tmp + 
     &                  (bavg*dxij*Dt2tha-Chi1hf*dxij*Clkp1*B(ijp1)
     &                  *Theta2i)*Z(ijp1)
     &                  + (bavg*dxij*Dt2tha-Chi1hf*dxij*Clkij*B(ij)
     &                  *Theta2i)*Z(ij)
              ELSE
                delta = tmp - Chi1hf*dxij*Theta2i*(Clkp1*B(ijp1)*Z(ijp1)
     &                  +Clkij*B(ij)*Z(ij))
              ENDIF
            ELSE
              gamma = dxij*btavg*Dt2tha
              delta = gamma*zp1pz - Dtheta*qp1mq + Thetai*dxij*Qlat(ij)
            ENDIF
            mu = betcor*qavg*Ginv2*aavgsq
            epslon = (lambda-Dchi*sigma)*qp1pq - Dtheta*(mu*qp1mq+zp1mz)
     &               + aavgcu*Gtheta*(bavg*zpp1mz+dazpij)
     &               + Cw*bavg*Wangle(ij)*dxij*Thetai*aavgi - 
     &               Denthi*ravg*0.5*(Rho(ijp1)-Rho(ij))
     &               + dxij*Gtheta*aavgi*Qlat(ij)*Ulat(ij)
            zeta = lambda + sigma + mu
            omega = lambda + sigma - mu
            det = 1.0/(1.0-zeta*gamma)
C ==> BEGIN TEMPORARY CODE TO WRITE EPSILON TERMS TO CRT
C
C           ep1 = lambda*qp1pq
C           ep2 = -dchi*sigma*qp1pq
C           ep3 = -dtheta*mu*qp1mq
C           ep4 = -dtheta*zp1mz
C           ep5 = aavgcu*gtheta*bavg*zpp1mz
C           ep6 = aavgcu*gtheta*dazpij
C           ep7 = cw*bavg*Wangle(ij)*dxij*thetai*aavgi
C           ep8 = -denthi*(ravg*0.5)*(rho(ijp1)-rho(ij))
C           ep9 = dxij*gtheta*aavgi*qlat(ij)*ulat(ij)
C           WRITE (*,*) ep1,ep2,ep3,ep4,ep5,ep6,ep7,ep8,ep9
C
C ==> END TEMPORARY CODE TO WRITE EPSILON TERMS TO CRT
C
C---PRINT GOVERNING EQUATION TERMS AT PRINTOUT FREQUENCY (IDTRAT)
C---EVERY TIME STEP AT CONVERGENCE (IPROPT=5) OR FOR EVERY SOLUTION
C---(IPROPT=6)
   60       IF (Ipropt.EQ.5 .OR. Ipropt.EQ.6) THEN
              IF (.NOT.(MOD(M,Idtrat).NE.0 .OR. (Ipropt.EQ.5 .AND. N.NE.
     &            1))) THEN
                IF (N.EQ.1 .AND. i.EQ.1 .AND. j.EQ.1)
     &              WRITE (Printr,9015)
                totc = Dzdt(ij) + Dqdxc(ij) - Qlat(ij)
                totm = Dqdt(ij) + Dqdxm(ij) + Dadx(ij) + Dzdx(ij)
     &                 + Fric(ij) + Wind(ij) + Dpdx(ij) + Qlatm(ij)
                WRITE (Printr,9020) Dzdt(ij), Dqdxc(ij), Qlat(ij), totc, 
     &                              Dqdt(ij), Dqadx(ij), Dzdx(ij), 
     &                              Fric(ij), Wind(ij), Dpdx(ij), 
     &                              Qlatm(ij), totm
C---CALL ROUTINE TO FILE EQUATION TERMS FOR POST-PROCESS PLOTTING
                IF (Luterm.NE.0) CALL WRTERM(i,j,ij)
              ENDIF
              Dzdt(ij) = btavg*Twodtn*(zpp1pz-zp1pz)
              Dqdxc(ij) = thetdx*(qpp1mq+Dtheta*qp1mq)
              Dqdt(ij) = thetdx*lambda*(qpp1pq-qp1pq)
              Dqdxm(ij) = thetdx*mu*(qpp1mq+Dtheta*qp1mq)
              Dadx(ij) = -Ginv*dxiji*aavgcu*(bavg*zpp1mz+dazpij)
              Dqadx(ij) = Dqdxm(ij) + Dadx(ij)
              Dzdx(ij) = thetdx*(zpp1mz+Dtheta*zp1mz)
              Fric(ij) = thetdx*sigma*(qpp1pq+Dchi*qp1pq)
              Wind(ij) = Cw*bavg*Wangle(ij)*aavgi
              Dpdx(ij) = H2odi*(ravg*0.5)*(Rho(ijp1)-Rho(ij))*dxiji
              Qlatm(ij) = Qlat(ij)*Ulat(ij)*Ginv*aavgi
C
C---PRINT VARIABLES AND COEFFICIENTS AT PRINTOUT FREQUENCY (IDTRAT)
C---EVERY TIME STEP AT CONVERGENCE (IPROPT=7) OR FOR EVERY SOLUTION
C---(IPROPT=8)
C
            ELSEIF (Ipropt.EQ.7 .OR. Ipropt.EQ.8) THEN
              IF (.NOT.(MOD(M,Idtrat).NE.0 .OR. (Ipropt.EQ.7 .AND. N.NE.
     &            1))) THEN
                WRITE (Printr,9025) N, i, j, ij, jp1, ijp1
                WRITE (Printr,9030) zij, qij, zijp1, qijp1, zpij, qpij, 
     &                              zpijp1, qpijp1
                WRITE (Printr,9030) Apzpij(ijp1), A(ij), B(ij), R(ij), 
     &                              A(ijp1), B(ijp1), R(ijp1)
                WRITE (Printr,9030) Ap(ij), Bp(ij), Rp(ij), Ap(ijp1), 
     &                              Bp(ijp1), Rp(ijp1)
                WRITE (Printr,9030) dxij, bavg, aavg, ravg, qavg, 
     &                              betcor, rnij, Wangle(ij)
                WRITE (Printr,9030) lambda, sigma, mu, epslon, zeta, 
     &                              omega, gamma, delta
              ENDIF
            ENDIF
C
C---SEGMENT MATRIX COMPUTATION
            ij2 = (ij-1)*2
            ij2p1 = ij2 + 1
            ij2p2 = ij2 + 2
            ij4 = ij2*2
            ij4p1 = ij4 + 1
            ij4p2 = ij4 + 2
            ij4p3 = ij4 + 3
            ij4p4 = ij4 + 4
            i2 = (i-1)*2
            i4 = i2*2
            i4p1 = i4 + 1
            i4p2 = i4 + 2
            i4p3 = i4 + 3
            i4p4 = i4 + 4
            i2p1 = i2 + 1
            i2p2 = i2 + 2
            IF (Modflow) THEN
              uu(ij4p1) = DBLE((1.0+zeta*alpha)*det)
              uu(ij4p3) = DBLE((-alpha-gamma)*det)
            ELSE
              uu(ij4p1) = DBLE((1.0+zeta*gamma)*det)
              uu(ij4p3) = DBLE((-2.0*gamma)*det)
            ENDIF
            uu(ij4p2) = DBLE((-omega-zeta)*det)
            uu(ij4p4) = DBLE((1.0+omega*gamma)*det)
            u(ij2p1) = DBLE((epslon-zeta*delta)*det)
            u(ij2p2) = DBLE((delta-epslon*gamma)*det)
C
C---BRANCH MATRIX COMPUTATION
            IF (j.GT.1) THEN
              c1 = buu(i4p1)
              c2 = buu(i4p2)
              c3 = buu(i4p3)
              c4 = buu(i4p4)
              uuijp1 = uu(ij4p1)
              uuijp2 = uu(ij4p2)
              uuijp3 = uu(ij4p3)
              uuijp4 = uu(ij4p4)
              buu(i4p1) = uuijp1*c1 + uuijp2*c3
              buu(i4p2) = uuijp1*c2 + uuijp2*c4
              buu(i4p3) = uuijp3*c1 + uuijp4*c3
              buu(i4p4) = uuijp3*c2 + uuijp4*c4
              c1 = bu(i2p1)
              c2 = bu(i2p2)
              bu(i2p1) = uuijp1*c1 + uuijp2*c2 + u(ij2p1)
              bu(i2p2) = uuijp3*c1 + uuijp4*c2 + u(ij2p2)
            ELSE
              buu(i4p1) = uu(ij4p1)
              buu(i4p2) = uu(ij4p2)
              buu(i4p3) = uu(ij4p3)
              buu(i4p4) = uu(ij4p4)
              bu(i2p1) = u(ij2p1)
              bu(i2p2) = u(ij2p2)
            ENDIF
   70     CONTINUE
   80   CONTINUE
C
C---SET UP NETWORK MATRIX AND VECTOR
        Nn = 1
        mm = 1
        DO 90 i = 1, Nbch
C
C---INSERT BRANCH MATRICES
          nnn = IAR(Nn,mm,Ii)
          i2 = (i-1)*2
          i4 = i2*2
          Am(nnn) = SNGL(buu(i4+1))
          Am(nnn+Ii) = SNGL(buu(i4+2))
          Am(nnn+Ii2) = -1.0
          nnn = IAR(Nn+1,mm,Ii)
          Am(nnn) = SNGL(buu(i4+3))
          Am(nnn+Ii) = SNGL(buu(i4+4))
          Am(nnn+Ii3) = -1.0
C
C---CONSTRUCT RIGHT SIDE VECTOR
          Bmx(Nn) = SNGL(-bu(i2+1))
          Bmx(Nn+1) = SNGL(-bu(i2+2))
          Nn = Nn + 2
          mm = mm + 4
   90   CONTINUE
C
C---INSERT BOUNDARY CONDITIONS FOR INTERNAL JUNCTIONS
        CALL INTBC
C
C---RETRIEVE ADDITIONAL BOUNDARY-VALUE DATA FROM DATA BASE STORAGE
        IF (N.EQ.1 .AND. .NOT.Modeta) THEN
          IF (Morebd) THEN
            k = IFIX(Nd*Dt/Dtt(1)+1.0001)
            IF (k.GT.Ndpart) THEN
              CALL GETBVD
              Nd = IFIX(Dtmax/Dt+0.0001)
            ENDIF
          ENDIF
C
C---RETRIEVE ADDITIONAL NODAL FLOW DATA FROM DATA BASE STORAGE
          IF (Morenf) THEN
            k = IFIX(Nnd*Dt/Dttn(1)+1.0001)
            IF (k.GT.Nnpart) THEN
              CALL GETNDF(Dt,Prtmsg)
              Nnd = IFIX(Dtmxnf/Dt+0.0001)
            ENDIF
          ENDIF
        ENDIF
C
C---INSERT BOUNDARY CONDITIONS FOR EXTERNAL JUNCTIONS
        CALL EXTBC
C
C---CHECK FOR SQUARE MATRIX
        IF (Ii.NE.Nn-1) CALL ERRSTP(
     &'MATRIX NOT SQUARE: REVIEW SCHEMATIZATION AND EXTERNAL B.C. SPECIF
     &ICATIONS')
C
C---SOLVE EQUATION MATRICES
        CALL GEMXP
        Ktmats = Ktmats + 1
C
C---COMPUTE INTERMEDIATE VALUES AT SEGMENT ENDS
        Nn = 1
        bigq = 0.0
        bigz = 0.0
        DO 110 i = 1, Nbch
          ij = MAXS - Xskt(i)
          ijp1 = ij + 1
          ztemp = Zp(ijp1)
          qtemp = Qp(ijp1)
          Zp(ijp1) = Bmx(Nn)
          Qp(ijp1) = Bmx(Nn+1)
          ztol = ABS(ztemp-Zp(ijp1))
          Qtol = ABS(qtemp-Qp(ijp1))
c         write(*,*) ztol,bigz, qtol,bigq
          IF (ztol.GT.bigz) THEN
            bigz = ztol
            ibigz = i
            jbigz = 1
          ENDIF
          IF (Qtol.GT.bigq) THEN
            bigq = Qtol
            ibigq = i
            jbigq = 1
          ENDIF
          Nn = Nn + 4
          nsm1 = Nsec(i) - 1
          DO 100 j = 1, nsm1
            ij = ij + 1
            ijp1 = ij + 1
            ij2 = (ij-1)*2
            ij4 = (ij-1)*4
            ztemp = Zp(ijp1)
            qtemp = Qp(ijp1)
            Zp(ijp1) = SNGL(uu(ij4+1)*DBLE(Zp(ij))+uu(ij4+2)*DBLE(Qp(ij)
     &                 )+u(ij2+1))
            Qp(ijp1) = SNGL(uu(ij4+3)*DBLE(Zp(ij))+uu(ij4+4)*DBLE(Qp(ij)
     &                 )+u(ij2+2))
            IF (Zp(ijp1).GT.Za(Ipt(ijp1),ijp1)) Zp(ijp1)
     &          = Za(Ipt(ijp1),ijp1)
            IF (Zp(ijp1).LT.Za(1,ijp1)) Zp(ijp1) = Za(1,ijp1)
            ztol = ABS(ztemp-Zp(ijp1))
            Qtol = ABS(qtemp-Qp(ijp1))
            IF (ztol.GT.bigz) THEN
              bigz = ztol
              ibigz = i
              jbigz = j + 1
            ENDIF
            IF (Qtol.GT.bigq) THEN
              bigq = Qtol
              ibigq = i
              jbigq = j + 1
            ENDIF
  100     CONTINUE
  110   CONTINUE
C
C---CHECK FOR CONVERGENCE
        IF (bigz.LE.Zztol .AND. bigq.LE.Qqtol) GOTO 130
C
C---CALL DTOUT TO PRINT RESULTS FOR LAST SOLUTION IF NOT CONVERGED
C---(Noprit IS .TRUE. WHEN IPROPT .NE. 1, 6, AND 8)
        IF (.NOT.Noprit .AND. N.NE.Nit .AND. 
     &      (Lprtdt .OR. MOD(M+1,Idtrat).EQ.1))
     &      CALL DTOUT(1,N,Qp,Zp,Ap,Bp,Btp)
  120 CONTINUE
C
C---END ITERATIVE IMPROVEMENT LOOP
      IF (.NOT.Noconv) THEN
        IF (Iunit.EQ.En) THEN
          bigz = bigz*0.3048
          bigq = bigq*0.02832
        ELSE
          bigz = bigz*3.281
          bigq = bigq*35.31
        ENDIF
      ENDIF
      convrg = .FALSE.
  130 IF (ABS(Tolerr).GT.SMLVL) THEN
        CALL OPTETA
        IF (Modeta) GOTO 20
      ENDIF
      Nd = Nd + 1
      Nwd = Nwd + 1
      Nnd = Nnd + 1
      IF (Modflow) CALL LEAKVL(NCOL,NROW,NLAY,HNEW,HOLD,IBOUND,KKITER)
C
C---END COMPUTATION LOOP
      M = M + 1
      IF (M.LE.NSTP) GOTO 10
  140 RETURN
C
C---INPUT/OUTPUT FORMAT STATEMENTS
 9005 FORMAT (' SIMULATION TIME =',I3.2,2('/',I2.2),I3.2,':',I2.2)
 9010 FORMAT (
     &  ' *WARNING* MAXIMUM ITERATIONS COMPLETED WITHOUT CONVERGENCE AT'
     &  ,I3.2,':',I2.2,' ON',I3.2,2('/',I2.2),5X,'Z-ZP(',I2,',',I2,')=',
     &  F12.5,' Q-QP(',I2,',',I2,')=',F12.5)
 9015 FORMAT ('   dZ/dt      dQ/dx        q      MASS TOTAL   dQ/dt   ',
     & 'd(Q*Q/A)/dx   dZ/dx       Sf         Sw        dp/dx       qu  '
     & ,'    MOM. TOTAL')
 9020 FORMAT (12E11.5E1)
 9025 FORMAT (1X,8I16)
 9030 FORMAT (1X,8E16.8)
      END
