      PROGRAM TRKPLOT
C ##############################################################################
C #                        PLOTTING OF PARTICLE TRACKS                         #
C #                              VERSION 95/08/25                              #
C #                     BY R. W. SCHAFFRANEK, R. S. REGAN                      #
C ##############################################################################
C #                                                                            #
C #    THIS PROGRAM PRODUCES A PLOT FILE OF THE CONSTITUENT MOVEMENT           #
C #    THROUGH, AND BOUNDARY CONDITIONS OF, A RIVER REACH USING THE FLOW       #
C #    RESULTS COMPUTED BY THE ONE-DIMENSIONAL BRANCH-NETWORK FLOW MODEL.      #
C #                                                                            #
C #============================================================================#
C # VARIABLE             DEFINITION              FORMAT VALUES  DEFAULT COLUMN #
C #============================================================================#
C #                                                                            #
C #----------------------------------------------------------------------------#
C # PLOTTING OPTIONS CARD (ONE REQUIRED PER EXECUTION):                        #
C #----------------------------------------------------------------------------#
C # PLTTYP    PLOT STYLE - COMIC (READ LEFT TO    A4 'MOVI'/'COMI' MOVIE  1-4  #
C #           RIGHT) MOVIE (READ TOP TO BOTTOM).                               #
C # PENTYP    NO. OF PLOT PENS - PAPER(2) PEN 1=  A4 'SLID'/'PAPE' SLIDE  7-10 #
C #           BOLD, PEN 2=FINE; SLIDE(4) PEN 1=                                #
C #           BLACK, PEN 2=BLUE, PEN 3=GREEN,                                  #
C #           PEN 4=RED                                                        #
C # SIZE      PLOT LETTER SIZE (IN INCHES)       F6.3    -----     0.12  11-16 #
C # ONELAB    LABEL WITH (ME) OR (EN) UNITS ONLY  A2   'ME'/'EN'   BOTH  18-19 #
C #           OR BOTH (BO)                               'BO'                  #
C # PLTFAC    SCALING FACTOR TO SET SIZE OF PLOT F5.3    -----     0.85  20-24 #
C #           (DEFAULT PRODUCES 8.5 X 11 INCH SIZE)                            #
C # IXPATH    PATHS TO EXCLUDE FROM PLOTTING     10I2   0-MXPATH      0  25-45 #
C #
C # EXAMPLE:                                                                   #
C # COMIC PAPER      ME 1.25116.8                                              #
C #
C #----------------------------------------------------------------------------#
C # PATH CONTROL RECORD (ONE PER PATH, OPTIONAL):
C #----------------------------------------------------------------------------#
C # NUMPATH   PATH NUMBER TO USE VALUE FOR        I2    0-MXPATH   current 1-2 #
C # LOCPRMPT  OPTION TO PROMPT FOR UPSTREAM,      I1     0/1          0      4 #
C #           DOWNSTREAM AND LABEL NAMES AND
C #           LOCATIONS (0=yes, 1=no)
C # UPRVML    RIVER REFERENCE MILE OF UPSTREAM   F8.2    -----     0.0    5-12 #
C #           BOUNDARY LOCATION                                                #
C # EXPART    LETERS OF PARTICLES FOR EXCLUSION  10A2  'A '-'J '   ---   13-31 #
C #           FROM PLOT                                                        #
C #
C # EXAMPLE:                                                                   #
C #  1 1 116.8 C E
C #
C #----------------------------------------------------------------------------#
C # BOUNDARY CONDITION LOCATION CARD (ONE PER PATH, OPTIONAL):
C # Defaults to the BRNAME as defined on the BRANCH IDENTIFICATION RECORDS     #
C # from the BRANCH model                                                      #
C #----------------------------------------------------------------------------#
C # UPLOCN    UPSTREAM BOUNDARY CONDITION NAME    A16    -----     ---    1-16 #
C # DNLOCN    DOWNSTREAM BOUNDARY CONDITION NAME  A16    -----     ---   17-32 #
C # NLABEL    NUMBER OF LABEL LOCATIONS            I2    0-MAXLOC   0    33-34 #
C #                                                                            #
C # EXAMPLE:                                                                   #
C # CHAIN BRIDGE    INDIAN HEAD 3                                              #
C #                                                                            #
C #----------------------------------------------------------------------------#
C # LABEL LOCATION NAME CARDS (0 TO 10 OPTIONAL PER PATH):
C #----------------------------------------------------------------------------#
C # RMLOC     RIVER MILE DISTANCE FROM UPSTREAM  F8.2    -----     ---    1-8  #
C #           BOUNDARY                                                         #
C # LABEL     LOCATION NAME                      60A1    -----     ---   10-69 #
C #                                                                            #
C # EXAMPLE DECK FOR TRACKPLT EXECUTION WITH FOUR PATHS IN THE BRANCH FILE
C #
C # MOVIE SLID 0.150 BO0.800 0 0 0 0 0
C #  1 1    0.00 C E
C # CHAIN BRIDGE    INDIAN HEAD, MD  6
C #     0.00 Chain Bridge                                                
C #     4.85 Memorial Bridge                                             
C #    12.13 Wilson Bridge                                               
C #    20.16 Mount Vernon                                                
C #    26.26 Hallowing Point                                             
C #    30.83 Indian Head                                                 
C #  2 1    0.00  
C # E/W ANACOSTIA   INDIAN HEAD, MD  9
C #     0.00 Anacostia E/W
C #     3.00 Lake Kingman US
C #     4.99 Lake Kingman DS
C #     6.42 11th St. Bridge
C #     8.38 Greenleaf Point
C #     9.01 Hains Point
C #    11.33 Marbury Point
C #    12.91 Wilson Bridge
C #    31.60 Indian Head
C #  3 1    0.00
C # POHICK BAY      INDIAN HEAD, MD  0
C #  4 1    0.00
C # WASHINGTON CHAN.INDIAN HEAD, MD  0
C
      INCLUDE 'dimens.cmn'
      INTEGER MAXLOC
      PARAMETER (MAXLOC=10)
      REAL rm(MAXCZQ,MXPART), zq(MAXCZQ,2), y(MAXCZQ), fac(3), one, 
     &     rmloc(MAXLOC,MXPATH), angle, size, pltfac, uprvml(MXPATH), 
     &     usmn, usmx, dsmn, dsmx, mid
      REAL zqusmn, zqusmx, zqdsmn, zqdsmx, cnvi, sizeq, szelbl(MXPATH), 
     &     uscnv, dscnv, uscnvi, dscnvi, zqus, zqds, ystep, rchlen, 
     &     tenpwr, tx, yscl, yscli, xlen, outchk, y1, onezer, half, 
     &     anghf, szx3, shftmv, shift, tmstep, rkm, te, tm, tde, tdm, 
     &     ypos, xpos, tic, t, pos, off, yshft, sgn, six0, ylast, 
     &     zqumax, zqumin, zqinch, zqdmax, zqdmin, tsize, xp, yp, ylen, 
     &     dxtot(MXPATH), usinch, dsinch, ang
      INTEGER reader, printr, indata, nchar(MAXLOC), pltunt, ktpath, 
     &        ktpart(MXPATH), kt, nstpsv, numpath, locprmpt, ios
      INTEGER iyr, imo, idy, i, nmax, iarayn(MXPATH), j, nlabel(MXPATH), 
     &        n, idtm, idelt, idigit, maxhrs, nchr, nc, nsteps, n1, n2, 
     &        pen1, pen2, pen3, pen4, mxhrsv, nchrus(MXPART), 
     &        nchrds(MXPART), npart, date(10), ixpath(MXPATH)
      LOGICAL multpt, movie, iend, mlabel, slides, prtlab, enonly, 
     &        meonly, newfile, yes
      CHARACTER*40 brnmus(MXPATH), brnmds(MXPATH), ctmp, msg
      CHARACTER*16 uplocn(MXPATH), dnlocn(MXPATH), blk16, uploc, dnloc
      CHARACTER*4 dxut, dxm, plttyp, pentyp, movi, slide, blank
      CHARACTER*2 letter(MXPART), inunit, itbcus, itbcds, ztype, qtype, 
     &            iunit, expart(MXPART), onelab, me, en, blnk, both
      CHARACTER*1 label(60,10,MXPATH), blk
      CHARACTER symlbl*60, clbl(MAXLOC,MXPATH)*60, netnam*24, lbl6*6, 
     &          versn*72, dashes*60, lbl*60, xprt*24
      EQUIVALENCE (clbl(1,1),label(1,1,1)), (xprt,expart(1))
      EXTERNAL SYSDATE, TIOFIL, CHKLOC, GETCVL, PLOTS, FACTOR, NEWPEN, 
     &         PLOT, SYMBOL, NUMBER, PLTBAX, TRSCAL, LBLNUM, AXLABL, 
     &         NXTPLT, LFTJUS, GETRVL, GETIVL, GETYES, PGMCTL
      INTEGER IFIX, MIN0
      REAL ALOG10, ABS, AMOD, AMAX1
      INTRINSIC IFIX, MIN0, ALOG10, ABS, AMOD, AMAX1
C
      DATA slide/'SLID'/, movi/'MOVI'/, me/'ME'/, en/'EN'/, both/'BO'/
      DATA blnk/'  '/, blank/'    '/, ylen/6.0/
C     set pen variables to appropriate values for device
C     pen1=2 for postscript on prime
      DATA pen1/2/, pen2/2/, pen3/3/, pen4/4/
      DATA blk/' '/, blk16/'                '/
      DATA reader/55/, printr/66/, indata/67/, pltunt/9/
      DATA letter/'A ', 'B ', 'C ', 'D ', 'E ', 'F ', 'G ', 'H ', 'I ', 
     &     'J '/, inunit/'ME'/, ztype/' Z'/, qtype/' Q'/, iunit/'EN'/
      DATA dxut/'(FT)'/, dxm/'(M) '/, multpt/.FALSE./, iend/.FALSE./
      DATA dashes/
     &    '------------------------------------------------------------'
     &    /
C
      versn = 
     &   '@(#)TRKPLOT - Postprocessor for BRANCH to plot particle paths'
      versn = '@(#)TRKPLOT - Contact: h2osoft@usgs.gov'
      versn = '@(#)TRKPLOT - Version: 4.1x 1997/03/05'
      versn = 
     &  '$Id: trkplot.f,v 4.1 1997/03/05 18:03:41 rsregan Exp rsregan $'
C
C---versn(1:58) used in printed output
      lbl = 'Particle Track Plotting Program - Version: '//versn(18:32)
      versn = lbl
      CALL SYSDATE(iyr,imo,idy)
      cnvi = 1.0/1.6093
      CALL TIOFIL(reader,printr,indata,pltunt,versn,newfile)
C
C---READ FIRST TWO RECORDS OF BRANCH OUTPUT FILE
      READ (indata) netnam
      READ (indata) date, ktpath
      IF (ktpath.GT.MXPATH) THEN
        WRITE (*,*) 
     &'*Error* exceeded maximum number of paths allowed, increase MXPATH
     & in dimens.cmn'
        STOP
      ENDIF
      IF (.NOT.newfile) THEN
        msg = 'USE THE PROGRAM-CONTROL FILE AS IS'
        CALL GETYES(msg,yes)
      ELSE
        yes = .FALSE.
      ENDIF
      IF (.NOT.yes) CALL PGMCTL(reader,ktpath)
C
      READ (reader,9010,ERR=530) plttyp, pentyp, size, onelab, pltfac, 
     &                           ixpath
      IF (plttyp.EQ.blank) plttyp = movi
      IF (pentyp.EQ.blank) pentyp = slide
      IF (onelab.EQ.blnk) onelab = both
      IF (size.EQ.0.0) size = 0.12
      sizeq = size*0.75
      IF (pltfac.EQ.0.0) pltfac = 0.85
      movie = plttyp.EQ.movi
      slides = pentyp.EQ.slide
      enonly = onelab.EQ.en
      meonly = onelab.EQ.me
      WRITE (printr,9015) versn(:58), plttyp, pentyp, size, onelab, 
     &                    pltfac, MXPATH, ixpath
C
      READ (indata) ktpart(1), nsteps, idtm, dxtot(1), itbcus, itbcds, 
     &              iunit, inunit, brnmus(1), brnmds(1)
      DO 10 kt = 2, ktpath
        READ (indata) ktpart(kt), dxtot(kt), brnmus(kt), brnmds(kt)
   10 CONTINUE
      IF (inunit.EQ.blnk) inunit = en
      IF (iunit.EQ.blnk) iunit = en
      IF (inunit.EQ.me) dxut = dxm
C
      DO 80 kt = 1, ktpath
C
C---Left justify upstream and downstream locations
        nc = 40
        ctmp = brnmus(kt)
        CALL CHKLOC(ctmp,nc)
        brnmus(kt) = ctmp
        nc = 40
        ctmp = brnmds(kt)
        CALL CHKLOC(ctmp,nc)
        brnmds(kt) = ctmp
C
C---Read first record of path data
        READ (reader,9095,IOSTAT=ios,END=20) numpath, locprmpt, 
     &        uprvml(kt), expart
        IF (ios.NE.0) GOTO 540
        IF (numpath.EQ.0) numpath = kt
        IF (numpath.GT.kt) THEN
          BACKSPACE reader
          GOTO 20
        ELSEIF (numpath.LT.kt) THEN
          GOTO 540
        ENDIF
        GOTO 30
   20   locprmpt = 0
        xprt = ' '
        uprvml(kt) = 0.0
C
C---Read second record of path data (locations)
   30   IF (locprmpt.EQ.1) THEN
          READ (reader,9020,END=540) uploc, dnloc, nlabel(kt)
        ELSE
          uploc = brnmus(kt)(1:16)
          dnloc = brnmds(kt)(1:16)
          DO 40 i = 1, ktpath
            IF (ixpath(i).EQ.kt) GOTO 50
   40     CONTINUE
          WRITE (symlbl,9030) 'UPSTREAM', kt
          CALL GETCVL(symlbl,uploc)
          WRITE (symlbl,9030) 'DOWNSTREAM', kt
          CALL GETCVL(symlbl,dnloc)
          nlabel(kt) = 0
          WRITE (symlbl,9100) 'THE NUMBER OF LABEL LOCATIONS FOR PATH', 
     &                        kt
          CALL GETIVL(symlbl,0,MAXLOC,nlabel(kt))
        ENDIF
   50   IF (nlabel(kt).GT.MAXLOC) THEN
          WRITE (printr,*) '*ERROR* Only', MAXLOC, 
     &                     ' label locations can be input per path'
          GOTO 550
        ENDIF
        IF (uploc.EQ.blk16 .OR. uploc.EQ.'upstream bndry  ')
     &      uploc = brnmus(kt)(1:16)
        IF (dnloc.EQ.blk16 .OR. dnloc.EQ.'downstream bndry')
     &      dnloc = brnmds(kt)(1:16)
        nchrus(kt) = 16
        nchrds(kt) = 16
        CALL CHKLOC(uploc,nchrus(kt))
        CALL CHKLOC(dnloc,nchrds(kt))
        WRITE (printr,9025) kt, uploc, dnloc, nlabel(kt)
        uplocn(kt) = uploc
        dnlocn(kt) = dnloc
C
C---Read third records of path data (label locations)
        IF (nlabel(kt).NE.0) THEN
          WRITE (printr,9035) kt
          DO 70 i = 1, nlabel(kt)
            IF (locprmpt.EQ.1) THEN
              READ (reader,9040,END=80) rmloc(i,kt), clbl(i,kt)
            ELSE
              DO 60 j = 1, ktpath
                IF (ixpath(j).EQ.kt) GOTO 70
   60         CONTINUE
              WRITE (symlbl,9100) 'RIVER MILE FOR LABEL LOCATION', i
              rmloc(i,kt) = 0.0
              CALL GETRVL(symlbl,2,-9999.0,9999.0,rmloc(i,kt))
              symlbl = 'LABEL LOCATION NAME'
              clbl(i,kt) = uploc
              CALL GETCVL(symlbl,clbl(i,kt))
            ENDIF
            WRITE (printr,9045) rmloc(i,kt), clbl(i,kt)
   70     CONTINUE
        ENDIF
   80 CONTINUE
C
C---DETERMINE LENGTH AND SIZE OF LABELING
      nmax = 0
      DO 110 kt = 1, ktpath
        iarayn(kt) = IFIX(ylen/size+0.5)
        IF (iarayn(kt).GT.60) iarayn(kt) = 60
        IF (nlabel(kt).NE.0) THEN
          DO 90 i = 1, nlabel(kt)
C
C---left justify label
            nchar(i) = 60
            CALL LFTJUS(clbl(i,kt),nchar(i))
            IF (nchar(i).GT.nmax) nmax = nchar(i)
   90     CONTINUE
          szelbl(kt) = size
          IF (nmax.GT.iarayn(kt)) iarayn(kt) = nmax + 6
          IF (iarayn(kt).GT.60) iarayn(kt) = 60
          IF (nmax.GT.iarayn(kt)) szelbl(kt) = ylen/iarayn(kt)
          DO 100 i = 1, nlabel(kt)
            n = nchar(i)
            IF (n.EQ.0) THEN
              clbl(i,kt) = dashes
            ELSE
              lbl = clbl(i,kt)(1:n)
              n1 = (iarayn(kt)-n)/2
              n2 = iarayn(kt) - n - n1
              clbl(i,kt) = dashes(1:n1)//lbl(1:n)//dashes(1:n2)
            ENDIF
  100     CONTINUE
        ENDIF
  110 CONTINUE
C
      j = 11
      DO 120 i = MXPART, 1, -1
        j = i
        IF (expart(j).NE.blnk) GOTO 130
  120 CONTINUE
  130 npart = j
      nsteps = nsteps - 1
      IF (itbcus.EQ.ztype) THEN
        uscnv = 0.3048006
      ELSE
        uscnv = 0.0283168
      ENDIF
      IF (itbcds.EQ.qtype) THEN
        dscnv = 0.0283168
      ELSE
        dscnv = 0.3048006
      ENDIF
      uscnvi = 1.0/uscnv
      dscnvi = 1.0/dscnv
      WRITE (printr,9050) netnam, date, ktpath, nsteps, idtm, iunit, 
     &                    inunit
      WRITE (printr,9055) (kt,ktpart(kt),dxut,dxtot(kt),kt=1,ktpath)
      WRITE (printr,9060)
      IF (inunit.EQ.me) THEN
        DO 140 kt = 1, ktpath
          dxtot(kt) = dxtot(kt)*3.280833
  140   CONTINUE
      ENDIF
      mxhrsv = (nsteps*idtm)/60
      nstpsv = nsteps
      idelt = 30
      IF (mxhrsv.LT.61) THEN
        idelt = 10
      ELSEIF (mxhrsv.LT.121) THEN
        idelt = 20
      ENDIF
      ystep = idtm/60./idelt
      CALL PLOTS(-pltunt,1,0)
      CALL FACTOR(pltfac)
      CALL NEWPEN(pen1)
      IF (movie) THEN
        angle = 180.0
        one = -1.0
        y1 = -0.05
        onezer = 1.0
      ELSE
        y1 = -size - 0.1
        angle = 0.0
        one = 1.0
        onezer = 0.0
      ENDIF
      half = size*0.5
      anghf = angle*0.5
      szx3 = size*3.0
      shftmv = onezer*szx3
      shift = size*one
      n1 = MIN0(nsteps,MAXCZQ)
      DO 150 i = 1, n1
        y(i) = i*ystep
  150 CONTINUE
      DO 520 kt = 1, ktpath
        DO 160 i = 1, ktpath
          IF (ixpath(i).EQ.kt) GOTO 520
  160   CONTINUE
        mlabel = uprvml(kt).EQ.0.0
        tmstep = 0.0
        iend = .FALSE.
        maxhrs = mxhrsv
        nsteps = nstpsv
C
C---SET UP CONSTANT SCALING FOR BOUNDARY-CONDITION DATA
        zqusmn = BIGPOS
        zqusmx = BIGNEG
        zqdsmn = BIGPOS
        zqdsmx = BIGNEG
        DO 190 i = 1, nsteps
          DO 170 j = 1, kt - 1
            READ (indata,END=200)
  170     CONTINUE
          READ (indata,END=200) zqus, zqds
          DO 180 j = kt + 1, ktpath
            READ (indata,END=200)
  180     CONTINUE
          IF (zqus.LT.zqusmn) zqusmn = zqus
          IF (zqus.GT.zqusmx) zqusmx = zqus
          IF (zqds.LT.zqdsmn) zqdsmn = zqds
          IF (zqds.GT.zqdsmx) zqdsmx = zqds
  190   CONTINUE
  200   REWIND indata
        DO 210 j = 1, ktpath + 2
          READ (indata)
  210   CONTINUE
        IF (iunit.EQ.me) THEN
          zqusmn = zqusmn*uscnvi
          zqusmx = zqusmx*uscnvi
          zqdsmn = zqdsmn*dscnvi
          zqdsmx = zqdsmx*dscnvi
        ENDIF
        IF (ABS(zqusmn-zqusmx).LT.0.0005) THEN
          j = 10**IFIX(ALOG10(ABS(zqusmx))+0.0005)/5
          zqusmn = zqusmn - j
          zqusmx = zqusmx + j
        ELSE
          t = (zqdsmx-zqdsmn)*0.025
          zqdsmn = zqdsmn - t
          zqdsmx = zqdsmx + t
          t = (zqusmx-zqusmn)*0.025
          zqusmn = zqusmn - t
          zqusmx = zqusmx + t
        ENDIF
        IF (zqusmn.GT.-0.5 .AND. zqusmx.LT.0.5) THEN
          zqusmn = -10.0
          zqusmx = 10.0
        ENDIF
        IF (zqdsmn.GT.-0.5 .AND. zqusmx.LT.0.5) THEN
          zqdsmn = -10.0
          zqdsmx = 10.0
        ENDIF
        usinch = 1.0/(zqusmx-zqusmn)
        dsinch = 1.0/(zqdsmx-zqdsmn)
C
        IF (maxhrs.GT.120) nsteps = MIN0(nsteps,MAXCZQ)
        IF (movie) THEN
          CALL PLOT(2.1,3.1,-3)
        ELSE
          CALL PLOT(2.4,3.4,-3)
        ENDIF
        rchlen = dxtot(kt)/5280.0
        tenpwr = 10.00**(IFIX(ALOG10(rchlen))-2)
        fac(1) = tenpwr
        fac(2) = tenpwr*2.0
        fac(3) = tenpwr*5.0
  220   DO 230 i = 1, 3
          tx = rchlen/fac(i)
          IF (tx.GT.7.0) THEN
            fac(i) = fac(i)*10.0
          ELSE
            yscl = fac(i)
            yscli = 1.000/yscl
            xlen = rchlen*yscli
            outchk = xlen - xlen*0.000001
            rkm = uprvml(kt)*1.6093
            te = IFIX(uprvml(kt)*yscli)*yscl
            tm = IFIX(rkm*yscli)*yscl
            tde = (uprvml(kt)-te)*yscli
            tdm = (rkm-tm)*cnvi*yscli
            GOTO 240
          ENDIF
  230   CONTINUE
        GOTO 220
C
  240   IF (maxhrs.GT.180) multpt = .TRUE.
C
C---READ IN BOUNDARY VALUE DATA AND DISTANCES, CONVERT TO ENGLISH UNITS
C---AND LOAD ARRAYS, AND DETERMINE MAX AND MIN FOR SINGLE FRAME PLOTS.
        DO 280 i = 1, nsteps
          DO 250 j = 1, kt - 1
            READ (indata,END=290)
  250     CONTINUE
          READ (indata,END=290) zq(i,1), zq(i,2), 
     &                          (rm(i,j),j=1,ktpart(kt))
          DO 260 j = kt + 1, ktpath
            READ (indata,END=290)
  260     CONTINUE
          n1 = i
          IF (inunit.NE.en) THEN
            DO 270 j = 1, ktpart(kt)
              rm(i,j) = rm(i,j)*cnvi
  270       CONTINUE
          ENDIF
          IF (iunit.NE.en) THEN
            zq(i,1) = zq(i,1)*uscnvi
            zq(i,2) = zq(i,2)*dscnvi
          ENDIF
  280   CONTINUE
        GOTO 300
  290   nsteps = n1
        iend = .TRUE.
C
C     ******** RIVER MILE DISTANCES PLOTTING SECTION ********
C
  300   ypos = -0.1 - szx3 - half - onezer*(0.05+size)
        IF (mlabel) THEN
          xpos = (xlen-13.0*shift)*0.5
          CALL SYMBOL(xpos,ypos,size,'DISTANCE FROM',angle,13)
          xpos = (xlen-nchrus(kt)*shift)*0.5
          CALL SYMBOL(xpos,ypos-(size+0.05)*one,size,uplocn(kt),angle,
     &                nchrus(kt))
        ENDIF
        IF (meonly) THEN
          IF (mlabel) THEN
            CALL SYMBOL((xlen-13.*shift)*.5,ypos-2.0*(size+.05)*one,
     &                  size,'IN KILOMETERS',angle,13)
          ELSE
            CALL SYMBOL((xlen-24.*shift)*.5,ypos,size,
     &                  'DISTANCE FROM MOUTH (KM)',angle,24)
          ENDIF
          tic = 1.00/1.6093
          CALL PLOT(0.0,0.0,3)
          CALL PLOT(tdm,0.0,2)
          CALL PLOT(0.0,ylen,3)
          CALL PLOT(tdm,ylen,2)
          t = tm
          tx = tdm
          n = IFIX(xlen*1.6093-tdm) + 1
        ELSE
          IF (mlabel) THEN
            CALL SYMBOL((xlen-8.*shift)*.5,ypos-2.0*(size+.05)*one,size,
     &                  'IN MILES',angle,8)
          ELSE
            CALL SYMBOL((xlen-27.*shift)*.5,ypos,size,
     &                  'DISTANCE FROM MOUTH (MILES)',angle,27)
          ENDIF
          tic = 1.0
          n = IFIX(xlen-tde) + 1
          tx = tde
          t = te
          CALL PLOT(0.0,0.0,3)
          CALL PLOT(tde,0.0,2)
          CALL PLOT(0.0,ylen,3)
          CALL PLOT(tde,ylen,2)
        ENDIF
        pos = 0.0 + tx
        off = half
        DO 310 i = 1, n
          off = size
          yshft = 1.0
          IF (t.LT.10.0) off = half
          IF (t.GT.9.0) yshft = 2.0
          IF (t.GT.99.0) THEN
            yshft = 3.0
            off = size + half
          ENDIF
          IF (movie) THEN
            IF (yscl.GT.0.9) THEN
              CALL NUMBER(pos+half,y1-size*yshft,size,t,90.0,-1)
            ELSE
              CALL NUMBER(pos+half,y1-szx3,size,t,90.0,1)
            ENDIF
          ELSEIF (yscl.LT.0.9) THEN
            CALL NUMBER(pos-1.5*size,y1,size,t,0.0,1)
          ELSE
            CALL NUMBER(pos-off,y1,size,t,0.0,-1)
          ENDIF
          IF (mlabel) THEN
            t = t + yscl
          ELSE
            t = t - yscl
          ENDIF
          pos = pos + tic
  310   CONTINUE
        CALL PLTBAX(tx,0.0,xlen-tx,0.0,tic,0.1,0)
        IF (.NOT.(enonly .OR. meonly)) THEN
          n = IFIX(xlen*1.6093-tdm) + 1
          tic = 1.000/1.6093
          ypos = ylen + 5.0*size + onezer*(size-0.05)
          IF (mlabel) THEN
            xpos = (xlen-nchrus(kt)*shift)*0.5
            CALL SYMBOL(xpos,ypos-(size+0.05)*one,size,uplocn(kt),angle,
     &                  nchrus(kt))
            xpos = (xlen-13.0*shift)*0.5
            CALL SYMBOL(xpos,ypos,size,'DISTANCE FROM',angle,13)
            CALL SYMBOL(xpos,ypos-2.0*(size+0.05)*one,size,
     &                  'IN KILOMETERS',angle,13)
          ELSE
            CALL SYMBOL((xlen-24.*shift)*.5,ypos,size,
     &                  'DISTANCE FROM MOUTH (KM)',angle,24)
          ENDIF
          pos = 0.0 + tdm
          t = tm
          DO 320 i = 1, n
            IF (t.LT.10.0) THEN
              off = half
            ELSE
              off = size
            ENDIF
            IF (t.GT.99.0 .OR. yscl.LT.0.9) off = size + half
            IF (movie) off = -half
            IF (yscl.GT.0.9) THEN
              CALL NUMBER(pos-off,6.1,size,t,anghf,-1)
            ELSE
              CALL NUMBER(pos-off,6.1,size,t,anghf,1)
            ENDIF
            IF (mlabel) THEN
              t = t + yscl
            ELSE
              t = t - yscl
            ENDIF
            pos = pos + tic
  320     CONTINUE
        ENDIF
        CALL PLTBAX(tdm,ylen,xlen-tdm,0.0,tic,-0.1,0)
        CALL PLOT(0.0,0.0,3)
        CALL PLOT(xlen,0.0,2)
        CALL PLOT(xlen,ylen,3)
        CALL PLOT(0.0,ylen,2)
        IF (slides) CALL NEWPEN(pen3)
C
C---LABEL PATHS-OF-TRAVEL WITH PARTICLE LETTER
        DO 350 j = 1, ktpart(kt)
          DO 330 i = 1, npart
            IF (letter(j).EQ.expart(i)) GOTO 350
  330     CONTINUE
          xpos = rm(1,j)*yscli
          IF (xpos.LT.outchk .AND. xpos.GT.0.001) THEN
            IF (.NOT.slides) CALL NEWPEN(pen1)
            IF (ABS(xpos).LE.sizeq) xpos = sizeq
            IF (.NOT.movie) CALL SYMBOL(xpos-half,.025,size,letter(j),
     &                                  0.0,1)
            IF (movie) CALL SYMBOL(xpos,0.05,size,letter(j),90.0,1)
            IF (.NOT.slides) CALL NEWPEN(pen2)
            CALL PLOT(xpos,y(1),3)
C
C---PLOT PATH-OF-TRAVEL OF PARTICLE LETTER(J)
            DO 340 i = 2, nsteps
              xpos = rm(i,j)*yscli
              CALL PLOT(xpos,y(i),2)
              IF (xpos.LT.0.001 .OR. xpos.GT.outchk) GOTO 350
  340       CONTINUE
          ENDIF
  350   CONTINUE
        IF (slides) CALL NEWPEN(pen2)
        IF (nlabel(kt).EQ.0) THEN
          CALL PLOT(0.0,0.0,3)
          CALL PLOT(0.0,ylen,2)
          CALL PLOT(xlen,ylen,3)
          CALL PLOT(xlen,0.0,2)
        ELSE
C
C---LABEL PLOT WITH RIVER LOCATION NAMES
          CALL NEWPEN(pen1)
          DO 370 i = 1, nlabel(kt)
            xpos = rmloc(i,kt)*yscli
            IF (xpos.LE.outchk) THEN
              xpos = xpos + szelbl(kt)*0.45
              ypos = 0.0
              ang = 90.0
              DO 360 j = 1, iarayn(kt)
                CALL SYMBOL(xpos,ypos,szelbl(kt),label(j,i,kt),ang,1)
                ypos = ypos + szelbl(kt)
  360         CONTINUE
            ENDIF
  370     CONTINUE
        ENDIF
C
C     ******** UPSTREAM BOUNDARY-VALUE DATA PLOTTING SECTION ********
C
        CALL PLOT(-1.0-szx3,0.0,-3)
        CALL NEWPEN(pen1)
        CALL PLTBAX(0.0,0.0,ylen,90.0,1.0,0.1,0)
        IF (.NOT.slides) CALL NEWPEN(pen2)
        CALL PLOT(1.0,0.0,3)
        CALL PLOT(1.0,ylen,2)
        CALL NEWPEN(pen1)
        IF (.NOT.movie) THEN
          DO 380 i = 1, 7
            pos = i - 1
            t = pos*idelt + tmstep
            IF (t.LT.10.0) THEN
              xpos = -size - 0.13
            ELSEIF (t.LT.100.0) THEN
              xpos = -2.0*size - 0.13
            ELSE
              xpos = -szx3 - 0.13
            ENDIF
            CALL NUMBER(xpos,pos-half*0.8,size,t,0.0,-1)
  380     CONTINUE
          CALL SYMBOL(xpos-size,3.0-7.0*size,size,'TIME, IN HOURS',90.0,
     &                14)
        ENDIF
        sgn = -1.0
        IF (meonly) THEN
          usmn = zqusmn*uscnv
          usmx = zqusmx*uscnv
        ELSE
          usmn = zqusmn
          usmx = zqusmx
        ENDIF
        lbl6 = '  LEFT'
        WRITE (printr,9065) itbcus, itbcus
        DO 410 i = 1, 2
          six0 = ylen*(i-1)
          prtlab = ((.NOT.enonly) .AND. (.NOT.meonly)) .OR. i.EQ.1
          CALL TRSCAL(printr,usmx,usmn,ystep,ylast)
          zqumax = usmx - AMOD(usmx,ystep) + ystep
  390     IF (zqumax.GE.usmx) THEN
            zqumax = zqumax - ylast
            GOTO 390
          ENDIF
          zqumin = usmn - AMOD(usmn,ystep) - ylast
  400     IF (zqumin.LE.usmn) THEN
            zqumin = zqumin + ylast
            GOTO 400
          ENDIF
          zqinch = 1.0/(usmx-usmn)
          WRITE (printr,9005) lbl6, usmx, usmn, zqinch, zqumin, zqumax
          t = AMAX1(ABS(usmx),ABS(usmn))
          IF (t.LT.10.0) THEN
            idigit = 0
          ELSE
            idigit = IFIX(ALOG10(t))
          ENDIF
          tenpwr = 1.00/10**idigit
          xpos = (usmx-zqumin)*zqinch
          CALL PLOT(xpos,0.07+six0,3)
          CALL PLOT(xpos,-0.07+six0,2)
          IF (prtlab) THEN
            xpos = xpos - half*one
            CALL LBLNUM(i,zqumin,y1,shftmv,onezer,size,xpos,tenpwr,
     &                  anghf)
          ENDIF
          xpos = (usmx-zqumax)*zqinch
          CALL PLOT(xpos,0.07+six0,3)
          CALL PLOT(xpos,-0.07+six0,2)
          IF (prtlab) THEN
            xpos = xpos - 2.5*size + shftmv
            CALL LBLNUM(i,zqumax,y1,shftmv,onezer,size,xpos,tenpwr,
     &                  anghf)
            CALL AXLABL(one,angle,meonly,itbcus,idigit,i-1,sgn,onezer,
     &                  uplocn(kt),nchrus(kt),size)
          ENDIF
          IF (i.NE.2) THEN
            lbl6 = ' RIGHT'
            sgn = 1.0
            IF (.NOT.(enonly .OR. meonly)) THEN
              usmn = usmn*uscnv
              usmx = usmx*uscnv
            ENDIF
          ENDIF
  410   CONTINUE
        xpos = (zqusmx-zq(1,1))*usinch
        IF (slides) THEN
          CALL NEWPEN(pen4)
        ELSE
          CALL NEWPEN(pen2)
        ENDIF
        CALL PLOT(xpos,y(1),3)
        DO 420 i = 2, nsteps
          xpos = (zqusmx-zq(i,1))*usinch
          CALL PLOT(xpos,y(i),2)
  420   CONTINUE
        CALL NEWPEN(pen1)
C
C---THIS NEXT SECTION PLOTS AXIS FRAMES FOR BOTH UPSTREAM AND
C---DOWNSTREAM BOUNDARY-VALUE DATA
        DO 430 i = 1, 2
          CALL PLOT(0.0,0.1,3)
          CALL PLOT(0.0,0.0,2)
          CALL PLOT(1.0,0.0,2)
          CALL PLOT(1.0,0.1,2)
          CALL PLOT(0.0,5.9,3)
          CALL PLOT(0.0,ylen,2)
          CALL PLOT(1.0,ylen,2)
          CALL PLOT(1.0,5.9,2)
          IF (i.NE.2) CALL PLOT(xlen+1.0+6.0*size,0.0,-3)
  430   CONTINUE
C
C     ******** DOWNSTREAM BOUNDARY-VALUE DATA PLOTTING SECTION ********
C
        CALL PLTBAX(1.0,0.0,ylen,90.0,1.0,-0.1,0)
        IF (.NOT.slides) CALL NEWPEN(pen2)
        CALL PLOT(0.0,0.0,3)
        CALL PLOT(0.0,ylen,2)
        CALL NEWPEN(pen1)
        IF (movie) THEN
          xpos = 1.1 + szx3 + half
          CALL SYMBOL(xpos,3.0-7.0*size,size,'TIME, IN HOURS',90.0,14)
          DO 440 i = 1, 7
            pos = i - 1
            t = pos*idelt + tmstep
            IF (t.LT.10.0) THEN
              off = half
            ELSEIF (t.LT.100.0) THEN
              off = size
            ELSE
              off = 1.5*size
            ENDIF
            CALL NUMBER(1.1+size,pos-off,size,t,90.0,-1)
  440     CONTINUE
        ENDIF
        sgn = -1.0
        IF (meonly) THEN
          dsmn = zqdsmn*dscnv
          dsmx = zqdsmx*dscnv
        ELSE
          dsmn = zqdsmn
          dsmx = zqdsmx
        ENDIF
        lbl6 = '  LEFT'
        WRITE (printr,9070) itbcds, itbcds
        DO 470 i = 1, 2
          six0 = ylen*(i-1)
          prtlab = ((.NOT.enonly) .AND. (.NOT.meonly)) .OR. i.EQ.1
          CALL TRSCAL(printr,dsmx,dsmn,ystep,ylast)
          zqdmax = dsmx - AMOD(dsmx,ystep) + ystep
  450     IF (zqdmax.GE.dsmx) THEN
            zqdmax = zqdmax - ylast
            GOTO 450
          ENDIF
          zqdmin = dsmn - AMOD(dsmn,ystep) - ylast
  460     IF (zqdmin.LE.dsmn) THEN
            zqdmin = zqdmin + ylast
            GOTO 460
          ENDIF
          zqinch = 1.0/(dsmx-dsmn)
          WRITE (printr,9005) lbl6, dsmx, dsmn, zqinch, zqdmin, zqdmax
          t = AMAX1(ABS(dsmx),ABS(dsmn))
          IF (t.LT.10.0) THEN
            idigit = 0
          ELSE
            idigit = IFIX(ALOG10(t))
          ENDIF
          tenpwr = 1.00/10**idigit
          xpos = (dsmx-zqdmin)*zqinch
          CALL PLOT(xpos,0.07+six0,3)
          CALL PLOT(xpos,-0.07+six0,2)
          IF (prtlab) THEN
            xpos = xpos - half*one
            CALL LBLNUM(i,zqdmin,y1,shftmv,onezer,size,xpos,tenpwr,
     &                  anghf)
          ENDIF
          xpos = (dsmx-zqdmax)*zqinch
          CALL PLOT(xpos,0.07+six0,3)
          CALL PLOT(xpos,-0.07+six0,2)
          IF (prtlab) THEN
            xpos = xpos - 2.5*size + shftmv
            CALL LBLNUM(i,zqdmax,y1,shftmv,onezer,size,xpos,tenpwr,
     &                  anghf)
            CALL AXLABL(one,angle,meonly,itbcds,idigit,i-1,sgn,onezer,
     &                  dnlocn(kt),nchrds(kt),size)
          ENDIF
          IF (i.NE.2) THEN
            lbl6 = ' RIGHT'
            sgn = 1.0
            IF (.NOT.(enonly .OR. meonly)) THEN
              dsmn = dsmn*dscnv
              dsmx = dsmx*dscnv
            ENDIF
          ENDIF
  470   CONTINUE
        IF (slides) THEN
          CALL NEWPEN(pen4)
        ELSE
          CALL NEWPEN(pen2)
        ENDIF
        xpos = (zqdsmx-zq(1,2))*dsinch
        CALL PLOT(xpos,y(1),3)
C
C---PLOT BOUNDARY-VALUE DATA
        DO 480 i = 2, nsteps
          xpos = (zqdsmx-zq(i,2))*dsinch
          CALL PLOT(xpos,y(i),2)
  480   CONTINUE
        CALL NEWPEN(pen1)
C
C     ******** BEGIN TO PLACE TITLE AND DATES AT BOTTOM OF PLOT ********
C
        IF (movie) THEN
          tx = 6.5 + 12.0*size
        ELSE
          tx = xlen + 10.0*size + 2.0
        ENDIF
        tsize = tx/55.0
        DO 490 i = 24, 1, -1
          nchr = i
          IF (netnam(i:i).NE.blk) GOTO 500
  490   CONTINUE
  500   xpos = 13.0*size + 2.0*tsize
        IF (movie) THEN
          mid = 3.0
          IF (meonly .OR. enonly) mid = mid - (8.5*size+0.25)*0.5
          ypos = mid - (29.0+nchr)*0.5*tsize
        ELSE
          mid = -xlen*0.5 - szx3
          xp = -xpos
          xpos = mid - (29.0+nchr)*0.5*tsize
          ypos = xp
        ENDIF
        symlbl = 'CONSTITUENT MOVEMENT THROUGH '//netnam
        CALL SYMBOL(xpos,ypos,tsize,symlbl,anghf,nchr+29)
        ypos = mid - 27.5*tsize
        IF (movie) THEN
          xpos = xpos + 2.0*tsize
        ELSE
          xpos = ypos
          ypos = xp - 2.0*tsize
        ENDIF
        WRITE (symlbl,9075) date
        CALL SYMBOL(xpos,ypos,tsize,symlbl,anghf,55)
        IF (movie) THEN
          xpos = xpos + 3.0*tsize
          yp = mid - 1.05
        ELSE
          xpos = mid - 1.05
          yp = xp - 5.0*tsize
        ENDIF
        WRITE (symlbl,9080) iyr, imo, idy
        CALL SYMBOL(xpos,yp,0.1,symlbl,anghf,23)
        WRITE (printr,9090)
        IF (multpt) THEN
          multpt = .FALSE.
          maxhrs = maxhrs - 180
          IF (maxhrs.GT.0 .AND. iend) WRITE (printr,9085) maxhrs
          tmstep = tmstep + 180
          IF (.NOT.iend) THEN
            CALL NXTPLT()
            CALL NEWPEN(pen1)
            CALL FACTOR(pltfac)
            IF (movie) THEN
              CALL PLOT(2.1,3.4,-3)
            ELSE
              CALL PLOT(2.4,3.4,-3)
            ENDIF
            GOTO 240
          ENDIF
        ENDIF
        IF (kt.NE.ktpath) THEN
          REWIND indata
          DO 510 j = 1, ktpath + 2
            READ (indata)
  510     CONTINUE
        ENDIF
        CALL NXTPLT()
        CALL NEWPEN(pen1)
        CALL FACTOR(pltfac)
  520 CONTINUE
      CALL PLOT(-7.0,10.0,999)
      GOTO 550
  530 WRITE (*,*) '**Error reading, probably using file in old format'
      GOTO 550
  540 WRITE (*,*) '**Error, input record out of order, expecting path', 
     &            kt
  550 STOP
 9005 FORMAT (A6,'-HAND LABELING',F11.3,F12.3,F11.6,F11.3,F12.3)
 9010 FORMAT (A4,2X,A4,F6.3,1X,A2,F5.3,10I2)
 9015 FORMAT ('1',2X,A,//,13X,'PATHS-OF-TRAVEL OF INJECTED PARTICLES',/,
     &        11X,'COMPUTED BY THE BRANCH-NETWORK FLOW MODEL',///,18X,
     &        'PROGRAM CONTROL PARAMETERS',/,1X,60('='),//,13X,
     &        'PLOT STYLE (MOVI,COMI)     =',4X,A4,/,13X,
     &        'PEN TYPE (PAPE,SLID)       =',4X,A4,/,13X,
     &        'PLOT LETTER SIZE (F5.3)    =',3X,F5.3,/,13X,
     &        'UNIT LABELING (ME,EN,BO)   =',6X,A2,/,13X,
     &        'SCALING FACTOR (F5.3)      =',3X,F5.3,/,13X,
     &        'EXCLUDED PATHS (0-',I2,')       =',2X,10I2,//,1X,60('='))
 9020 FORMAT (A16,A16,I2)
 9025 FORMAT (//,' VALUES FOR PATH',I3,/,
     &        ' UPSTREAM BOUNDARY CONDITION LOCATION NAME:   ',A16,/,
     &        ' DOWNSTREAM BOUNDARY CONDITION LOCATION NAME: ',A16,/,
     &        ' NUMBER OF LABEL LOCATIONS SPECIFIED:         ',I16)
 9030 FORMAT (A,' LOCATION FOR PATH',I2,' (16 CHARACTERS)        ')
 9035 FORMAT (/,14X,'DISTANCE       RIVER LOCATION NAME',5X,'PATH',I4)
 9040 FORMAT (F8.2,1X,A60)
 9045 FORMAT (15X,F6.2,8X,A60)
 9050 FORMAT (//,8X,'INPUT DATA PARAMETERS FROM BRANCH OUTPUT FILE',/,
     &        1X,60('='),//,8X,'NETWORK NAME         =',2X,A24,/,8X,
     &        'BEGINNING DATE       =',I4.2,2('/',I2.2),I3.2,':',I2.2,/,
     &        8X,'ENDING TIME          =',I4.2,2('/',I2.2),I3.2,':',
     &        I2.2,/,8X,'NUMBER OF PATHS      =',I10,/,8X,
     &        'NUMBER OF TIME STEPS =',I10,/,8X,
     &        'TIME INCREMENT (MIN) =',I10,/,8X,
     &        'BOUNDARY VALUES UNIT =',8X,A2,/,8X,
     &        'RIVER MILE UNIT      =',8X,A2)
 9055 FORMAT (17X,'PATH',I3.2,/,8X,'NUMBER OF PARTICLES  =',I10,/,8X,
     &        'CHANNEL LENGTH ',A4,'  =',F10.2)
 9060 FORMAT (//,1X,60('='),/,'1')
 9065 FORMAT (/,14X,'UPSTREAM BOUNDARY-VALUE DATA STATISTICS',//,22X,
     &        'MAXIMUM',A2,'   MINIMUM',A2,
     &        '   Y-SCALE   Y-LABEL 1   Y-LABEL 2')
 9070 FORMAT (/,14X,'DOWNSTREAM BOUNDARY-VALUE DATA STATISTICS',//,22X,
     &        'MAXIMUM',A2,'   MINIMUM',A2,
     &        '   Y-SCALE   Y-LABEL 1   Y-LABEL 2')
 9075 FORMAT ('FOR FLOW CONDITIONS OF',I3.2,2('/',I2.2),I3.2,':',I2.2,
     &        ' TO',I3.2,2('/',I2.2),I3.2,':',I2.2)
 9080 FORMAT ('DATE PLOTTED:',I4.2,2('/',I2.2))
 9085 FORMAT (' WARNING:  INPUT DATA FILE SHORT, EXPECTED ',I4,
     &        ' MORE HOURS OF DATA.')
 9090 FORMAT (10X,'******** ONE PLOT FRAME COMPLETED ********',//)
 9095 FORMAT (I2,1X,I1,F8.2,15A2)
 9100 FORMAT (A,I3)
      END
C
      SUBROUTINE TIOFIL(READER,PRINTR,INDATA,PLTUNT,VERSN,NEWFILE)
C
      CHARACTER card*80, msg*80, VERSN*72
      CHARACTER*48 pcfile, prname, brtnam, pltnam, blank
      LOGICAL okflag, yes, NEWFILE
      INTEGER mastr, len, lenp1, lenp4, i, READER, PRINTR, INDATA, 
     &        PLTUNT, retcod
      INTEGER retc
      CHARACTER*12 mstrnm
      INTEGER LENCHR
      EXTERNAL OPNRDR, IOPRMP, OPNRDU, OPNFIL, LENCHR, GETYES
C
      mstrnm = 'TRKPLOT.MTR '
      mastr = 75
      blank = '                                                '
      okflag = .FALSE.
      WRITE (*,9005) VERSN(:58)
C
C---CHECK STATUS OF TRACKPLOT MASTER FILE
      CALL OPNRDR(83,'TRKPLOT.LOG',1,retcod)
      IF (retcod.LT.0) STOP 
     &        '**Error** in iofile, check access rights or file in use.'
      CALL OPNRDR(mastr,mstrnm,1,retcod)
      IF (retcod.LT.0) GOTO 60
      IF (retcod.EQ.0) THEN
        WRITE (*,9010)
        DO 10 i = 1, 4
          READ (mastr,'(A80)',END=50) card
          WRITE (*,'(A80)') card
   10   CONTINUE
        REWIND mastr
        msg = 'ARE THESE THE REQUIRED FILES FOR THIS EXECUTION'
        CALL GETYES(msg,yes)
        IF (yes) okflag = .TRUE.
      ENDIF
      IF (.NOT.okflag) WRITE (*,9015)
C
C---IF MASTER FILE NAMES ARE CORRECT USE THEM, OTHERWISE PROMPT USER
   20 pcfile = blank
      brtnam = blank
      prname = blank
      pltnam = blank
      IF (retcod.EQ.0) THEN
        READ (mastr,9035) pcfile
        READ (mastr,9035) brtnam
        READ (mastr,9035) prname
        READ (mastr,9035) pltnam
      ENDIF
      IF (pcfile.EQ.blank) pcfile = 'TRACKPLOT.CTL'
      msg = 'ENTER PROGRAM-CONTROL FILE NAME.'
      IF (.NOT.okflag) CALL IOPRMP(msg,pcfile)
      CALL OPNRDR(READER,pcfile,1,retc)
C
      NEWFILE = .FALSE.
      IF (retc.LT.0) GOTO 60
      IF (retc.GT.0) NEWFILE = .TRUE.
C
      len = LENCHR(pcfile)
      lenp1 = 0
      DO 30 i = len, 1, -1
        IF (pcfile(i:i).EQ.'.') THEN
          lenp1 = i - 1
          GOTO 40
        ENDIF
   30 CONTINUE
   40 IF (len.GT.4 .AND. len-lenp1.LT.5) len = lenp1
      IF (len.GT.44) len = 44
      lenp1 = len + 1
      lenp4 = len + 4
C
C---BRANCH OUTPUT FILE DESIGNATION.
      IF (brtnam.EQ.blank) THEN
        brtnam(1:len) = pcfile(1:len)
        brtnam(lenp1:lenp4) = '.pto'
        okflag = .FALSE.
      ENDIF
      msg = 'ENTER BRANCH PARTICLE LOCATIONS FILE NAME (BRANCH OUTPUT)'
      CALL OPNRDU(msg,INDATA,brtnam,okflag)
C
C---ASSIGN THE PRINTED OUTPUT FILE NAME AND OPEN ON UNIT 66
      IF (prname.EQ.blank) THEN
        prname(1:len) = pcfile(1:len)
        prname(lenp1:lenp4) = '.prt'
        IF (okflag) WRITE (*,9020) prname
      ENDIF
      IF (.NOT.okflag) THEN
        msg = 'ENTER PRINTER OUTPUT FILE NAME.'
        CALL IOPRMP(msg,prname)
      ENDIF
      CALL OPNFIL(PRINTR,prname)
C
C---IF PLOTTER INSTRUCTIONS ARE TO BE OUTPUT, OPEN FILE
      IF (pltnam.EQ.blank) THEN
        pltnam(1:len) = pcfile(1:len)
        pltnam(lenp1:lenp4) = '.plt'
        IF (okflag) WRITE (*,9025) pltnam
      ENDIF
      IF (.NOT.okflag) THEN
        msg = 'ENTER DIGITAL-PLOTTER OUTPUT FILE NAME.'
        CALL IOPRMP(msg,pltnam)
      ENDIF
      CALL OPNFIL(PLTUNT,pltnam)
C
      REWIND mastr
      WRITE (mastr,9030) ' PROGRAM-CONTROL FILE    : ', pcfile
      WRITE (mastr,9030) ' BRANCH OUTPUT FILE      : ', brtnam
      WRITE (mastr,9030) ' PRINTER OUTPUT FILE     : ', prname
      WRITE (mastr,9030) ' DIGITAL-PLOTTING OUTPUT : ', pltnam
      ENDFILE mastr
      CLOSE (mastr)
      WRITE (*,*) 'File initialization completed!'
      RETURN
C
   50 WRITE (*,9040)
      REWIND mastr
      retcod = 1
      GOTO 20
C
   60 STOP 'error opening file, in tiofil'
C
 9005 FORMAT (//,' Welcome to the ',A,//,
     &' You may enter "q" for any prompt to discontinue initiation of th
     &e program.',/,
     &' If the displayed or default value following a prompt is acceptab
     &le, a',/,
     &' carriage return is sufficient response, otherwise a new value mu
     &st be',/,
     &' entered.  The default response to a "yes/no" query is yes.',/)
 9010 FORMAT (
     &' These are the current input/output file names in your trkplot ma
     &ster file:',/)
 9015 FORMAT (/
     &' Prompts are issued for the file names of all required files.  To
     & accept the',/,
     &' name as defined in the prompt, enter a carriage return.  The spe
     &cified file',/,
     &' names will be saved for use in subsequent exections.')
 9020 FORMAT (/
     &      '*WARNING* Printed output file unspecified, always required'
     &      ,/,'**** Default file name assigned: ',A48,/)
 9025 FORMAT (/
     &      '*WARNING* Plotter output file unspecified, always required'
     &      ,/,'**** Default file name assigned: ',A48,/)
 9030 FORMAT (A27,A48)
 9035 FORMAT (27X,A48)
 9040 FORMAT (/,
     &    ' ** End of file reading master file, will create new file **'
     &    ,/)
      END
C
      SUBROUTINE PGMCTL(READER,KTPATH)
C
      INCLUDE 'dimens.cmn'
      INTEGER READER, MAXLOC, KTPATH
      PARAMETER (MAXLOC=10)
      REAL rmloc(MAXLOC,MXPATH), size, pltfac, uprvml(MXPATH)
      CHARACTER*16 uplocn(MXPATH), dnlocn(MXPATH)
      CHARACTER*60 label(MAXLOC,MXPATH), msg
      INTEGER nvar, i, j, ios, ixpath(MXPATH), nlabel(MXPATH), 
     &        locprmpt(MXPATH), numpath(MXPATH), kt, nl, k
      CHARACTER*2 expart(MAXLOC,MXPATH), onelab, me, en, blnk, both
      CHARACTER*20 exprt(MXPATH), expath
      CHARACTER ans*1
      EQUIVALENCE (exprt(1),expart(1,1))
      CHARACTER*4 plttyp, pentyp, movi, slide, blank, paper, comic
      EXTERNAL GETCHR, GETCVL, GETRVL, GETIVL, LBSORT
      DATA slide/'SLID'/, movi/'MOVI'/, me/'ME'/, en/'EN'/, both/'BO'/
      DATA blnk/'  '/, blank/'    '/, paper/'PAPE'/, comic/'COMI'/
      DATA exprt/MXPATH*'                    '/
C
      READ (READER,9005,IOSTAT=ios) plttyp, pentyp, size, onelab, 
     &                              pltfac, ixpath
      IF (ios.EQ.0) THEN
        IF (plttyp.EQ.blank) plttyp = movi
        IF (pentyp.EQ.blank) pentyp = slide
        IF (onelab.EQ.blnk) onelab = both
        IF (size.EQ.0.0) size = 0.15
        IF (pltfac.EQ.0.0) pltfac = 0.85
      ELSE
        WRITE (*,*) 
     &            'Warning or EOF reading 1st record, defaults assigned'
        plttyp = movi
        pentyp = slide
        onelab = both
        size = 0.15
        pltfac = 0.85
        DO 10 i = 1, MXPATH
          ixpath(i) = 0
   10   CONTINUE
      ENDIF
C
      DO 40 kt = 1, KTPATH
        READ (READER,9010,IOSTAT=ios) numpath(kt), locprmpt(kt), 
     &                                uprvml(kt), expart(1,kt)
        IF (ios.NE.0) THEN
          WRITE (*,9070) '2nd', kt
          numpath(kt) = kt
          locprmpt(kt) = 1
          uprvml(kt) = 0.0
          DO 20 i = 1, MAXLOC
            expart(i,kt) = '  '
   20     CONTINUE
        ENDIF
        IF (numpath(kt).NE.kt) numpath(kt) = kt
        READ (READER,9015,IOSTAT=ios) uplocn(kt), dnlocn(kt), nlabel(kt)
        IF (ios.NE.0) THEN
          WRITE (*,9070) '3nd', kt
          uplocn(kt) = 'upstream bndry'
          dnlocn(kt) = 'downstream bndry'
          nlabel(kt) = 0
        ENDIF
        DO 30 nl = 1, nlabel(kt)
          READ (READER,9050,IOSTAT=ios) rmloc(nl,kt), label(nl,kt)
          IF (ios.NE.0) THEN
            WRITE (*,9070) '4th', kt
            uplocn(kt) = 'upstream bndry'
            dnlocn(kt) = 'downstream bndry'
            nlabel(kt) = 0
          ENDIF
   30   CONTINUE
   40 CONTINUE
      ans = '?'
      GOTO 60
   50 msg = 'ENTER A VARIABLE NUMBER(#), ?, OR F'
      CALL GETCHR(msg,ans)
   60 IF (ans.EQ.'?' .OR. ans.EQ.'T' .OR. ans.EQ.'t') THEN
        WRITE (*,9020) plttyp, pentyp, size, onelab, pltfac, ixpath
        WRITE (*,9030)
        GOTO 50
      ELSEIF (ans.NE.' ' .AND. ans.NE.'F' .AND. ans.NE.'f') THEN
        READ (ans,'(I1)',IOSTAT=ios) nvar
        IF (ios.GT.0 .OR. nvar.LT.1 .OR. nvar.GT.6) THEN
          WRITE (*,*) '** Invalid response, please try again'
        ELSEIF (nvar.EQ.1) THEN
          WRITE (msg,9045) nvar, 'PLTTYP'
          CALL GETCVL(msg,plttyp)
          IF (plttyp.EQ.'comi') plttyp = comic
          IF (plttyp.NE.comic) plttyp = movi
        ELSEIF (nvar.EQ.2) THEN
          WRITE (msg,9045) nvar, 'PENTYP'
          CALL GETCVL(msg,pentyp)
          IF (pentyp.EQ.'pape') pentyp = paper
          IF (pentyp.NE.paper) pentyp = slide
        ELSEIF (nvar.EQ.3) THEN
          WRITE (msg,9045) nvar, 'SIZE'
          CALL GETRVL(msg,3,0.0,1.0,size)
        ELSEIF (nvar.EQ.4) THEN
          WRITE (msg,9045) nvar, 'ONELAB'
          CALL GETCVL(msg,onelab)
          IF (onelab.EQ.'en') onelab = en
          IF (onelab.EQ.'me') onelab = me
          IF (onelab.NE.en .OR. onelab.NE.me) onelab = both
        ELSEIF (nvar.EQ.5) THEN
          WRITE (msg,9045) nvar, 'PLTFAC'
          CALL GETRVL(msg,3,0.0,9.0,pltfac)
        ELSEIF (nvar.EQ.6) THEN
          WRITE (*,9075) KTPATH
          WRITE (msg,9045) nvar, 'IXPATH'
          CALL GETCVL(msg,expath)
          DO 70 i = 1, MXPATH
            ixpath(i) = 0
   70     CONTINUE
          k = 1
          i = 1
   80     IF (expath(i:i).NE.' ') THEN
            j = i + 1
            IF (expath(j:j).EQ.' ' .OR. expath(j:j).EQ.',') THEN
              READ (expath(i:i),'(I1)') ixpath(k)
            ELSE
              READ (expath(i:j),'(I2)') ixpath(k)
              i = i + 1
            ENDIF
            k = k + 1
          ENDIF
          i = i + 1
          IF (i.LE.MXPATH*2) GOTO 80
        ENDIF
        GOTO 50
      ENDIF
      kt = 1
      ans = '?'
      GOTO 100
   90 msg = 'ENTER A VARIABLE NUMBER(#), L, ?, OR F'
      CALL GETCHR(msg,ans)
  100 IF (ans.EQ.'?' .OR. ans.EQ.'T' .OR. ans.EQ.'t') THEN
        WRITE (*,9025) numpath(kt), locprmpt(kt), uprvml(kt), 
     &                 (expart(i,kt),i=1,10), uplocn(kt), dnlocn(kt)
        WRITE (*,9035)
        GOTO 90
      ELSEIF (ans.EQ.'L' .OR. ans.EQ.'l') THEN
        IF (nlabel(kt).GT.MAXLOC) THEN
          WRITE (*,9065) MAXLOC
          nlabel(kt) = MAXLOC
        ENDIF
        ans = '?'
  110   IF (ans.EQ.'?' .OR. ans.EQ.'T' .OR. ans.EQ.'t') THEN
          WRITE (*,9040)
          IF (nlabel(kt).EQ.0) THEN
            WRITE (*,9090)
          ELSE
            DO 120 i = 1, nlabel(kt)
              WRITE (*,9055) i, rmloc(i,kt), label(i,kt)
  120       CONTINUE
          ENDIF
          WRITE (*,9060)
          IF (nlabel(kt).EQ.0) THEN
            ans = 'A'
            GOTO 110
          ENDIF
        ELSEIF (ans.EQ.' ' .OR. ans.EQ.'F' .OR. ans.EQ.'f') THEN
          ans = '?'
          IF (nlabel(kt).GT.0) locprmpt(kt) = 1
          GOTO 100
C
C---ADD A LABEL IF COMMAND "A" OR "a" IS FOUND
        ELSEIF (ans.EQ.'A' .OR. ans.EQ.'a') THEN
          j = nlabel(kt) + 1
          IF (j.GT.MAXLOC) THEN
            WRITE (*,9065) MAXLOC
          ELSE
            msg = 'THE RIVER MILE DISTANCE'
            CALL GETRVL(msg,2,0.0,9999.0,rmloc(j,kt))
            msg = 'THE RIVER LOCATION NAME'
            label(j,kt) = 'Bridge 1'
            CALL GETCVL(msg,label(j,kt))
            nlabel(kt) = j
          ENDIF
C
C---DELETE A LABEL IF COMMAND "D" OR "d" IS FOUND
        ELSEIF (ans.EQ.'D' .OR. ans.EQ.'d') THEN
          msg = 'THE NUMBER OF THE LABEL TO BE DELETED'
          CALL GETIVL(msg,1,nlabel(kt),i)
          nlabel(kt) = nlabel(kt) - 1
          DO 130 j = i, nlabel(kt)
            rmloc(j,kt) = rmloc(j+1,kt)
            label(j,kt) = label(j+1,kt)
  130     CONTINUE
C
C---Modify the specified river mile location
        ELSE
          READ (ans,'(I1)',IOSTAT=ios) nvar
          IF (ios.LE.0 .OR. (nvar.GT.0 .AND. nvar.LE.nlabel(kt))) THEN
            WRITE (msg,9045) nvar, 'RMLOC'
            CALL GETRVL(msg,2,0.0,9999.0,rmloc(nvar,kt))
            WRITE (msg,9045) nvar, 'LABEL'
            CALL GETCVL(msg,label(nvar,kt))
          ENDIF
        ENDIF
        IF (nlabel(kt).GT.1) CALL LBSORT(nlabel(kt),rmloc(1,kt),
     &      label(1,kt))
        msg = 'ENTER A LABEL NUMBER(#), A, D, ?, OR F'
        CALL GETCHR(msg,ans)
        GOTO 110
      ELSEIF (ans.NE.' ' .AND. ans.NE.'F' .AND. ans.NE.'f') THEN
        READ (ans,'(I1)',IOSTAT=ios) nvar
        IF (ios.GT.0 .OR. nvar.LT.1 .OR. nvar.GT.5) THEN
          WRITE (*,*) '** Invalid response, please try again'
        ELSEIF (nvar.EQ.1) THEN
          WRITE (*,9085)
          WRITE (msg,9045) nvar, 'LOCPRMPT'
          CALL GETIVL(msg,0,1,locprmpt(kt))
          IF (locprmpt(kt).EQ.0) THEN
            uplocn(kt) = '  '
            dnlocn(kt) = '  '
          ENDIF
        ELSEIF (nvar.EQ.2) THEN
          WRITE (msg,9045) nvar, 'UPRVML'
          CALL GETRVL(msg,2,0.0,9999.0,uprvml(kt))
        ELSEIF (nvar.EQ.3) THEN
          WRITE (*,*) 'To include all particles, enter Z to prompt'
          WRITE (msg,9045) nvar, 
     &                     'EXPART (use upper-case letters A-J only)'
          CALL GETCVL(msg,exprt(kt))
          IF (exprt(kt)(1:1).EQ.'Z' .OR. exprt(kt)(1:1).EQ.'z')
     &        exprt(kt) = '    '
        ELSEIF (nvar.EQ.4) THEN
          WRITE (*,9080)
          WRITE (msg,9045) nvar, 'UPLOCN'
          CALL GETCVL(msg,uplocn(kt))
        ELSEIF (nvar.EQ.5) THEN
          WRITE (*,9080)
          WRITE (msg,9045) nvar, 'DNLOCN'
          CALL GETCVL(msg,dnlocn(kt))
        ENDIF
        GOTO 90
      ENDIF
      IF ((uplocn(kt).NE.'upstream bndry  ' .AND. uplocn(kt).NE.'  ')
     &    .OR. (dnlocn(kt).NE.'downstream bndry' .AND. dnlocn(kt)
     &    .NE.'  ')) locprmpt(kt) = 1
      kt = kt + 1
      IF (kt.LE.KTPATH) THEN
        ans = '?'
        GOTO 100
      ENDIF
C
      REWIND READER
      WRITE (READER,9005) plttyp, pentyp, size, onelab, pltfac, ixpath
      DO 150 kt = 1, KTPATH
        WRITE (READER,9010) numpath(kt), locprmpt(kt), uprvml(kt), 
     &                      expart(1,kt)
        IF (locprmpt(kt).EQ.1) THEN
          WRITE (READER,9015) uplocn(kt), dnlocn(kt), nlabel(kt)
          DO 140 i = 1, nlabel(kt)
            WRITE (READER,9050) rmloc(i,kt), label(i,kt)
  140     CONTINUE
        ENDIF
  150 CONTINUE
      ENDFILE READER
      REWIND READER
      RETURN
C
 9005 FORMAT (A4,2X,A4,F6.3,1X,A2,F5.3,10I2)
 9010 FORMAT (I2,1X,I1,F8.2,10A2)
 9015 FORMAT (A16,A16,I2)
 9020 FORMAT (/,22X,'Plotting options',/,1X,60('='),/,' Num',10X,
     &        'Description',14X,'Value',/,2X,
     &        '1  -  Plot style (MOVI,COMI)     =',4X,A4,/,2X,
     &        '2  -  Pen type (PAPE,SLID)       =',4X,A4,/,2X,
     &        '3  -  Plot letter size (F6.3)    =',F8.3,/,2X,
     &        '4  -  Unit labeling (ME,EN,BO)   =',6X,A2,/,2X,
     &        '5  -  Scaling factor (F5.3)      =',F8.3,/,2X,
     &        '6  -  Excluded paths (0-MXPATH)  =',2X,10I2)
 9025 FORMAT (/,2X,
     &        'Options and boundary condition location records for path'
     &        ,I2,/,1X,60('='),/,' Num',10X,'Description',14X,'Value',/,
     &        2X,'1  -  Prompt option (0/1)         =',I8,/,2X,
     &        '2  -  Upstream river mile (F5.2)  =',F8.2,/,2X,
     &        '3  -  Excluded particles (A-J)    =',2X,10A2,/,2X,
     &        '4  -  Upstream B.C. name (A16)    =',2X,A16,/,2X,
     &        '5  -  Downstream B.C. name (A16)  =',2X,A16)
 9030 FORMAT (/,' Code',T33,'Function',/,3X,
     &   '#  -  To change the value of a variable, enter its NUMBER (as'
     &   ,/,13X,'shown in the table) and the new VALUE when requested',
     &   /,3X,'?  -  To redisplay this message',/,3X,
     &   'F  -  When FINISHED with changes to this record (default)')
 9035 FORMAT (/,5X,'Code',T33,'Function',/,3X,
     &   '#  -  To change the value of a variable, enter its NUMBER (as'
     &   ,/,13X,'shown in the table) and the new VALUE when requested',
     &   /,3X,'L  -  Update label locations and names',/,3X,
     &   '?  -  To redisplay this message',/,3X,
     &   'F  -  When FINISHED with changes to this path (default)')
 9040 FORMAT (/,20X,'Label location records',/,1X,60('='),/,
     &        ' Num   Distance',6X,'River location name')
 9045 FORMAT ('VARIABLE',I2,1X,A)
 9050 FORMAT (F8.2,1X,A60)
 9055 FORMAT (I3,'  -',F8.2,4X,A60)
 9060 FORMAT (/,5X,'Code',T35,'Function',/,3X,
     &      '#  -  To change the values of an existing label, enter its'
     &      ,/,13X,
     &      'NUMBER (as shown above) and the new VALUES when requested.'
     &      ,/,3X,'A  -  To add a label',/,3X,'D  -  To delete a label',
     &      /,3X,'?  -  To redisplay this message',/,3X,
     &      'F  -  When FINISHED with changes to these labels (default)'
     &      )
 9065 FORMAT (/,'*ERROR* Only',I3,' label locations can be input',/)
 9070 FORMAT (' Warning or EOF reading ',A,' record type, path',I2,
     &        ', defaults assigned')
 9075 FORMAT (/,
     &' Enter the path numbers that you want excluded from the plot as a
     & list of',/,' numbers separated by a comma or space (range=1-',I1,
     &')')
 9080 FORMAT (/,
     &' The upstream and downstrem boundary location names are used to l
     &abel the axis',/,
     &' of the produced plot, they default to the names set in the BRANC
     &H output file.')
 9085 FORMAT (/,
     &' Set LOCPRMPT=1 to have TRKPLOT read boundary-condition location 
     &names and label',/,
     &' locations from program-control file.  SET LOCPRMPT=0 to have TRK
     &PLOT prompt for',/,
     &' them during execution (values not saved in program-control file 
     &when LOCPRMPT=0')
 9090 FORMAT (22X,'**No labels found**')
      END
C
      SUBROUTINE LBSORT(NLABEL,RMLOC,LABEL)
      INTEGER i, j, NLABEL, k
      REAL swapr, RMLOC(*)
      CHARACTER*60 swapl, LABEL(*)
C
      DO 20 k = 1, NLABEL
        DO 10 i = 1, NLABEL - 1
          j = i + 1
          IF (RMLOC(i).GT.RMLOC(j)) THEN
            swapr = RMLOC(i)
            RMLOC(i) = RMLOC(j)
            RMLOC(j) = swapr
            swapl = LABEL(i)
            LABEL(i) = LABEL(j)
            LABEL(j) = swapl
          ENDIF
   10   CONTINUE
   20 CONTINUE
      RETURN
      END
C
      SUBROUTINE TRSCAL(PRINTR,DMAX,DMIN,YSTEP,YLAST)
C
      INTEGER PRINTR, ipower
      REAL DMAX, DMIN, YSTEP, YLAST, ytest, range
      REAL ALOG10
      INTRINSIC ALOG10
C
      range = DMAX - DMIN
      IF (range.LT.10.0) THEN
        ipower = -1
      ELSE
        ipower = IFIX(ALOG10(range)-1.0)
      ENDIF
      YSTEP = 10.0**ipower
      YLAST = (10.0**(ipower-1))*0.5
      ytest = YSTEP
C
   10 ytest = 2.0*ytest
      IF (ytest.LT.range) THEN
        YLAST = YSTEP
        YSTEP = ytest
        ytest = 2.5*ytest
        IF (ytest.LT.range) THEN
          YLAST = YSTEP
          YSTEP = ytest
          ipower = ipower + 1
          IF (ipower.LT.6) THEN
            ytest = 10.0**ipower
            IF (ytest.GE.range) GOTO 20
            YLAST = YSTEP
            YSTEP = ytest
            GOTO 10
          ENDIF
          WRITE (PRINTR,9005)
        ENDIF
      ENDIF
   20 RETURN
 9005 FORMAT (/,
     &' *ERROR* Step size in plot scale algorithm exceeds maximum limit'
     &)
      END
C
      SUBROUTINE LBLNUM(ISDE,ZQVAL,Y1,SHFTMV,ONEZER,SIZE,XPOS,TENPWR,
     &                  ANGHF)
C
      INTEGER idigt, ISDE
      REAL ypos, ZQVAL, Y1, SHFTMV, ONEZER, SIZE, XPOS, TENPWR, ANGHF
      REAL ABS
      INTRINSIC ABS
      EXTERNAL NUMBER
C
      idigt = 1
      IF (ISDE.EQ.2) THEN
        ypos = 6.13
        IF (ABS(ZQVAL).GT.0.0005 .AND. ABS(ZQVAL).LT.0.1) idigt = 2
      ELSEIF (ZQVAL.LT.0.0) THEN
        ypos = Y1 - SHFTMV - ONEZER*SIZE - 0.02
      ELSE
        ypos = Y1 - SHFTMV - 0.02
      ENDIF
      CALL NUMBER(XPOS,ypos,SIZE,ZQVAL*TENPWR,ANGHF,idigt)
C
      RETURN
      END
C
      SUBROUTINE AXLABL(ONE,ANGLE,MEONLY,TYPE,IDIGIT,ISDE,SGN,ONZ,LOC,
     &                  NCHR,SIZE)
C
      REAL ONE, ANGLE
      LOGICAL MEONLY
      INTEGER IDIGIT, ISDE, NCHR
      REAL SGN, ONZ, SIZE
      CHARACTER*2 qtype, TYPE
      CHARACTER*16 LOC
      REAL oneme, half, szshft, ypos1, ypos2, ypos3, xpos
      REAL FLOAT
      INTRINSIC FLOAT
      EXTERNAL SYMBOL, NUMBER
      SAVE qtype
      DATA qtype/' Q'/
C
      oneme = 0.0
      IF (MEONLY) oneme = 1.0
      half = SIZE*0.5
      szshft = SIZE*ONE
      ypos1 = SGN*(SIZE*3.5+.1+ONZ*(SIZE*3.+0.1))
     &        + ISDE*(6.1+szshft-ONZ*.2)
      ypos2 = SGN*(SIZE*4.5+0.15+ONZ*SIZE) + ISDE*(6.0-szshft)
      ypos3 = SGN*(SIZE*5.5+0.2-ONZ*(SIZE+0.1))
     &        + ISDE*(5.9-SIZE*3.0*ONE+ONZ*0.2)
      IF (TYPE.EQ.qtype) THEN
        xpos = 0.5 - szshft*6.0
        CALL SYMBOL(xpos,ypos1,SIZE,'DISCHARGE AT',ANGLE,12)
        CALL SYMBOL(0.5-NCHR*half*ONE,ypos2,SIZE,LOC,ANGLE,NCHR)
        IF (ISDE.NE.0) THEN
          xpos = 0.5 - szshft*5.5
          CALL SYMBOL(xpos,ypos3,SIZE,'IN 10  M /S',ANGLE,11)
        ELSEIF (MEONLY) THEN
          xpos = 0.5 - szshft*5.5
          CALL SYMBOL(xpos,ypos3,SIZE,'IN 10  M /S',ANGLE,11)
        ELSE
          xpos = 0.5 - szshft*6.0
          CALL SYMBOL(xpos,ypos3,SIZE,'IN 10  FT /S',ANGLE,12)
        ENDIF
        CALL NUMBER(xpos+szshft*5.0,ypos3+ONE*half,half,FLOAT(IDIGIT),
     &              ANGLE,-1)
        CALL SYMBOL(xpos+(9.0-ISDE-oneme)*szshft,ypos3+ONE*half,half,
     &              '3',ANGLE,1)
      ELSE
        CALL SYMBOL(.5-szshft*4.0,ypos1,SIZE,'STAGE AT',ANGLE,8)
        CALL SYMBOL(0.5-NCHR*half*ONE,ypos2,SIZE,LOC,ANGLE,NCHR)
        IF (ISDE.NE.0 .OR. MEONLY) THEN
          CALL SYMBOL(0.5-szshft*9.5,ypos3,SIZE,'IN METERS ABOVE MSL',
     &                ANGLE,19)
        ELSE
          CALL SYMBOL(0.5-szshft*8.5,ypos3,SIZE,'IN FEET ABOVE MSL',
     &                ANGLE,17)
        ENDIF
      ENDIF
      RETURN
      END
C
      SUBROUTINE CHKLOC(LOC,NCHR)
C
C     Left justify a label and add a trailing comma if the
C     label is less than 16 characters or already has one.
C
      CHARACTER*(*) LOC
      INTEGER NCHR, numchr, MIN0, i
      INTRINSIC MIN0
      EXTERNAL LFTJUS
C
      numchr = NCHR
      CALL LFTJUS(LOC,numchr)
      numchr = MIN0(numchr,16)
      IF (numchr.EQ.16 .AND. LOC(16:16).NE.',') THEN
        DO 10 i = 15, 1, -1
          IF (LOC(i:i).NE.' ' .AND. LOC(i:i).NE.'-') THEN
            numchr = i
            GOTO 20
          ENDIF
          LOC(i+1:i+1) = ' '
   10   CONTINUE
      ENDIF
   20 IF (numchr.LT.16 .AND. LOC(numchr:numchr).NE.',') THEN
        numchr = numchr + 1
        LOC(numchr:numchr) = ','
      ENDIF
      NCHR = numchr
      RETURN
      END
C
      SUBROUTINE LFTJUS(STR,NCHR)
C
C     Left justify a string and return the number of characters
C     excluding trailing blanks
C
      CHARACTER*(*) STR
      INTEGER NCHR, i, numchr, k
      CHARACTER ctmp*80
C
      numchr = NCHR
      DO 20 i = 1, numchr
        IF (STR(i:i).NE.' ') THEN
          ctmp = STR(i:numchr)
          STR = ' '
          STR = ctmp(:numchr)
          DO 10 k = numchr, 1, -1
            IF (STR(k:k).NE.' ') THEN
              NCHR = k
              GOTO 30
            ENDIF
   10     CONTINUE
        ENDIF
   20 CONTINUE
      NCHR = 0
   30 RETURN
      END
