#!/bin/sh
# usr/opt/wrdapp/one-d/test/test.sh -- run one-d test data sets
#
# Usage: test.sh [start [stop]]
#        test.sh [start [stop] [cstart] [cstop]] | tee test.out
#
#        where: start = starting test number for BRANCH
#                stop = ending test number (may be same as start)
#               cstart = starting test number for CGAP
#                cstop = ending test number (may be same as start)
#
# To use this script for another program in a different directory, globally
# change the program name (be sure to change both upper and lower case
# occurrences) and set the default value for the Stop variable as appropriate.
#
# History: 12/24/91, mblalock, initial coding
#          04/27/92, mygoze, restructuring
#          06/17/92, rsregan, modified for use with one-d
#


#
# Variable definitions
# --------------------
#
  TOPDIR=..
  BIN=$TOPDIR/bin
  DATA=$TOPDIR/data
  CHECK=$TOPDIR/test/check.sh
  CLEAN=$TOPDIR/test/clean.sh
  BRANCH=$BIN/branch
  BRNWDM=$BIN/branchwdm
  TRK=$BIN/trkplot
  END=6
  DIVD=========================================

#
#
  exec 2>&1                                # stderr shows up in .out file

  Start=${1:-1}                            # by default, start at 1
  Stop=${2:-$END}                          # by default, stop at 6
  if [ $Start -lt 1 ] ; then Start=1 ; fi
  if [ $Stop -lt 1 ] ; then Stop=$END ; fi
  if [ $Start -gt $END -o $Stop -gt $END ] ; then
    echo
    echo "Warning, invalid arguments--test range is 1 - $END for BRANCH"
    echo "input arguments were $Start - $Stop"
    echo
    if [ $Stop -gt $END ] ; then Stop=$END ; fi
    if [ $Start -gt $END ] ; then
      echo "no BRANCH tests will be performed"
    else
      echo "Tests $Start - $Stop will be performed"
    fi
    echo
  fi

#
# begin test runs
#
  $CLEAN
  Test=$Start

  if [ $Test -ge $Start -a $Test -le $Stop ] ; then
    echo
    echo $DIVD$DIVD
    echo "Begin processing one-d BRANCH test runs $Start to $Stop"
    echo
    date
  fi

  while [ $Test -ge $Start -a $Test -le $Stop ]
  do
    echo
    echo
    echo $DIVD$DIVD
    echo "Test run number $Test"
    PROG=$BRANCH
    GO=$DATA/go
    if [ $Test -eq 1 ] ; then
      echo "Sacramento-Freeport model"
      echo $DATA/branch1.ctl >> go
      echo n >> go
      echo branch1.prt >> go
      echo " " >> go
      GO=go
    fi
    if [ $Test -eq 2 ] ; then  echo "Sacramento TWRI model" ; fi
    if [ $Test -eq 3 ] ; then  echo "Sacramento-Freeport model" ; fi
    if [ $Test -eq 4 ] ; then
      echo "Potomac model plotting computed discharge"
      echo "branch4.dpo contains graphics in postscript format"
      cp $DATA/branch4.cfg DIGSPLT1.CFG
      chmod 644 DIGSPLT1.CFG
    fi
    if [ $Test -eq 5 ] ; then
      echo "Potomac model with particle-tracking, also runs trkplot program"
    fi
    if [ $Test -eq 6 ] ; then
      echo "Potomac model with particle-tracking using multiple paths,"
      echo "also runs trkplot program"
    fi
    echo

    Name=branch$Test
    rm -f BRANCH.MTR
    if [ $Test -ne 1 ] ; then 
	cp $DATA/$Name.MTR BRANCH.MTR 
	chmod 644 BRANCH.MTR
    fi
    $PROG < $GO > $Name.log
    if [ $Test -eq 5 -o $Test -eq 6 ] ; then
      echo "trkplot$Test.plt contains graphics in postscript format"
      cp $DATA/trkplot$Test.cfg DIGSPLT1.CFG
      chmod 644 DIGSPLT1.CFG
      rm -f TRKPLOT.MTR
      cp $DATA/TRKPLOT$Test.MTR TRKPLOT.MTR
      chmod 644 TRKPLOT.MTR
      $TRK < $GO > trkplot$Test.log
    fi
    Test=`expr $Test + 1`
  done
#
  if [ $Test -ge $Start -a $Test -le $Stop ] ; then
    echo
    echo
    echo $DIVD$DIVD
    echo "Completed one-d BRANCH test runs $Start to $Stop"
    echo
  fi
# check output against original output in $DATA directory
  $CHECK branch trkplot
